/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require([
    "app/common",
    "app/quotas",
    "app/messages",
    "aps/ResourceStore",
    "aps/WizardData",
    "aps/load",
    "aps/xhr",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "aps/Memory",
    "dojo/when",
    "dojo/promise/all"
], function (
    common,
    quotas,
    messages,
    Store,
    WizardData,
    load,
    xhr,
    registry,
    at,
    getStateful,
    getPlainValue,
    Memory,
    when,
    all
) {
    var tenant = aps.context.vars.tenant,
        tenantId = tenant.aps.id,
        userData = WizardData.get(),
        user = aps.context.params.user,
        login = user.login.toLowerCase(),
        setMaxQuota = !userData || !userData[login],
        subscriptionStore = new Store({
            target: "/aps/2/resources/" + tenantId + "/subscription",
            idProperty: "aps.id"
        });

    function renderQuota(quotasStatistic, setMaxQuota, userQuota) {
        var quotaContainer = registry.byId("quotaContainer"),
            controls = quotas.getQuotaControlsForNewUser(tenant, quotasStatistic, setMaxQuota, userQuota);

        quotaContainer.addChild(controls.storage);
        if (controls.localStorage) {
            quotaContainer.addChild(controls.localStorage);
        }
        if (controls.workstations) {
            quotaContainer.addChild(controls.workstations);
        }
        if (controls.servers) {
            quotaContainer.addChild(controls.servers);
        }
        if (controls.vms) {
            quotaContainer.addChild(controls.vms);
        }
        if (controls.mobiles) {
            quotaContainer.addChild(controls.mobiles);
        }
        if (controls.webHostingServers) {
            quotaContainer.addChild(controls.webHostingServers);
        }
        if (controls.websites) {
            quotaContainer.addChild(controls.websites);
        }
    }

    function getModel() {
        function getLimitValue(limitFieldValue) {
            return limitFieldValue === "" ? common.QUOTA_UNLIMITED_VALUE : limitFieldValue;
        }

        function getStorageLimitValue(limitFieldValue) {
            return limitFieldValue === "" ? common.QUOTA_UNLIMITED_VALUE : limitFieldValue * common.BYTES_IN_GB;
        }

        var notificationsManagement = registry.byId("notificationsManagement").checked ?
                common.NOTIFICATION_MANAGEMENT.ENABLE : common.NOTIFICATION_MANAGEMENT.DISABLE,
            privileged = registry.byId("adminPrivileges").checked ?
                common.USER_PRIVILEGED.YES : common.USER_PRIVILEGED.NO,
            notifyBackup = Number(registry.byId("notifyBackup").value),
            userStorageUsage = registry.byId("userStorageUsage"),
            userServerCount = registry.byId("userServerCount"),
            userVmCount = registry.byId("userVmCount"),
            userWorkstationCount = registry.byId("userWorkstationCount"),
            userMobileDeviceCount = registry.byId("userMobileDeviceCount"),
            userWebHostingServerCount = registry.byId("userWebHostingServerCount"),
            userWebsiteCount = registry.byId("userWebsiteCount");

        return {
            aps: {type: common.TYPES.USERS},
            tenant: [
                {aps: {id: tenantId}}
            ],
            userStorageUsage: {limit: userStorageUsage ? getStorageLimitValue(userStorageUsage.value) : 0},
            userServerCount: {limit: userServerCount ? getLimitValue(userServerCount.value) : 0},
            UserVmCount: {limit: userVmCount ? getLimitValue(userVmCount.value) : 0},
            UserWorkstationCount: {limit: userWorkstationCount ? getLimitValue(userWorkstationCount.value) : 0},
            UserMobileDeviceCount: {limit: userMobileDeviceCount ? getLimitValue(userMobileDeviceCount.value) : 0},
            userWebHostingServerCount: {limit: userWebHostingServerCount ? getLimitValue(userWebHostingServerCount.value) : 0},
            UserWebsiteCount: {limit: userWebsiteCount ? getLimitValue(userWebsiteCount.value) : 0},
            notifyBackup: notifyBackup,
            notificationsManagement: notificationsManagement,
            privileged: privileged,
            activateDuringCreation: true
        };
    }

    function setBackupNotificationChecked(value) {
        registry.byId("notifyBackup").set({value: value});
    }

    function setAdminPrivilegesChecked(value) {
        registry.byId("adminPrivileges").set("checked", !!value);
    }

    function setManagementNotificationChecked(value) {
        registry.byId("notificationsManagement").set("checked", !!value);
    }

    function restoreUserData(userData) {
        var userStorageUsage = registry.byId("userStorageUsage"),
            userServerCount = registry.byId("userServerCount"),
            userVmCount = registry.byId("userVmCount"),
            userWorkstationCount = registry.byId("userWorkstationCount"),
            userMobileDeviceCount = registry.byId("userMobileDeviceCount"),
            userWebHostingServerCount = registry.byId("userWebHostingServerCount"),
            userWebsiteCount = registry.byId("userWebsiteCount");

        setBackupNotificationChecked(userData.notifyBackup);
        setAdminPrivilegesChecked(userData.privileged);
        setManagementNotificationChecked(userData.notificationsManagement);

        if (userStorageUsage) {
            userStorageUsage.set("value", userData.userStorageUsage.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.userStorageUsage.limit / common.BYTES_IN_GB);
        }
        if (userWorkstationCount) {
            userWorkstationCount.set("value", userData.UserWorkstationCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserWorkstationCount.limit);
        }
        if (userServerCount) {
            userServerCount.set("value", userData.userServerCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.userServerCount.limit);
        }
        if (userVmCount) {
            userVmCount.set("value", userData.UserVmCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserVmCount.limit);
        }
        if (userMobileDeviceCount) {
            userMobileDeviceCount.set("value", userData.UserMobileDeviceCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserMobileDeviceCount.limit);
        }
        if (userWebHostingServerCount) {
            userWebHostingServerCount.set("value", userData.userWebHostingServerCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.userWebHostingServerCount.limit);
        }
        if (userWebsiteCount) {
            userWebsiteCount.set("value", userData.UserWebsiteCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserWebsiteCount.limit);
        }
    }

    all({
        subscriptions: subscriptionStore.query(),
        profiles: xhr.get("/aps/2/resources/" + tenantId + "/profile"),
        quotasStatistics: xhr.get("/aps/2/resources/" + tenantId + "/quotasStatistics"),
        listWithAnyExistingUser: xhr.get("/aps/2/resources/" + tenantId + "/users?limit(0,1)")
    }).then(function (response) {
        var subscriptions = response.subscriptions,
            subscription = subscriptions[0],
            backupProfile = response.profiles.length ? response.profiles[0] : null,
            quotasStatistics = response.quotasStatistics,
            userQuota = backupProfile ? backupProfile.userQuota : common.USER_QUOTA.STRICT,
            isFirstUser = !response.listWithAnyExistingUser.length;

        load(["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {title: _("Main")}, [
                ["aps/Output", {
                    label: _("User name"),
                    value: common.escapeAngularBrackets(user.displayName + ' (' + user.login + ')')
                }],
                ["aps/CheckBox", {
                    id: "adminPrivileges",
                    label: _("Enable administrator privileges"),
                    checked: isFirstUser,
                    hint: _('An administrator can manage backups of other users and back up Office 365 seats.')
                }]
            ]],
            ["aps/FieldSet", {title: _('Notifications')}, [
                ["aps/CheckBox", {
                    id: "notificationsManagement",
                    label: _("Enable notifications about quota excess")
                }],
                ["aps/Select", {
                    id: "notifyBackup",
                    label: _("Backup notifications"),
                    store: new Memory({data: common.getBackupNotificationOptions(), idProperty: "value"})
                }]
            ]],
            ["aps/FieldSet", {
                id: "quotaContainer",
                visible: userQuota != common.USER_QUOTA.OFF,
                description: quotas.getQuotaDescription(userQuota),
                title: _('Quotas')
            }]
        ]]).then(function () {
            if (userQuota != common.USER_QUOTA.OFF) {
                var quotaContainer = registry.byId("quotaContainer");
                quotaContainer.set("description", quotas.getQuotaDescription(userQuota));
                quotaContainer.set("visible", true);
                renderQuota(quotasStatistics, setMaxQuota, userQuota);
            }

            if (userData && userData[login]) {
                restoreUserData(userData[login]);
            } else {
                setBackupNotificationChecked(backupProfile ? backupProfile.backupNotification : common.NOTIFICATION_LEVEL.MORE);
                setManagementNotificationChecked(backupProfile ? backupProfile.managementNotification : false);
            }
        });

        aps.app.onPrev = function () {
            var dataForPrevStep = {};
            dataForPrevStep[login] = getModel();
            WizardData.put(dataForPrevStep);

            aps.apsc.prev();
        };

        aps.app.onNext = function () {
            var page = registry.byId("apsPageContainer"),
                resourcesStore = new Store({
                    target: "/aps/2/resources/" + subscription.aps.id + "/resources",
                    idProperty: "aps.id"
                });

            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            xhr.get("/aps/2/resources/" + tenant.aps.id + "/loginExists", {
                query: {
                    login: user.login
                }
            }).then(function (existing) {
                if (existing) {
                    messages.removeAll();
                    messages.displayWarning(_("The account with email address already exists in the backup service. Delete the old account or specify a different service user."));
                    aps.apsc.cancelProcessing();
                    return;
                }
                resourcesStore.query().then(function (resources) {
                    for (var i = 0; i < resources.length; i++) {
                        if (resources[i].apsType == common.TYPES.USERS) {
                            if (resources[i].usage >= resources[i].limit) {
                                messages.displayWarning(_("Limit for backup users has been reached."));

                                aps.apsc.cancelProcessing();
                                return;
                            }
                        }
                    }
                    var model = getModel();
                    aps.apsc.gotoView("fake", null, {
                        objects: [getPlainValue(model)],
                        userAttr: "user"
                    });
                }, messages.displayError);
            }, messages.displayError);
        };
    }, messages.displayError);
});