/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/xhr",
    "app/common",
    "app/messages",
    "app/wizardData",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue"
], function (
    load,
    xhr,
    common,
    messages,
    wizardData,
    registry,
    at,
    getStateful,
    getPlainValue
) {
    var user = aps.context.vars.user,
        model = getStateful({
            password: ''
        });

    load(["aps/PageContainer", {id: "apsPageContainer"}, [
        ["aps/FieldSet", {
            id: "passwordFieldSet",
            title: _("New Password")
        }, [
            ["aps/Output", {
                id: "login",
                label: _("Login"),
                value: user.endUserLogin
            }],
            ["./js/widgets/Password.js", {
                id: "password",
                label: _("Password"),
                value: at(model, "password"),
                hint: _("7-characters minimum, including letters in different cases and digits"),
                required: true
            }]
        ]]
    ]]).then(function () {
        aps.app.onSubmit = function () {
            messages.removeAll();
            var fieldSet = registry.byId("passwordFieldSet");
            if (!fieldSet.validate()) {
                messages.displayMessage(_("Incorrect or empty field value(s). Please retype info in the highlighted fields."), "error");
                aps.apsc.cancelProcessing();
                return;
            }

            xhr("/aps/2/resources/" + user.aps.id + "/changePassword", {
                method: "POST",
                headers: {"Content-Type": "application/json"},
                data: JSON.stringify(getPlainValue(model))
            }).then(function () {
                var viewId = common.getRelatedViewId('view'),
                    data = {};

                data.message = {
                    text: _('The backup service password has been successfully changed for user "__name__".', {
                        name: user.endUserLogin
                    }),
                    type: "info"
                };
                wizardData.setWizardData(data, viewId);
                aps.apsc.gotoView(viewId, user.aps.id);
            }, messages.displayError);
        };
    });

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('view'), user.aps.id);
    };
});