/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/xhr",
    "aps/ResourceStore",
    "aps/Memory",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "dijit/registry",
    "dojo/promise/all",
    "./js/common.js",
    "./js/messages.js",
    "./js/quotas.js",
    "./js/wizardData.js"
],
function (
    load,
    xhr,
    Store,
    Memory,
    at,
    getStateful,
    getPlainValue,
    registry,
    all,
    common,
    messages,
    quotas,
    WizardData
) {
    var firstStepViewId = common.getRelatedViewId('new-select'),
        secondStepViewId = common.getRelatedViewId('new-configure'),
        secondStepData = WizardData.getWizardData(secondStepViewId),
        user = secondStepData.user,
        tenant = aps.context.vars.tenant,
        tenantId = tenant.aps.id,
        backupProfileDefer = xhr.get("/aps/2/resources/" + tenantId + "/profile"),
        quotasStatisticsDefer = xhr.get("/aps/2/resources/" + tenantId + "/quotasStatistics"),
        userStore = new Store({
            apsType: common.TYPES.USERS,
            target: "/aps/2/resources/" + tenantId + "/users"
        }),
        enduserStore = new Store({
            apsType: "http://aps-standard.org/types/core/service-user/1.0",
            target: "/aps/2/resources/"
        });

    function restoreUserData(userData) {
        var userStorageUsage = registry.byId("userStorageUsage"),
            userServerCount = registry.byId("userServerCount"),
            userVmCount = registry.byId("userVmCount"),
            userWorkstationCount = registry.byId("userWorkstationCount"),
            userMobileDeviceCount = registry.byId("userMobileDeviceCount"),
            userWebHostingServerCount = registry.byId("userWebHostingServerCount"),
            userWebsiteCount = registry.byId("userWebsiteCount");

        if (userStorageUsage) {
            userStorageUsage.set("value", userData.userStorageUsage.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.userStorageUsage.limit / common.BYTES_IN_GB);
        }
        if (userWorkstationCount) {
            userWorkstationCount.set("value", userData.UserWorkstationCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserWorkstationCount.limit);
        }
        if (userServerCount) {
            userServerCount.set("value", userData.userServerCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.userServerCount.limit);
        }
        if (userVmCount) {
            userVmCount.set("value", userData.UserVmCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserVmCount.limit);
        }
        if (userMobileDeviceCount) {
            userMobileDeviceCount.set("value", userData.UserMobileDeviceCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserMobileDeviceCount.limit);
        }
        if (userWebHostingServerCount) {
            userWebHostingServerCount.set("value", userData.userWebHostingServerCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.userWebHostingServerCount.limit);
        }
        if (userWebsiteCount) {
            userWebsiteCount.set("value", userData.UserWebsiteCount.limit == common.QUOTA_UNLIMITED_VALUE ?
                "" : userData.UserWebsiteCount.limit);
        }
    }

    function renderQuotasControls(quotasStatistics, setMaxQuota, userQuota) {
        var quotaContainer = registry.byId("quotaContainer"),
            controls = quotas.getQuotaControlsForNewUser(tenant, quotasStatistics, setMaxQuota, userQuota);
        quotaContainer.addChild(controls.storage);
        if (controls.localStorage) {
            quotaContainer.addChild(controls.localStorage);
        }
        if (controls.workstations) {
            quotaContainer.addChild(controls.workstations);
        }
        if (controls.servers) {
            quotaContainer.addChild(controls.servers);
        }
        if (controls.vms) {
            quotaContainer.addChild(controls.vms);
        }
        if (controls.mobiles) {
            quotaContainer.addChild(controls.mobiles);
        }
        if (controls.webHostingServers) {
            quotaContainer.addChild(controls.webHostingServers);
        }
        if (controls.websites) {
            quotaContainer.addChild(controls.websites);
        }
    }

    function hasQuotasToManage(userQuota, quotasStatistics) {
        var availableQuotas = Object.values(quotasStatistics).filter(function(quota) {
           return quota.limit != 0;
        });
        return userQuota != common.USER_QUOTA.OFF && availableQuotas.length;
    }

    function addQuotaValues(userData) {
        var userStorageUsage = registry.byId("userStorageUsage"),
            userServerCount = registry.byId("userServerCount"),
            userVmCount = registry.byId("userVmCount"),
            userWorkstationCount = registry.byId("userWorkstationCount"),
            userMobileDeviceCount = registry.byId("userMobileDeviceCount"),
            userWebHostingServerCount = registry.byId("userWebHostingServerCount"),
            userWebsiteCount = registry.byId("userWebsiteCount");

        function getLimitValue(limitFieldValue) {
            return limitFieldValue === "" ? common.QUOTA_UNLIMITED_VALUE : limitFieldValue;
        }

        function getStorageLimitValue(limitFieldValue) {
            return limitFieldValue === "" ? common.QUOTA_UNLIMITED_VALUE : limitFieldValue * common.BYTES_IN_GB;
        }

        userData.userStorageUsage = {limit: userStorageUsage ? getStorageLimitValue(userStorageUsage.value) : 0};
        userData.userServerCount = {limit: userServerCount ? getLimitValue(userServerCount.value) : 0};
        userData.UserVmCount = {limit: userVmCount ? getLimitValue(userVmCount.value) : 0};
        userData.UserWorkstationCount = {limit: userWorkstationCount ? getLimitValue(userWorkstationCount.value) : 0};
        userData.UserMobileDeviceCount = {limit: userMobileDeviceCount ? getLimitValue(userMobileDeviceCount.value) : 0};
        userData.userWebHostingServerCount = {limit: userWebHostingServerCount ? getLimitValue(userWebHostingServerCount.value) : 0};
        userData.UserWebsiteCount = {limit: userWebsiteCount ? getLimitValue(userWebsiteCount.value) : 0};
    }

    var requests = {
        profiles: backupProfileDefer,
        quotasStatistics: quotasStatisticsDefer,
        listWithAnyExistingUser: xhr.get("/aps/2/resources/" + tenantId + "/users?limit(0,1)")
    };
    if (user.aps.id) {
        requests.user = xhr.get("/aps/2/resources/" + user.aps.id);
    }

    all(requests).then(function(response) {
        var backupProfile = response.profiles.length ? response.profiles[0] : null,
            userQuota = backupProfile ? backupProfile.userQuota : common.USER_QUOTA.STRICT,
            savedUser = response.user,
            quotasStatistics = response.quotasStatistics,
            quotasSectionIsVisible = hasQuotasToManage(userQuota, quotasStatistics),
            userData = secondStepData.userData ? secondStepData.userData : null,
            displayName = user.displayName || savedUser.displayName,
            setMaxQuota = !userData,
            isFirstUser = !response.listWithAnyExistingUser.length;

        var userModel = getStateful(userData || {
            userStorageUsage: 0,
            userServerCount: 0,
            UserVmCount: 0,
            UserWorkstationCount: 0,
            UserMobileDeviceCount: 0,
            userWebHostingServerCount: 0,
            UserWebsiteCount: 0,
            privileged: isFirstUser ? common.USER_PRIVILEGED.YES : common.USER_PRIVILEGED.NO,
            notifyBackup: String(backupProfile ? backupProfile.backupNotification : common.NOTIFICATION_LEVEL.MORE),
            notificationsManagement: backupProfile ? !!backupProfile.managementNotification : false,
            activateDuringCreation: !savedUser || (savedUser.aps.type.indexOf("http://parallels.com/aps/types/pa/admin-user/") < 0)
        });

        load(["aps/PageContainer", { id: "apsPageContainer" }, [
            ["aps/WizardControl", {
                id: common.genId('wizardcontrol'),
                steps: [
                    {viewId: firstStepViewId, text: _("Select user")},
                    {viewId: secondStepViewId, text: _("Configure service"), active: true}
                ]
            }],
            ["aps/FieldSet", {
                id: common.genId('fieldset'),
                title: _("Main")
            }, [
                ["aps/Output", {
                    id: common.genId('output'),
                    label: _("User name"),
                    value: displayName
                }],
                ["aps/CheckBox", {
                    id: "adminPrivileges",
                    label: _("Enable administrator privileges"),
                    checked: at(userModel, "privileged").transform({
                        format: function(value){
                            return value === common.USER_PRIVILEGED.YES;
                        },
                        parse: function(value){
                            return (value)? common.USER_PRIVILEGED.YES : common.USER_PRIVILEGED.NO;
                        }
                    }),
                    hint: _('An administrator can manage backups of other users and back up Office 365 seats.')
                }]
            ]],
            ["aps/FieldSet", {
                id: common.genId('fieldset'),
                title: _('Notifications')
            }, [
                ["aps/CheckBox", {
                    id: "quotaSelect",
                    label: _("Enable notifications about quota excess"),
                    checked: at(userModel, "notificationsManagement")
                }],
                ["aps/Select", {
                    id: "notificationSelect",
                    label: _("Backup notifications"),
                    value: at(userModel, "notifyBackup"),
                    store: new Memory({ data: common.getBackupNotificationOptions(), idProperty: "value" })
                }]
            ]],
            ["aps/FieldSet", {
                id: "quotaContainer",
                visible: quotasSectionIsVisible,
                description: quotas.getQuotaDescription(userQuota),
                title: _('Quotas')
            }]
        ]]).then(function(){
            if (quotasSectionIsVisible) {
                renderQuotasControls(quotasStatistics, setMaxQuota, userQuota);
            }

            if (userData) {
                restoreUserData(userData);
            }
        });

        aps.app.onSubmit = function () {
            var page = registry.byId("apsPageContainer");
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            function createAccount(userModel, user) {
                userModel.user = [ user ];
                userStore.put(userModel).then(function() {
                    WizardData.resetWizardData(firstStepViewId);
                    WizardData.resetWizardData(secondStepViewId);
                    aps.apsc.gotoView(common.getRelatedViewId('list'));
                }, messages.displayError);
            }

            addQuotaValues(userModel);
            userModel.aps = { type: common.TYPES.USERS };
            userModel.tenant = [{ aps: { id: tenantId } }];

            xhr.get("/aps/2/resources/" + tenant.aps.id + "/loginExists", {
                query: {
                    login: user.login ? user.login : savedUser.login
                }
            }).then(function (existing) {
                if (existing) {
                    messages.removeAll();
                    messages.displayWarning(_("The account with email address already exists in the backup service. Delete the old account or specify a different service user."));
                    aps.apsc.cancelProcessing();
                    return;
                }
                if (user.aps.id) {
                    createAccount(userModel, user);
                } else {
                    enduserStore.put(user).then(function(enduser){
                        user = { aps: { id: enduser.aps.id }};
                        createAccount(userModel, user);
                    }, messages.displayError);
                }
            }, messages.displayError);
        };

        aps.app.onPrev = function () {
            addQuotaValues(userModel);
            secondStepData.userData = getPlainValue(userModel);
            WizardData.setWizardData(secondStepData);
            aps.apsc.gotoView(firstStepViewId);
        };
    }, messages.displayError);

    aps.app.onCancel = function () {
        WizardData.resetWizardData(firstStepViewId);
        WizardData.resetWizardData(secondStepViewId);
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    };
});
