/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

/*
 System phrases for localization parser
 _("All");
 */
define([
    "aps/i18n",
    "aps/xhr",
    "dijit/registry",
    "dojo/Deferred",
    "aps/ResourceStore"
], function (
    _,
    xhr,
    registry,
    Deferred,
    Store
) {
    return {
        APP_ID: "http://www.acronis.com/aps/cloud-backup-v1",

        SERVICE_TITLE: _("Backup"),

        BYTES_IN_GB: 1024 * 1024 * 1024,

        QUOTA_UNLIMITED_VALUE: -1,

        USER_QUOTA: {
            OFF: 0,
            STRICT: 1,
            SOFT: 2
        },

        USER_PRIVILEGED: {
            NO: 0,
            YES: 1
        },

        STATUS: {
            INACTIVE: "-1",
            DISABLED: "0",
            ENABLED: "1"
        },

        NOTIFICATION_LEVEL: {
            OFF: 0,
            LESS: 1,
            MORE: 3,
            ALL: 7
        },

        NOTIFICATION_MANAGEMENT: {
            DISABLE: false,
            ENABLE: true
        },

        STORAGE_POLICY: {
            PARENT: "0",
            OWN: "1"
        },

        STORAGE_INHERITED: {
            NO: 0,
            YES: 1
        },

        TYPES: {
            USERS: "http://acronis.com/aps/cloud-backup-v1/users/7.7",
            TENANT: "http://acronis.com/aps/cloud-backup-v1/tenant/9.7",
            GLOBALS: "http://acronis.com/aps/cloud-backup-v1/globals/9.5",
            RESELLER: "http://acronis.com/aps/cloud-backup-v1/reseller/1.0",
            PROFILE: "http://acronis.com/aps/cloud-backup-v1/backup-profile/5.0",
            DATACENTER: "http://acronis.com/aps/cloud-backup-v1/datacenter/4.1",
            ADMIN: "http://acronis.com/aps/cloud-backup-v1/admin/4.0",
            MINIMAL_COMMITMENT_TIER: "http://acronis.com/aps/cloud-backup-v1/minimal-commitment-tier/4.0"
        },

        USER_STATUS: {
            ERROR: "-1",
            INACTIVE: "0",
            ACTIVE: "1",
            IN_PROGRESS: "2",
            NOT_ACTIVATED: "3"
        },

        BACKUP_LOCATIONS: {
            ONLY_CLOUD: "0",
            LOCAL_AND_CLOUD: "1"
        },

        PACKAGE_TYPE: {
            PREMIUM: 'premium'
        },
        GROUP_TYPE: {
            CUSTOMER: "0",
            PARTNER: "1"
        },
        TENANT_TYPE: {
            RESELLER: "RESELLER",
            CUSTOMER: "CUSTOMER",
            PARTNER: "PARTNER"
        },

        formatStorageSize: function (value) {
            var units = [_('B'), _('KB'), _('MB'), _('GB'), _('TB'), _('PB'), _('EB')];
            var multiplier = 1024;
            while (value >= multiplier) {
                value = value / multiplier;
                units.shift();
            }
            var decimalPlaces = 1;
            value = Number(value);
            value = value.toFixed(decimalPlaces) == Math.round(value) ? Math.round(value) : value.toFixed(decimalPlaces);
            return value + " " + units.shift();
        },

        getBackupNotificationDescription: function (value) {
            switch (value) {
                case this.NOTIFICATION_LEVEL.OFF:
                    return _('Off');
                case this.NOTIFICATION_LEVEL.LESS:
                    return _('Less');
                case this.NOTIFICATION_LEVEL.MORE:
                    return _('More');
                case this.NOTIFICATION_LEVEL.ALL:
                    return _('All');
                default:
                    return _('Unknown');
            }
        },

        getBackupNotificationText: function (value) {
            switch (value) {
                case this.NOTIFICATION_LEVEL.OFF:
                    return _('Do not send any backup notifications');
                case this.NOTIFICATION_LEVEL.LESS:
                    return _('Send notifications about errors only');
                case this.NOTIFICATION_LEVEL.MORE:
                    return _('Send notifications about errors and warnings');
                case this.NOTIFICATION_LEVEL.ALL:
                    return _('Send all types backup notifications');
                default:
                    return _('Unknown');
            }
        },

        getBackupNotificationOptions: function () {
            return [
                {
                    value: this.NOTIFICATION_LEVEL.OFF.toString(),
                    label: this.getBackupNotificationText(this.NOTIFICATION_LEVEL.OFF)
                },
                {
                    value: this.NOTIFICATION_LEVEL.LESS.toString(),
                    label: this.getBackupNotificationText(this.NOTIFICATION_LEVEL.LESS)
                },
                {
                    value: this.NOTIFICATION_LEVEL.MORE.toString(),
                    label: this.getBackupNotificationText(this.NOTIFICATION_LEVEL.MORE)
                },
                {
                    value: this.NOTIFICATION_LEVEL.ALL.toString(),
                    label: this.getBackupNotificationText(this.NOTIFICATION_LEVEL.ALL)
                }
            ];
        },

        getEnabledDisabledOptions: function () {
            return [
                {value: this.STATUS.ENABLED, label: _("Enabled")},
                {value: this.STATUS.DISABLED, label: _("Disabled")}
            ];
        },

        getStorageNameOption: function (storageId, storages) {
            var self = this;

            switch (storageId) {
                case "default":
                    return _("Vendor's default");
                case "reseller":
                    return _("Resellers must use their own storage");
                default:
                    var storageName = "";
                    storages.some(function (storage) {
                        if (storage.id == storageId) {
                            storageName = self.escapeAngularBrackets(storage.name);
                            return true;
                        }
                        return false;
                    });
                    return storageName;
            }
        },

        getRadioButtonGroupValue: function (groupName) {
            var radios = document.getElementsByName(groupName);
            for (var i = 0; i < radios.length; i++) {
                if (radios[i].checked === true) {
                    return radios[i].value;
                }
            }
        },

        formatLocalDateString: function (date, locale) {
            locale = locale.replace('_', '-');
            try {
                return date.toLocaleString(locale);
            } catch (e) {
                return date.toLocaleString();
            }
        },

        filterAccountsBySubscription: function (subscriptionId, accounts) {
            var result = [];
            accounts.forEach(function (account) {
                if (account.subscriptionId == subscriptionId) {
                    result.push(account);
                }
            });
            return result;
        },

        escapeAngularBrackets: function (string) {
            return string ? string.replace(/</g, '&lt;').replace(/>/g, '&gt;') : string;
        },

        getRelatedViewId: function (view) {
            var uri = this.getCurrentView();
            if (uri === undefined) {
                return view;
            }
            var parts = uri.split('--').slice(0, -1);
            parts.push(view);
            return parts.join('--');
        },

        getCurrentView: function() {
            return typeof aps.context.viewId === "undefined" ? aps.context.view.id.split('#').pop() : aps.context.viewId;
        },

        getStatusLabel: function (statusId) {
                var statusesLabels = {
                10: _("Ordered"),
                15: _("Trial"),
                30: _("Active"),
                40: _("Graced"),
                50: _("Expired"),
                60: _("Terminated"),
                70: _("Canceled"),
                80: _("Administrative Hold"),
                85: _("Credit Hold"),
                89: _("Credit and  Administrative Hold")
            };
            return statusesLabels[statusId];
        },

        getServiceStatusLabel: function (statusId) {
            var serviceStatuses = {
                10: _('Not Provisioned'),
                20: _('Provisioning'),
                30: _('Stopped'),
                40: _('Starting'),
                50: _('Running'),
                60: _('Stopping'),
                70: _('Removing'),
                80: _('Changing Plan'),
                90: _('Removed')
            };
            return serviceStatuses[statusId];
        },

        getLanguage: function (locale) {
            return locale.replace(/_.*/, '');
        },

        cmpType: function (originType, inheritType) {
            var regexp = new RegExp(/^(.+\/)(\d+.\d+)$/);
            var originMatches = regexp.exec(originType);
            if (!originMatches || originMatches.length !== 3) {
                return false;
            }

            var inheritMatches = regexp.exec(inheritType);
            if (!inheritMatches || inheritMatches.length !== 3) {
                return false;
            }

            var originURI = originMatches[1],
                inheritURI = inheritMatches[1];

            return originURI === inheritURI;
        },

        isUsers: function (apsType) {
            return this.cmpType(apsType, this.TYPES.USERS);
        },

        isTenant: function (apsType) {
            return this.cmpType(apsType, this.TYPES.TENANT);
        },

        isGlobals: function (apsType) {
            return this.cmpType(apsType, this.TYPES.GLOBALS);
        },

        isReseller: function (apsType) {
            return this.cmpType(apsType, this.TYPES.RESELLER);
        },

        isProfile: function (apsType) {
            return this.cmpType(apsType, this.TYPES.PROFILE);
        },

        getBackupProfile: function (tenant) {
            var deferred = new Deferred();
            var store = new Store({target: "/aps/2/resources/" + tenant.aps.id + "/profile"});
            if (tenant.profile) {
                deferred.resolve(tenant.profile);
            }
            else {
                store.query().then(function (profiles) {
                    profiles && profiles.length ? deferred.resolve(profiles[0]) : deferred.resolve(null);
                }, deferred.reject);
            }
            return deferred;
        },

        getUserQuota: function (profile) {
            return profile ? Number(profile.userQuota) : this.USER_QUOTA.STRICT;
        },

        getBackupNotification: function (profile) {
            return profile ? Number(profile.backupNotification) : this.NOTIFICATION_LEVEL.OFF;
        },

        getManagementNotification: function (profile) {
            return profile ? (!!profile.managementNotification) : this.NOTIFICATION_MANAGEMENT.DISABLE;
        },
        isBackupAccountResource: function (resource) {
            var possibleResources = [
                {"apsType": this.TYPES.TENANT, "property": "StorageUsage"},
                {"apsType": this.TYPES.TENANT, "property": "LocalStorageUsage"},
                {"apsType": this.TYPES.TENANT, "property": "ServerCount"},
                {"apsType": this.TYPES.TENANT, "property": "VmCount"},
                {"apsType": this.TYPES.TENANT, "property": "MobileDeviceCount"},
                {"apsType": this.TYPES.TENANT, "property": "MailboxCount"},
                {"apsType": this.TYPES.TENANT, "property": "WorkstationCount"},
                {"apsType": this.TYPES.TENANT, "property": "WebsiteCount"},
                {"apsType": this.TYPES.TENANT, "property": "AcronisHostedStorage"},
                {"apsType": this.TYPES.TENANT, "property": "ServiceProviderHostedStorage"}
            ];
            return possibleResources.some(function (possibleResource) {
                return resource.apsType === possibleResource.apsType && resource.property === possibleResource.property;
            });
        },
        isStorageResource: function (resource) {
            var possibleResources = [
                {"apsType": this.TYPES.TENANT, "property": "StorageUsage"},
                {"apsType": this.TYPES.TENANT, "property": "LocalStorageUsage"},
                {"apsType": this.TYPES.TENANT, "property": "AcronisHostedStorage"},
                {"apsType": this.TYPES.TENANT, "property": "ServiceProviderHostedStorage"}
            ];
            return possibleResources.some(function (possibleResource) {
                return resource.apsType === possibleResource.apsType && resource.property === possibleResource.property;
            });
        },
        getMinimalCommitmentLevelName: function (level) {
            var n = parseInt(level, 10);
            if (isNaN(n)) return '';
            if (n < -1) return _("Fixed custom commitment");
            if (n == -1) return _("No commitment");
            if (n === 0) return _("Minimum");
            return String(n);
        },
        genId: function (widgetType) {
            return dijit.registry.getUniqueId(widgetType);
        },
        getIconRoute: function (path) {
            var pathGenerated = "";
            var executionPath = window.location.pathname;
            var elementsPath = executionPath.substring(executionPath.indexOf("/ui/"), executionPath.length).split("/");

            for (var i = 1; i < elementsPath.length - 1; i++) {
                pathGenerated = pathGenerated + "../";
            }

            return pathGenerated + path;
        },
        loginToConsole: function (backupAccount) {
            var deferred = new Deferred();
            if (!backupAccount) {
                deferred.reject();
                return deferred;
            }
            xhr.get("/aps/2/resources/" + backupAccount.aps.id + "/loginByToken").then(function (uri) {
                deferred.resolve(uri);
            }, deferred.reject);

            return deferred;
        }
    };
});
