/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

define([
    'aps/Tile',
    'aps/FieldSet',
    'aps/Output',
    './../utils',
    './../../common',
    './getResourceOutput',
    './getStorageOutput'
], function (
    Tile,
    FieldSet,
    Output,
    utils,
    common,
    getResourceOutput,
    getStorageOutput
) {

    function createFieldSet(tenant) {
        var fs = new FieldSet({
            gridSize: 'md-12 xs-12'
        });

        if (tenant.StorageUsage && tenant.StorageUsage.limit) {
            fs.addChild(getStorageOutput({
                gridSize: 'md-4 xs-12',
                label: _('Cloud Storage'),
                noneLegend: _('None'),
                limitedLegend: _('${usage} ${usageUnit} of ${limit} ${limitUnit} used'),
                unlimitedLegend: _('${usage} ${usageUnit} used'),
                usage: tenant.StorageUsage.usage || 0,
                limit: tenant.StorageUsage.limit || 0
            }));
        }

        if (tenant.LocalStorageUsage && tenant.LocalStorageUsage.limit) {
            fs.addChild(getStorageOutput({
                gridSize: 'md-4 xs-12',
                label: _('Local Storage'),
                noneLegend: _('None'),
                limitedLegend: _('${usage} ${usageUnit} of ${limit} ${limitUnit} used'),
                unlimitedLegend: _('${usage} ${usageUnit} used'),
                usage: tenant.LocalStorageUsage.usage || 0,
                limit: tenant.LocalStorageUsage.limit || 0
            }));
        }

        if (tenant.WorkstationCount && tenant.WorkstationCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Workstations'),
                limitedLegend: _('${usage} of ${limit} machines protected'),
                zeroUsageLegend: _('${limit} machines can be protected'),
                unlimitedLegend: _('${usage} machines protected'),
                usage: tenant.WorkstationCount.usage || 0,
                limit: tenant.WorkstationCount.limit || 0
            }));
        }

        if (tenant.ServerCount && tenant.ServerCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Servers'),
                limitedLegend: _('${usage} of ${limit} machines protected'),
                zeroUsageLegend: _('${limit} machines can be protected'),
                unlimitedLegend: _('${usage} machines protected'),
                usage: tenant.ServerCount.usage || 0,
                limit: tenant.ServerCount.limit || 0
            }));
        }

        if (tenant.VmCount && tenant.VmCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Virtual Machines'),
                limitedLegend: _('${usage} of ${limit} machines protected'),
                zeroUsageLegend: _('${limit} machines can be protected'),
                unlimitedLegend: _('${usage} machines protected'),
                usage: tenant.VmCount.usage || 0,
                limit: tenant.VmCount.limit || 0
            }));
        }

        if (tenant.MobileDeviceCount && tenant.MobileDeviceCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Mobile Devices'),
                limitedLegend: _('${usage} of ${limit} devices protected'),
                zeroUsageLegend: _('${limit} devices can be protected'),
                unlimitedLegend: _('${usage} devices protected'),
                usage: tenant.MobileDeviceCount.usage || 0,
                limit: tenant.MobileDeviceCount.limit || 0
            }));
        }

        if (tenant.WebsiteCount && tenant.WebsiteCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Websites'),
                limitedLegend: _('${usage} of ${limit} websites protected'),
                zeroUsageLegend: _('${limit} websites can be protected'),
                unlimitedLegend: _('${usage} websites protected'),
                usage: tenant.WebsiteCount.usage || 0,
                limit: tenant.WebsiteCount.limit || 0
            }));
        }

        if (tenant.MailboxCount && tenant.MailboxCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Office 365 seats'),
                limitedLegend: _('${usage} of ${limit} seats protected'),
                zeroUsageLegend: _('${limit} seats can be protected'),
                unlimitedLegend: _('${usage} seats protected'),
                usage: tenant.MailboxCount.usage || 0,
                limit: tenant.MailboxCount.limit || 0
            }));
        }

        if (tenant.O365Mailboxes && tenant.O365Mailboxes.limit) {
            fs.addChild(new Output({
                gridSize: 'md-4 xs-12',
                label: _('Office 365 Mailboxes'),
                value: _('Enabled')
            }));
        }

        if (tenant.O365OneDrive && tenant.O365OneDrive.limit) {
            fs.addChild(new Output({
                gridSize: 'md-4 xs-12',
                label: _('Office 365 OneDrive'),
                value: _('Enabled')
            }));
        }

        if (tenant.O365SharePointSitesCount && tenant.O365SharePointSitesCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Office 365 SharePoint Online'),
                limitedLegend: _('${usage} of ${limit} sites protected'),
                zeroUsageLegend: _('${limit} sites can be protected'),
                unlimitedLegend: _('${usage} sites protected'),
                usage: tenant.O365SharePointSitesCount.usage || 0,
                limit: tenant.O365SharePointSitesCount.limit || 0
            }));
        }

        if (tenant.WebHostingServerCount && tenant.WebHostingServerCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Web Hosting Servers'),
                limitedLegend: _('${usage} of ${limit} servers protected'),
                zeroUsageLegend: _('${limit} servers can be protected'),
                unlimitedLegend: _('${usage} servers protected'),
                usage: tenant.WebHostingServerCount.usage || 0,
                limit: tenant.WebHostingServerCount.limit || 0
            }));
        }

        if (tenant.DrStorageUsage && tenant.DrStorageUsage.limit) {
            fs.addChild(getStorageOutput({
                gridSize: 'md-4 xs-12',
                label: _('Disaster recovery storage'),
                noneLegend: _('None'),
                limitedLegend: _('${usage} ${usageUnit} of ${limit} ${limitUnit} used'),
                unlimitedLegend: _('${usage} ${usageUnit} used'),
                usage: tenant.DrStorageUsage.usage || 0,
                limit: tenant.DrStorageUsage.limit || 0
            }));
        }

        if (tenant.ComputePointsCount && tenant.ComputePointsCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Compute points'),
                limitedLegend: _('${usage} of ${limit} compute points'),
                zeroUsageLegend: _('${limit} compute points available'),
                unlimitedLegend: _('${usage} compute points used'),
                usage: tenant.ComputePointsCount.usage || 0,
                limit: tenant.ComputePointsCount.limit || 0
            }));
        }

        if (tenant.PublicIPsCount && tenant.PublicIPsCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Public IP addresses'),
                limitedLegend: _('${usage} of ${limit} public IP addresses'),
                zeroUsageLegend: _('${limit} public IP addresses available'),
                unlimitedLegend: _('${usage} public IP addresses used'),
                usage: tenant.PublicIPsCount.usage || 0,
                limit: tenant.PublicIPsCount.limit || 0
            }));
        }

        if (tenant.CloudServersCount && tenant.CloudServersCount.limit) {
            fs.addChild(getResourceOutput({
                gridSize: 'md-4 xs-12',
                label: _('Cloud servers'),
                limitedLegend: _('${usage} of ${limit} cloud servers'),
                zeroUsageLegend: _('${limit} cloud servers available'),
                unlimitedLegend: _('${usage} cloud servers used'),
                usage: tenant.CloudServersCount.usage || 0,
                limit: tenant.CloudServersCount.limit || 0
            }));
        }

        if (tenant.InternetAccess && tenant.InternetAccess.limit) {
            fs.addChild(new Output({
                gridSize: 'md-4 xs-12',
                label: _('Internet access'),
                value: _('Enabled')
            }));
        }

        return fs;
    }

    function getTenantResourcesTile(tenant, subscriptionId, trial, canAdjustLimits) {

        var buttons = trial ? {
            autoBusy: false,
            title: _('Upgrade to Paid'),
            iconClass: 'fa-shopping-cart',
            onClick: function () {
                aps.apsc.gotoView('http://www.parallels.com/ccp-billing-subscription-management#upgrade-wizard', null, {
                    subscriptionId: subscriptionId,
                    returnUrl: utils.createReturnUrl()
                });
            }
        } : {
            visible: canAdjustLimits,
            autoBusy: false,
            title: _('Adjust Limits'),
            iconClass: 'fa-cog',
            onClick: function () {
                var viewId = tenant.type == common.TENANT_TYPE.PARTNER ? '#nextcp-ccp-partner' : '#nextcp-ccp';
                aps.apsc.gotoView('http://www.parallels.com/ccp-billing#change-resource-limits', null, {
                    subscriptionId: subscriptionId,
                    returnView: 'http://www.acronis.com/aps/cloud-backup-v1' + viewId
                });
            }
        };

        var tile = new Tile({
            title: _('Subscription Limits'),
            gridSize: 'md-8 xs-12',
            buttons: [buttons]
        });

        tile.addChild(createFieldSet(tenant));

        return tile;
    }

    return getTenantResourcesTile;
});
