/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/_base/declare",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "aps/_PopupView",
    "aps/xhr",
    "./../utils",
    "./../../messages"
], function (
    declare,
    at,
    getStateful,
    getPlainValue,
    _PopupView,
    xhr,
    utils,
    messages
) {

    return declare(_PopupView, {
        size: "md",
        _user: null,
        _model: null,

        init: function () {
            var self = this;
            self._model = getStateful({
                login: "",
                password: ""
            });
            return [
                ['aps/Panel', [
                    ['aps/FieldSet', [
                        ['aps/Output', {
                            id: self.genId('activate_description'),
                            value: _("Please specify a password to activate the backup user. You will need these credentials to install a backup agent on your device and to log in to the backup console. The password can be changed in the Control Panel at a later time.")
                        }],
                        ['aps/TextBox', {
                            id: self.genId('login'),
                            label: _("Login"),
                            value: at(self._model, "login"),
                            gridSize: "md-12 xs-12",
                            required: true,
                            disabled: true
                        }],
                        ['aps/Password', {
                            id: self.genId("password"),
                            label: _("Password"),
                            value: at(self._model, "password"),
                            gridSize: "md-12 xs-12",
                            required: true,
                            showResetButton: true,
                            passwordVisible: true,
                            showStrengthIndicator: true,
                            constraints: {
                                minimumLength: 8,
                                maximumLength: 16
                            },
                            generator: utils.passwordGenerator,
                            extraValidator: function (value) {
                                var res = utils.validatePassword(value);
                                if (res === true) {
                                    return true;
                                }
                                else {
                                    this.set("invalidMessage", res);
                                    return false;
                                }
                            }
                        }]
                    ]]
               ]]
            ];
        },
        onContext: function() {
            var self = this;
            self._user = aps.context.params.enduser;
            self._model.set({
                login: self._user.endUserLogin
            });
            aps.apsc.hideLoading();
        },

        onShow: function() {
            var password = this.byId("password");
            password.generate();
        },
        onHide: function() {
            var self = this;
            self._user = null;
            self._model.set({
                login: "",
                password: ""
            });
        },
        onCancel: function() {
            this.cancel();
        },
        onSubmit: function() {
            var self = this;
            if (!self.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            xhr("/aps/2/resources/" + self._user.aps.id + "/activateAccount", {
                method: "POST",
                headers: { "Content-Type": "application/json" },
                data: JSON.stringify(getPlainValue(self._model))
            }).then(function () {
                self.submit();
            }, messages.displayError);
        }
    });
});

