/*
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

define([
    'dojo/_base/declare',
    'dojox/mvc/getStateful',
    'aps/_View',
    'aps/load',
    'aps/xhr',
    'aps/Button',
    'aps/Tiles',
    'aps/Tile',
    'aps/FieldSet',
    'aps/Output',
    './../../messages',
    './../../agentsHelper',
    './../../common',
    './../../icons',
    './../utils'
], function (
    declare,
    getStateful,
    View,
    load,
    xhr,
    Button,
    Tiles,
    Tile,
    FieldSet,
    Output,
    messages,
    agentsHelper,
    common,
    icons,
    utils
) {
    return declare(View, {
        _agents: null,
        _container: null,
        _containerId: null,
        tiles: null,
        winTile: null,
        linuxTile: null,
        macTile: null,

        init: function () {
            var self = this,
                domNode = self.domNode;

            self._containerId = self.genId('download_view');
            self._agents = getStateful({});
            self._container = new Tiles({
                id: self._containerId
            });
            self._container.placeAt(domNode);
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function () {
            var self = this,
                backupAccount = aps.context.vars.enduser,
                agentsURI = '/aps/2/resources/' + backupAccount.aps.id + '/agents?language=' + common.getLanguage(aps.context._locale);

            self._container.removeAll();

            xhr.get(agentsURI).then(function(agents) {
                self.renderAgents(agents);
            });
        },
        renderAgents: function(agents) {
            var self = this,
                agentsList = agentsHelper.buildBackupAgentsList(agents),
                enduser = aps.context.vars.enduser,
                userId = enduser.aps.id,
                otherAgentsTile = new Tile({
                    title: _('All agents'),
                    gridSize: 'md-6 xs-12',
                    id:  self._containerId + '_login_tile'
                });

            function getAgentOutputContent() {
                return [
                    '<div style=\'cursor:pointer;border: 1px solid #000; padding: 10px;\'><span style=\'display:table;\'>',
                    '<span style=\'display:table-cell;\'><img src=\'${icon}\' height=\'42\' width=\'42\'></span>',
                    '<span style=\'display:table-cell; vertical-align:middle; padding-left:7px;\'>${value}</span>',
                    '</span></div>'
                ].join('');
            }

            agentsList.forEach(function(agentCategory) {
                var categoryTile = new Tile({
                        title: agentCategory.title,
                        gridSize: 'md-6 xs-12',
                        id: self._containerId + '_category_tile' + agentCategory.category
                    }),
                    categoryFieldset = new FieldSet({
                        gridSize: 'md-12 xs-12',
                        id: self._containerId + '_category_' + agentCategory.category
                    });

                agentCategory.agents.forEach(function(agent) {
                    var agentOutput = new Output({
                            gridSize: 'md-4 xs-12',
                            content: getAgentOutputContent(),
                            icon: self.buildStaticURL('ui/' + agent.icon),
                            id: self._containerId + '_category_' + agentCategory.category + '_agent_' + agent.id,
                            value: agent.title,
                            onClick: function () {
                                utils.openURL(agent.url);
                            }
                        });
                    categoryFieldset.addChild(agentOutput);
                });
                categoryTile.addChild(categoryFieldset);

                self._container.addChild(categoryTile);
            });

            otherAgentsTile.addChild(new Output({
                content: '<p style="margin-bottom: 20px;">${value}</p>',
                value: _('To download all agents, log in to Backup Console.'),
                id:  self._containerId + '_login_tile_output'
            }));
            otherAgentsTile.addChild(new Button({
                label: _('Go to Backup Console'),
                id:  self._containerId + '_login_tile_output_button',
                onClick: function () {
                    var self = this,
                        w = window.open('', '_blank');
                    xhr.get('/aps/2/resources/' + userId + '/loginByToken').then(function (url) {
                        w.location.href = url;
                        self.cancel();
                    }, function (error) {
                        self.cancel();
                        w.close();
                        messages.displayError(error);
                    });
                }
            }));
            self._container.addChild(otherAgentsTile);

            aps.apsc.hideLoading();
        }
    });
});
