/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "aps/xhr",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./js/common.js",
    "./js/messages.js",
    "./js/wizardData.js"
], function (
    load,
    Memory,
    xhr,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages,
    wizardData
) {
    var tenant = aps.context.vars.tenant,
        data = wizardData.getWizardData(common.getRelatedViewId('edit')),
        adminId = data && data.adminId ? data.adminId : null,
        datacenterId = data && data.datacenterId ? data.datacenterId : null,
        showDatacenters = data && data.showDatacenters ? data.showDatacenters : false;

    xhr.get(
        "/aps/2/resources/" + tenant.aps.id + "/administrators/" + adminId,
        {query: {datacenterId: datacenterId}}
    ).then(function (admin) {
        var enableOptions = [
                {value: "1", label: _("Enabled")},
                {value: "0", label: _("Disabled")}
            ],
            adminModel = getStateful({
                email: admin.email,
                firstName: admin.firstName,
                lastName: admin.lastName,
                notificationsBackup: admin.notificationsBackup,
                notificationsManagement: admin.notificationsManagement
            });
        load(["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {"id": common.genId('fieldset'), title: _("General")}, [
                ["aps/Output", {
                    id: "datacenterId",
                    label: _("Datacenter"),
                    visible: showDatacenters,
                    value: admin.datacenterName
                }],
                ["aps/Output", {
                    id: "login",
                    label: _("Login"),
                    value: admin.login
                }],
                ["aps/TextBox", {
                    id: "email",
                    label: _("Email"),
                    required: true,
                    trim: true,
                    pattern: '^[^\\s@]+@[^\\s@]+\\.[^\\s@.]+$',
                    invalidMessage: _('Please enter a valid email.'),
                    value: at(adminModel, "email")
                }],
                ["aps/TextBox", {
                    id: "firstName",
                    label: _("First name"),
                    required: false,
                    trim: true,
                    value: at(adminModel, "firstName")
                }],
                ["aps/TextBox", {
                    id: "lastName",
                    label: _("Last name"),
                    required: false,
                    trim: true,
                    value: at(adminModel, "lastName")
                }]
            ]],
            ["aps/FieldSet", {"id": common.genId('fieldset'), title: _("Notifications")}, [
                ["aps/Select", {
                    id: "notificationSelect",
                    label: _("Backup notifications"),
                    store: new Memory({data: common.getBackupNotificationOptions(), idProperty: "value"}),
                    value: at(adminModel, "notificationsBackup")
                }],
                ["aps/Select", {
                    id: "quotaSelect",
                    label: _("Business notifications"),
                    store: new Memory({data: enableOptions, idProperty: "value"}),
                    value: at(adminModel, "notificationsManagement")
                }]
            ]],
            ["aps/Output", {
                "id": common.genId('output'),
                value: _("Note: The administrator receives notifications from all customers.")
            }]
        ]]).then(function () {
            aps.app.onSubmit = function () {
                var page = registry.byId("apsPageContainer");
                if (!page.validate()) {
                    aps.apsc.cancelProcessing();
                    return;
                }
                xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/" + admin.id, {
                    method: "PUT",
                    headers: {"Content-Type": "application/json"},
                    query: {
                        datacenterId: admin.datacenterId
                    },
                    data: JSON.stringify(getPlainValue(adminModel))
                }).then(function () {
                    var data = {
                        adminId: adminId,
                        datacenterId: datacenterId,
                        showDatacenters: showDatacenters
                    };
                    if (admin.email !== adminModel.email) {
                        data.newAdminEmail = adminModel.email;
                    }
                    var viewId = common.getRelatedViewId('view');
                    wizardData.setWizardData(data, viewId);
                    aps.apsc.gotoView(viewId);
                }, messages.displayError);
            };
        });
    }, messages.displayError);
    aps.app.onCancel = function goToView() {
        var viewId = common.getRelatedViewId('view');
        wizardData.setWizardData({
            adminId: adminId,
            datacenterId: datacenterId,
            showDatacenters: showDatacenters
        }, viewId);
        aps.apsc.gotoView(viewId);
    };
});
