/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([], function() {
	var types = {
		IN_BYTES: 1,
		IN_BITS_PER_SECOND: 2
	};
	var map = {};

	function loadUomsByType(uoms, type) {
		for (var i = 0; i < uoms.names.length; ++i) {
			var prev = (i === 0) ? uoms.intial : map[uoms.names[i - 1]].value;
			map[uoms.names[i]] = {type: type, value: prev * uoms.multiplier};
		}
	}

	function fillMap() {
		var byteUoms = {multiplier: 1024, intial: 1, names: ["b", "kb", "mb", "gb", "tb", "pb", "eb" ]};
		var bitpersecUoms = {multiplier: 1000, intial: 1, names: ["bit/sec", "kbit/sec", "mbit/sec"]};

		loadUomsByType(byteUoms, types.IN_BYTES);
		loadUomsByType(bitpersecUoms, types.IN_BITS_PER_SECOND);
	}

	fillMap();

	return {
		convert: function(value, from, to) {
			var nameFrom = from.toLowerCase();
			var nameTo = to.toLowerCase();
			var uomFrom = map.hasOwnProperty(nameFrom) ? map[nameFrom] : undefined;
			var uomTo = map.hasOwnProperty(nameTo) ? map[nameTo] : undefined;

			if ((value === undefined || value === '' || value === '-1' ) || uomFrom === undefined ||
							uomTo=== undefined || uomFrom.type !== uomTo.type) {
				return undefined;
			}

			return value * uomFrom.value / uomTo.value;
		},

		convertByteToMajorityUnit: function (value) {
			var units = ["b", "kb", "mb", "gb", "tb", "pb", "eb"],
				multiplier = 1024;

			while ((value >= multiplier) && (units.length > 1))
			{
				value = value / multiplier;
				units.shift();
			}

			var decimalPlaces = 2,
				unit = units.shift();

			value = Number(value);
			value = value.toFixed(decimalPlaces) == Math.round(value) ? Math.round(value) : Number(value.toFixed(decimalPlaces));

			return {value: value, unit: unit};
		},

		getUnitName: function (unit, options, multilang, packageId) {
			switch (unit.toLowerCase()) {
				case 'b':
					return _('B', options, multilang, packageId);
				case 'kb':
					return _('KB', options, multilang, packageId);
				case 'mb':
					return _('MB', options, multilang, packageId);
				case 'gb':
					return _('GB', options, multilang, packageId);
				case 'tb':
					return _('TB', options, multilang, packageId);
				case 'pb':
					return _('PB', options, multilang, packageId);
				case 'eb':
					return _('EB', options, multilang, packageId);
			}
			return undefined;
		}
	};
});
