from common import *
import gettext
_ = gettext.gettext


def addBackupServicePlans(poa, service_template_id,
                          storage_resource_id, servers_resource_id, machines_resource_id, workstations_resource_id, app_dir):


    ret = poa.pem.packaging.listInstalledModules()
    checkRpcResponse(ret)

    if ret['result'].count('PPAB') == 0 and ret['result'].count('PBA') == 0:
        print 'The billing module is not installed. Skipping the service plans creation.'
        return

    ### set Gigabyte as measure unit for storage:
    ret = poa.bss.changeMeasurementUnit({
        'resourceID': storage_resource_id,
        'measurementUnit': 'GB'
    })
    checkRpcResponse(ret)

    ### add resource categories:
    resource_category1 = _('Acronis License')
    resource_category2 = _('Acronis Storage')

    ret = poa.bss.addResourceCategory({
        'name': resource_category1
    })
    checkRpcResponse(ret)

    ret = poa.bss.addResourceCategory({
        'name': resource_category2
    })
    checkRpcResponse(ret)

    ### add store screen
    screen_name = 'ACRONIS_BACKUP_SCREEN'
    sales_category = _('Cloud Backup')
    ret = poa.bss.addStoreScreen({
        'id': screen_name,
        'storeTemplateId': 'HOSTING',
        'title': sales_category,
        'storeScreenParams': [
            {'paramId': 'TOP_LEVEL_MENU', 'valueStr': 'ACRONIS_MENU'}
        ],
        'salesCategories': [
            {
                'name': sales_category,
                'imageFile': app_dir + 'logo.png',
                'shortDescription': sales_category
            }
        ]
    })
    checkRpcResponse(ret)

    ### add purchase scenario:
    purchase_scenario = 'ACRONIS_BACKUP_PATH'
    purchase_scenario_name = sales_category
    ret = poa.bss.addPurchaseScenario({
        'id': purchase_scenario,
        'name': purchase_scenario_name,
        'orderNum': 50,
        'imageFile':  app_dir + 'logo.png',
        'storeScreens': [screen_name, 'CHECKOUT_SCREEN']
    })
    checkRpcResponse(ret)

###########################################################
## Add service plan 1
## 100Gb and 1 workstation

    sp1_resource_rates = [
        {'resourceId': servers_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 32.00, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': machines_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 6.00, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': workstations_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 4.00, 'overusageFee': 0.00, 'included': 1.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': storage_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 0.10, 'overusageFee': 0.00, 'included': 100.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False}
    ]

    sp1_plan_periods = [
        {'period': 30, 'periodType': 1,
         'setupFee': 0.00, 'recurringFee': 0.00, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': True, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 1},

        {'period': 1, 'periodType': 2,
         'setupFee': 0.00, 'recurringFee': 6.99, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 2},

        {'period': 1, 'periodType': 3,
         'setupFee': 0.00, 'recurringFee': 5.83, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 3}
    ]

    service_plan1_name = _("Acronis Backup for PC and Mac")
    ret = poa.bss.addServicePlan({
        'name': service_plan1_name,
        'serviceTemplateId': service_template_id,
        'shortDescription': '<div class="ShortDescription">' +
                            _('<p>The Easiest, Fastest and Most Powerful Way to Backup Your PC and Mac.</p><ul><li>1 computer</li><li>100GB of cloud storage</li><li>Unlimited local storage</li></ul>') +
                            '</div>',
        'longDescription':  '<div class="ShortDescription">' +
                            _('<p>Acronis Cloud Backup Service</p><p>The Easiest, Fastest and Most Powerful Way to Backup Your PC and Mac.</p><ul><li>1 computer</li><li>100GB of cloud storage</li><li>Unlimited local storage</li></ul>') +
                            '</div>',
        'salesCategory': sales_category,
        'imageFile':  app_dir + 'logo.png',
        'published': True,
        'notificationTemplate': _("Welcome Email - Acronis"),
        'billPeriod': 1,
        'billPeriodType': 2,
        'chargingType': 30,
        'autoRenewType': 15,
        'renewOrderInterval': 3,
        'resourceRates': sp1_resource_rates,
        'planPeriods': sp1_plan_periods,
        'listingPriority': 1
    })
    checkRpcResponse(ret)

    service_plan1 = ret['result']
    print "'{0}' service plan has been created with ID #{1}".format(service_plan1_name, service_plan1)


###########################################################
## Add service plan 2    
## 500 gb and 1 server

    sp2_resource_rates = [
        {'resourceId': servers_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 32.00, 'overusageFee': 0.00, 'included': 1.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': machines_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 6.00, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': workstations_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 4.00, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': storage_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 0.10, 'overusageFee': 0.00, 'included': 500.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False}
    ]

    sp2_plan_periods = [
        {'period': 30, 'periodType': 1,
         'setupFee': 0.00, 'recurringFee': 0.00, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': True, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 1},

        {'period': 1, 'periodType': 2,
         'setupFee': 0.00, 'recurringFee': 55.00, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 2},

        {'period': 1, 'periodType': 3,
         'setupFee': 0.00, 'recurringFee': 49.99, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 3}
    ]

    service_plan2_name = _("Acronis Backup for Servers")
    ret = poa.bss.addServicePlan({
        'name': service_plan2_name,
        'serviceTemplateId': service_template_id,
        'shortDescription':  '<div class="ShortDescription">' +
                             _('<p>Protect your servers and application*</p><br><ul><li>1 Server</li><li>500GB of cloud storage</li><li>Unlimited local storage</li></ul><br><p>*Backup of Microsoft SQL, Microsoft Exchange. Application backup is counted as server backup</p>') +
                             '</div>',
        'longDescription':  '<div class="ShortDescription">' +
                            _('<p>Acronis Backup for Servers</p><br><ul><li>1 Server</li><li>500GB of cloud storage</li><li>Unlimited local storage</li></ul><br><p>*Backup of Microsoft SQL, Microsoft Exchange. Application backup is counted as server backup</p>') +
                            '</div>',
                           
        'salesCategory': sales_category,
        'imageFile':  app_dir + 'logo.png',
        'published': True,
        'notificationTemplate': _("Welcome Email - Acronis"),
        'billPeriod': 1,
        'billPeriodType': 2,
        'chargingType': 30,
        'autoRenewType': 15,
        'renewOrderInterval': 1,
        'resourceRates': sp2_resource_rates,
        'planPeriods': sp2_plan_periods,
        'listingPriority': 2
    })
    checkRpcResponse(ret)

    service_plan2 = ret['result']

    print "'{0}' service plan has been created with ID #{1}".format(service_plan2_name, service_plan2)


###########################################################
## Add service plan 3
## 250 Gb and 1 VM

    sp3_resource_rates = [
        {'resourceId': servers_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 32.00, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': machines_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 6.00, 'overusageFee': 0.00, 'included': 1.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': workstations_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 4.00, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': storage_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 0.10, 'overusageFee': 0.00, 'included': 250.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False}
    ]

    sp3_plan_periods = [
        {'period': 30, 'periodType': 1,
         'setupFee': 0.00, 'recurringFee': 0.00, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': True, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 1},

        {'period': 1, 'periodType': 2,
         'setupFee': 0.00, 'recurringFee': 12.99, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 2},

        {'period': 1, 'periodType': 3,
         'setupFee': 0.00, 'recurringFee': 10.83, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False, 'sortNumber': 3}
    ]

    service_plan3_name = _("Acronis Backup for Virtual Machines")
    ret = poa.bss.addServicePlan({
        'name': service_plan3_name,
        'serviceTemplateId': service_template_id,
        'shortDescription': '<div class="ShortDescription">' +
                            _('<p>Protect your Hyper-V and VMware ESXi virtual machines</p><ul><li>1 Virtual machine*</li><li>250GB of cloud storage</li><li>Unlimited local storage</li></ul><br><p>*Virtual machines backed up using Agent for VMware of Agent for <nobr>Hyper-V</nobr>. Virtual machines backed up using agent installed in guest OS are counted as server backup</p>') +
                            '</div>',
        'longDescription':  '<div class="ShortDescription">' +
                            _('<p>Acronis Backup for Servers</p><ul><li>1 Virtual machine*</li><li>250GB of cloud storage</li><li>Unlimited local storage</li></ul><br><p>*Virtual machines backed up using Agent for VMware of Agent for <nobr>Hyper-V</nobr>. Virtual machines backed up using agent installed in guest OS are counted as server backup</p>') +
                            '</div>',
        'salesCategory': sales_category,
        'imageFile':  app_dir + 'logo.png',
        'published': True,
        'notificationTemplate': _("Welcome Email - Acronis"),
        'billPeriod': 1,
        'billPeriodType': 2,
        'chargingType': 30,
        'autoRenewType': 15,
        'renewOrderInterval': 1,
        'resourceRates': sp3_resource_rates,
        'planPeriods': sp3_plan_periods,
        'listingPriority': 2
    })
    checkRpcResponse(ret)

    service_plan3 = ret['result']

    print "'{0}' service plan has been created with ID #{1}".format(service_plan3_name, service_plan3)


###########################################################
## Upgrade plans logic


    ret = poa.bss.addPlanUpgrade({
        'fromPlanId': service_plan1,
        'toPlanId': service_plan2
    })
    checkRpcResponse(ret)