from common import *

def addBackupServicePlans(poa, service_template_id,
                          storage_resource_id, servers_resource_id, machines_resource_id, workstations_resource_id, app_dir):

    def addCompositeStorageResource(resource_name, resource_id, multiplier):
        result = poa.bss.addCompositeResource({
            'resourceName': resource_name,
            'resourceDescription': resource_name,
            'unit': 'unit',
            'includedResources': [
                {
                    'resourceId': resource_id,
                    'multiplier': multiplier
                }
            ]
        })
        checkRpcResponse(result)

        resource_id = result['result']
        print "'{0}' resource type has been created with id: {1}".format(resource_name, resource_id)
        return resource_id

    ret = poa.pem.packaging.listInstalledModules()
    checkRpcResponse(ret)

    if ret['result'].count('PPAB') == 0 and ret['result'].count('PBA') == 0:
        print 'The billing module is not installed. Skipping the service plans creation.'
        exit()

    ret = poa.bss.changeMeasurementUnit({
        'resourceID': storage_resource_id,
        'measurementUnit': 'GB'
    })
    checkRpcResponse(ret)

    storage1tb_resource_id = addCompositeStorageResource('Cloud Backup Storage 1 TB', storage_resource_id, 1024)
    storage500gb_resource_id = addCompositeStorageResource('Cloud Backup Storage 500 GB', storage_resource_id, 512)
    storage250gb_resource_id = addCompositeStorageResource('Cloud Backup Storage 250 GB', storage_resource_id, 256)

    resource_category1 = 'Acronis License'
    resource_category2 = 'Acronis Storage'

    ret = poa.bss.addResourceCategory({
        'name': resource_category1
    })
    checkRpcResponse(ret)

    ret = poa.bss.addResourceCategory({
        'name': resource_category2
    })
    checkRpcResponse(ret)

    screen_name = 'ACRONIS_BACKUP_SCREEN'
    sales_category = 'Acronis Backup Cloud'
    ret = poa.bss.addStoreScreen({
        'id': screen_name,
        'storeTemplateId': 'HOSTING',
        'title': sales_category,
        'storeScreenParams': [
            {'paramId': 'TOP_LEVEL_MENU', 'valueStr': 'ACRONIS_MENU'}
        ],
        'salesCategories': [
            {
                'name': sales_category,
                'imageFile': app_dir + 'logo.png',
                'shortDescription': sales_category
            }
        ]
    })
    checkRpcResponse(ret)

    purchase_scenario = 'ACRONIS_BACKUP_PATH'
    purchase_scenario_name = sales_category
    ret = poa.bss.addPurchaseScenario({
        'id': purchase_scenario,
        'name': purchase_scenario_name,
        'orderNum': 30,
        'imageFile':  app_dir + 'logo.png',
        'storeScreens': [screen_name, 'CHECKOUT_SCREEN']
    })
    checkRpcResponse(ret)

    sp1_resource_rates = [
        {'resourceId': servers_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 34.99, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': machines_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 6.99, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': workstations_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 2.99, 'overusageFee': 0.00, 'included': 3.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': storage250gb_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 7.99, 'overusageFee': 0.00, 'included': 1.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': True},

        {'resourceId': storage500gb_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 14.99, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': True},

        {'resourceId': storage1tb_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 29.99, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': True}
    ]

    sp1_plan_periods = [
        {'period': 30, 'periodType': 1,
         'setupFee': 0.00, 'recurringFee': 0.00, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': True, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False},

        {'period': 1, 'periodType': 2,
         'setupFee': 0.00, 'recurringFee': 16.96, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False},

        {'period': 1, 'periodType': 3,
         'setupFee': 0.00, 'recurringFee': 14.16, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False}
    ]

    service_plan1_name = "Acronis Backup for Workstations"
    ret = poa.bss.addServicePlan({
        'name': service_plan1_name,
        'serviceTemplateId': service_template_id,
        'shortDescription': '<p>Acronis Cloud Backup Service</p>'
                            '<ul>'
                            '<li>Backup 3 Workstations</li>'
                            '<li>Includes 250GB of Storage</li>'
                            '</ul>',
        'longDescription': '<p>Acronis Cloud Backup Service</p>'
                           '<ul>'                          
                           '<li>Backup 3 Workstations</li>'
                           '<li>Includes 250GB of Storage</li>'
                           '</ul>',
        'salesCategory': sales_category,
        'imageFile':  app_dir + 'logo.png',
        'published': True,
        'notificationTemplate': "Welcome Email - Acronis",
        'billPeriod': 1,
        'billPeriodType': 2,
        'chargingType': 30,
        'autoRenewType': 15,
        'renewOrderInterval': 3,
        'resourceRates': sp1_resource_rates,
        'planPeriods': sp1_plan_periods
    })
    checkRpcResponse(ret)

    service_plan1 = ret['result']
    print "'{0}' service plan has been created with ID #{1}".format(service_plan1_name, service_plan1)

    sp2_resource_rates = [
        {'resourceId': servers_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 34.99, 'overusageFee': 0.00, 'included': 1.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': machines_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 6.99, 'overusageFee': 0.00, 'included': 5.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': workstations_resource_id, 'resourceCategory': resource_category1,
         'setupFee': 0.00, 'recurringFee': 2.99, 'overusageFee': 0.00, 'included': 5.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': False},

        {'resourceId': storage250gb_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 7.99, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': True},

        {'resourceId': storage500gb_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 14.99, 'overusageFee': 0.00, 'included': 1.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': True},

        {'resourceId': storage1tb_resource_id, 'resourceCategory': resource_category2,
         'setupFee': 0.00, 'recurringFee': 29.99, 'overusageFee': 0.00, 'included': 0.0, 'maximum': -1.0,
         'isSFperUnit': False, 'showInStore': True, 'showInCP': True, 'measurable': True}
    ]

    sp2_plan_periods = [
        {'period': 30, 'periodType': 1,
         'setupFee': 0.00, 'recurringFee': 0.00, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': True, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False},

        {'period': 1, 'periodType': 2,
         'setupFee': 0.00, 'recurringFee': 99.88, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False},

        {'period': 1, 'periodType': 3,
         'setupFee': 0.00, 'recurringFee': 83.32, 'renewalFee': 0.00, 'depositFee': 0.00, 'transferFee': 0.00,
         'trial': False, 'active': True, 'refundPeriod': 0, 'proratedRefundAfterRefundPeriod': False}
    ]

    service_plan2_name = "Acronis Backup for Servers"
    ret = poa.bss.addServicePlan({
        'name': service_plan2_name,
        'serviceTemplateId': service_template_id,
        'shortDescription': '<p>Acronis Cloud Backup Service for Servers, VMs, and Workstations</p>'
                            '<ul>'                          
                            '<li>Backup 1 Server</li>'
                            '<li>Backup 5 Virtual Machines</li>'
                            '<li>Backup 5 Workstations</li>'
                            '<li>Includes 500GB of Storage</li>'
                            '</ul>',
        'longDescription': '<p>Acronis Cloud Backup Service for Servers, VMs, and Workstations</p>'
                           '<ul>'
                           '<li>Backup 1 Server</li>'
                           '<li>Backup 5 Virtual Machines</li>'
                           '<li>Backup 5 Workstations</li>'
                           '<li>Includes 500GB of Storage</li>'
                           '</ul>',
        'salesCategory': sales_category,
        'imageFile':  app_dir + 'logo.png',
        'published': True,
        'notificationTemplate': "Welcome Email - Acronis",
        'billPeriod': 1,
        'billPeriodType': 2,
        'chargingType': 30,
        'autoRenewType': 15,
        'renewOrderInterval': 1,
        'resourceRates': sp2_resource_rates,
        'planPeriods': sp2_plan_periods
    })
    checkRpcResponse(ret)

    service_plan2 = ret['result']

    print "'{0}' service plan has been created with ID #{1}".format(service_plan2_name, service_plan2)
    ret = poa.bss.addPlanUpgrade({
        'fromPlanId': service_plan1,
        'toPlanId': service_plan2
    })
    checkRpcResponse(ret)