import sys
import time
import xmlrpclib, json, httplib, urlparse
from common import *
from billing import *

if len(sys.argv) < 4:
    print "Use the following command format: pa-resources.sh <POA host> <APS directory> <APS package name>"
    exit(-1)

POA_HOST = sys.argv[1]
APS_APACHE_DIRECTORY_NAME = sys.argv[2]
APS_PACKAGE_NAME = sys.argv[3]
PPA_FLAG = '0'
ENDPOINT_PORT = '10443'
if len(sys.argv) >= 5:
    PPA_FLAG = sys.argv[4]

if len(sys.argv) >= 6:
    APP_DIRECTORY_NAME = sys.argv[5]

HTTPS = 'https'
HTTP = 'http'

def openApiUrl():
    return "://%s:8440/RPC2" % POA_HOST

def importPkg(http_protocol=HTTPS):
    url = "%s://%s:%s/%s/%s" % (http_protocol, getLanIp(), ENDPOINT_PORT, APS_APACHE_DIRECTORY_NAME, APS_PACKAGE_NAME)
    tries = 10
    delay = 5 # sec
    iteration = 0
    while True:
        iteration += 1
        print "Importing APS package from %s, try #%s" % (url, iteration)
        if _check_url_exists(url):
            break

        if iteration < tries:
            time.sleep(delay)
        else:
            print "APS package URL location is not available. Exiting..."
            sys.exit(-1)

    return poa.pem.APS.importPackage({"package_url": url})

def addService(name, apsName, autoprovide):
    ret = poa.pem.addResourceType({"act_params": [{"var_name": "service_id", "var_value": apsName}
        , {"var_name": "app_id", "var_value": str(application_id)}
        , {"var_name": "autoprovide_service", "var_value": str(autoprovide)}]
        , "name": name, "resclass_name": "rc.saas.service"})
    checkRpcResponse(ret)
    print "'{0}' resource type has been created with id: {1}".format(name, ret['result']['resource_type_id'])
    return ret

def addCounter(name, apsName, rtClass):
    ret = poa.pem.addResourceType({"act_params": [{"var_name": "resource_id", "var_value": apsName}
        , {"var_name": "service_id", "var_value": "tenant"}
        , {"var_name": "app_id", "var_value": str(application_id)}]
        , "name": name, "resclass_name": rtClass})
    checkRpcResponse(ret)
    print "'{0}' resource type has been created with id: {1}".format(name, ret['result']['resource_type_id'])
    return ret

def activateServiceTemaplate(st_id):
    ret = poa.pem.activateST({"st_id": st_id})
    checkRpcResponse(ret)

def _check_url_exists(url):
    try:
        # python 2.x
        import httplib
        conn = httplib.HTTPConnection(str(getLanIp()), str(ENDPOINT_PORT))
        conn.request("HEAD", "/%s/%s" % (APS_APACHE_DIRECTORY_NAME, APS_PACKAGE_NAME))
        response = conn.getresponse()
    except ImportError:
        # python 3+
        import httplib2
        h = httplib2.Http()
        response, _ = h.request(str(url))

    return 200 == response.status


def addBackupProfile(backup_profile_name):
    ret = poa.pem.APS.getUserToken({"user_id": 1})
    checkRpcResponse(ret)

    aps_token = ret['result']['aps_token']
    aps_uri = ret['result']['controller_uri']

    url = urlparse.urlparse(aps_uri)
    aps = httplib.HTTPSConnection(url.netloc)
    aps.connect()
    aps.request("POST", "/aps/2/resources/{0}/storages".format(application_resource_id), json.dumps({
      "aps": {
        "type": "http://acronis.com/aps/cloud-backup-v1/backup-profile/2.0"
      },
      "name": "Default Backup Profile (Acronis)",
      "backupNotification": 3,
      "managementNotification": 0,
      "storageId": "default"
    }), {
        "APS-Token": aps_token
    })
    res = aps.getresponse()
    if res.status != 200:
        print "Can't create Default Backup Profile"
        exit(-1)

    bp = json.loads(res.read())
    bp_id = bp['aps']['id']
    print "'{0}' resource has been created with id: {1}".format(backup_profile_name, bp_id)

    aps.request("GET", "/aps/2/resources/{0}/bind_resource_type".format(bp_id), None, {"APS-Token": aps_token})
    res = aps.getresponse()
    if res.status != 200:
        print "Can't create Default Backup Profile resource type"
        exit(-1)

    aps.request("GET", "/aps/2/resources/{0}".format(bp_id), None, {"APS-Token": aps_token})
    res = aps.getresponse()
    if res.status != 200:
        print "Can't create Default Backup Profile resource type"
        exit(-1)

    bp = json.loads(res.read())
    aps.close()

    profile_resource_id = bp['resourceTypeId']
    print "'{0}' resource type has been created with id: {1}".format(backup_profile_name, profile_resource_id)
    return profile_resource_id


poa = xmlrpclib.ServerProxy("http{0}".format(openApiUrl()))
print "Install APS application..."

try:
    poa.pem.getAccountInfo({'account_id': 1})
except:
    print "Warning: OpenAPI does not accept http requests, trying https"
    poa = xmlrpclib.ServerProxy("https{0}".format(openApiUrl()))

try:
    ret = importPkg()
except:
    print "Warning: APS package not available throw https, trying http"
    ret = importPkg(HTTP)
checkRpcResponse(ret)

application_id = ret['result']['application_id']
print "Acronis Backup Cloud imported, id:", application_id

ret = poa.pem.APS.getApplicationInstances({"app_id": application_id})
checkRpcResponse(ret)

if len(ret['result']):
    print "Acronis Backup Cloud Application was successfully upgraded."
    sys.exit(0)

ret = poa.pem.APS.provideApplicationInstance({"subscription_id": 0, "rt_id": 0, "app_id": application_id,
                                              "url_path": "https://%s:10443/%s" % (getLanIp(), APS_APACHE_DIRECTORY_NAME)})
checkRpcResponse(ret)

application_instance_id = ret['result']['application_instance_id']
application_resource_id = ret['result']['application_resource_id']

print "Acronis Backup Cloud instantiated :", application_instance_id, application_resource_id

customer_resource_types = []
reseller_resource_types = []

ret = poa.pem.addResourceType({"act_params": [{"var_name": "resource_uid", "var_value": application_resource_id}
    , {"var_name": "app_id", "var_value": str(application_id)}]
    , "name": "Backup Service (Acronis)", "resclass_name": "rc.saas.service.link"})
checkRpcResponse(ret)

backup_service_app_rt_id = ret['result']['resource_type_id']
customer_resource_types.append(backup_service_app_rt_id)
reseller_resource_types.append(backup_service_app_rt_id)
print "'Backup Service Application' resource type has been created with id: ", backup_service_app_rt_id

ret = addService("Backup Tenant (Acronis)", "tenant", 1)
tenant_rt_id = ret['result']['resource_type_id']
customer_resource_types.append(tenant_rt_id)
reseller_resource_types.append(tenant_rt_id)

ret = addService("Backup Users (Acronis)", "users", 0)
customer_resource_types.append(ret['result']['resource_type_id'])

ret = addService("Reseller Branding (Acronis)", "reseller", 1)
reseller_resource_types.append(ret['result']['resource_type_id'])

ret = addCounter("Cloud Backup Storage (Acronis)", "StorageUsage", "rc.saas.resource")
storage_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(storage_resource_id)

ret = addCounter("Protected Servers (Acronis)", "ServerCount", "rc.saas.resource.unit")
servers_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(servers_resource_id)

ret = addCounter("Protected Virtual Machines (Acronis)", "VmCount", "rc.saas.resource.unit")
machines_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(machines_resource_id)

ret = addCounter("Protected Workstations (Acronis)", "WorkstationCount", "rc.saas.resource.unit")
workstations_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(workstations_resource_id)

backup_profile_id = addBackupProfile("Default Backup Profile (Acronis)")
customer_resource_types.append(backup_profile_id)

if PPA_FLAG == '1':
    ret = poa.pem.addServiceTemplate({"owner_id": 1, "name": "Acronis Backup Service",
                                      "resources": [{'resource_type_id': i} for i in customer_resource_types],
                                      "category": "http://www.acronis.com/aps/cloud-backup-v1#acronis-service-template"})
else:
    ret = poa.pem.addServiceTemplate({"owner_id": 1, "name": "Acronis Backup Service",
                                      "resources": [{'resource_type_id': i} for i in customer_resource_types]})

checkRpcResponse(ret)

user_st_id = ret['result'].get('st_id', 'Id undefined')
print "'Acronis Backup Cloud' service template has been created with id: ", user_st_id

limited = [backup_service_app_rt_id, tenant_rt_id, backup_profile_id]
unlimited = list(set(customer_resource_types) - set(limited))
unlimits = [{"resource_id": i, "resource_limit": -1} for i in unlimited]
limits = unlimits + [{"resource_id": i, "resource_limit": 1} for i in limited]
st_limits = {"st_id": user_st_id, "limits": limits}
ret = poa.pem.setSTRTLimits(st_limits)
checkRpcResponse(ret)

activateServiceTemaplate(user_st_id)

ret = poa.pem.addServiceTemplate({"owner_id": 1, "name": "Acronis Backup Cloud for Reseller Branding",
                                  "resources": [{'resource_type_id': i} for i in reseller_resource_types]})
checkRpcResponse(ret)

reseller_st_id = ret['result'].get('st_id', 'Id undefined')
print "'Acronis Backup Cloud Reseller Branding' service template has been created with id: ", reseller_st_id

limits = [{"resource_id": i, "resource_limit": 1} for i in reseller_resource_types]
st_limits = {"st_id": reseller_st_id, "limits": limits}
ret = poa.pem.setSTRTLimits(st_limits)
checkRpcResponse(ret)

activateServiceTemaplate(reseller_st_id)

if PPA_FLAG == '1':
    addBackupServicePlans(poa, user_st_id,
                          storage_resource_id, servers_resource_id, machines_resource_id, workstations_resource_id, APP_DIRECTORY_NAME)

print "Acronis Backup Cloud Application was successfully installed."
