<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . "/app/bootstrap.php");
require_once(__DIR__ . "/migration/CsvMigration.php");

chdir(__DIR__);
define('APS_RUNTIME_NO_PROCESS', true);

const OUTPUT_OPTION_UNDEFINED = "Option '%s' undefined " . PHP_EOL;

const OPTION_PARSE_CHAR = ':';
const OPTION_FILE = 'f';
const OPTION_TERMINATE_STATUS = 't';
const OPTION_CONTINUE_STATUS = 'c';
const OPTION_HELP = 'h';

$opt = getopt(
    OPTION_FILE . OPTION_PARSE_CHAR .
    OPTION_TERMINATE_STATUS . OPTION_PARSE_CHAR .
    OPTION_CONTINUE_STATUS . OPTION_PARSE_CHAR .
    OPTION_HELP
);

if (isset($opt[OPTION_HELP])) {
    showUsage();
    return;
}

$file = getOptions(OPTION_FILE, $opt);

if (!$file) {
    echo "Please specify the path to the CSV file." . PHP_EOL;
    showUsage();
    exit(1);
}

$terminateStatuses = getOptions(OPTION_TERMINATE_STATUS, $opt);
$continueStatuses = getOptions(OPTION_CONTINUE_STATUS, $opt);

try {
    $apiFactory = new ApiFactory();
    $migrationManager = new CsvMigration($apiFactory, $continueStatuses, $terminateStatuses);
    $migrationManager->migrate($file);
} catch (Exception $e ) {
    echo sprintf('Error: %s', $e->getMessage()) . PHP_EOL;
    exit(1);
}

/**
 *  Prints usage to the screen
 */
function showUsage()
{
    $fileName = basename($_SERVER['PHP_SELF']);

    echo "Usage: php {$fileName} -f [CSV_FILE]

Options:
-f, CSV file
-h, Show this help message and exit
-c, optional, continue status of upgrade resource order
-t, optional, terminate status of upgrade resource order

Example: sudo -H -u apache bash -c 'php {$fileName} -f /tmp/migration.csv' |tee /tmp/migration.log
";
    echo PHP_EOL;
}

/**
 * @param $optionName
 * @param $options
 * @return null
 */
function getOptions($optionName, $options)
{
    $option = null;

    switch ($optionName) {
        case OPTION_FILE:
            $option = isset($options[OPTION_FILE]) ? trim($options[OPTION_FILE]) : false;
            break;
        case OPTION_TERMINATE_STATUS:
            $value = (isset($options[OPTION_TERMINATE_STATUS])) ? $options[OPTION_TERMINATE_STATUS] : CsvMigration::DEFAULT_TERMINATE_ORDER_STATUS;
            $option = (is_array($value)) ? $value : array($value);
            break;
        case OPTION_CONTINUE_STATUS:
            $value = (isset($options[OPTION_CONTINUE_STATUS])) ? $options[OPTION_CONTINUE_STATUS] : CsvMigration::DEFAULT_CONTINUE_ORDER_STATUS;
            $option = (is_array($value)) ? $value : array($value);
            break;
    }

    if (is_null($option)) {
        echo sprintf(OUTPUT_OPTION_UNDEFINED, $optionName);
        showUsage();
        return false;
    }
    return $option;
}
