<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Base;

use Arr;

class ApplicationCounter
{
    const TYPE_FEATURE = 'feature';
    const TYPE_LIMIT = 'limit';
    const TYPE_USAGE = 'usage';
    const TYPE_COUNTER = 'counter';

    const UNLIMITED_VALUE = -1;

    /** @var string */
    private $applicationType;

    /** @var string */
    private $type;

    /** @var string */
    private $name;

    /** @var string */
    private $unit;

    /** @var string */
    private $offeringItemName;

    /** @var string */
    private $offeringItemUnit;

    /** @var string */
    private $infraCapability;

    /** @var string[] */
    private $tenantTypes;

    /** @var mixed */
    private $aggregationRules;

    /** @var string[] */
    private $subOfferingItems;

    /** @var bool */
    private $isInfra;

    /** @var bool */
    private $isRanged;

    /** @var bool */
    private $isFake;

    /** @var bool */
    private $isFeature;

    /** @var bool */
    private $hasLimit;

    /** @var bool */
    private $hasUsage;

    /** @var string[] */
    private $dependencies;

    public function __construct(array $data)
    {
        $this->applicationType = Arr::get($data, 'application_type');
        $this->type = Arr::get($data, 'counter_type');
        $this->name = Arr::get($data, 'counter_name');
        $this->unit = Arr::get($data, 'counter_unit');
        $this->offeringItemName = Arr::get($data, 'offering_item_name');
        $this->offeringItemUnit = Arr::get($data, 'offering_item_unit');
        $this->infraCapability = Arr::get($data, 'infra_capability');
        $this->tenantTypes = Arr::get($data, 'tenant_types', [
            \tenant::TYPE_PARTNER,
            \tenant::TYPE_CUSTOMER,
            \tenant::TYPE_RESELLER,
        ]);
        $this->aggregationRules = Arr::get($data, 'aggregation_rules', []);
        $this->subOfferingItems = Arr::get($data, 'sub_offering_items', []);
        $this->dependencies = Arr::get($data, 'depends_on', []);
        $this->isInfra = (bool)Arr::get($data, 'is_infra', false);
        $this->isRanged = (bool)Arr::get($data, 'is_ranged', false);
        $this->isFake = (bool)Arr::get($data, 'is_fake', false);

        $this->isFeature = $this->getType() === static::TYPE_FEATURE;

        $this->hasLimit = in_array($this->getType(), [
            static::TYPE_LIMIT,
            static::TYPE_COUNTER,
        ]);

        $this->hasUsage = in_array($this->getType(), [
            static::TYPE_USAGE,
            static::TYPE_COUNTER,
        ]);
    }

    /**
     * @return string
     */
    public function getApplicationType()
    {
        return $this->applicationType;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return string
     */
    public function getUnit()
    {
        return $this->unit;
    }

    /**
     * @return string
     */
    public function getOfferingItemName()
    {
        return $this->offeringItemName;
    }

    /**
     * @return string
     */
    public function getOfferingItemUnit()
    {
        return $this->offeringItemUnit;
    }

    /**
     * @return string
     */
    public function getInfraCapability()
    {
        return $this->infraCapability;
    }

    /**
     * @return string[]|null
     */
    public function getTenantTypes()
    {
        return $this->tenantTypes;
    }

    /**
     * @return mixed
     */
    public function getAggregationRules()
    {
        return $this->aggregationRules;
    }

    /**
     * @return string[]
     */
    public function getSubOfferingItems()
    {
        return $this->subOfferingItems;
    }

    /**
     * @return string[]
     */
    public function getDependencies()
    {
        return $this->dependencies;
    }

    /**
     * @return bool
     */
    public function isInfra()
    {
        return $this->isInfra;
    }

    /**
     * @return bool
     */
    public function isRanged()
    {
        return $this->isRanged;
    }

    /**
     * @return bool
     */
    public function isFake()
    {
        return $this->isFake;
    }

    /**
     * @return bool
     */
    public function isFeature()
    {
        return $this->isFeature;
    }

    /**
     * @return bool
     */
    public function hasLimit()
    {
        return $this->hasLimit;
    }

    /**
     * @return bool
     */
    public function hasUsage()
    {
        return $this->hasUsage;
    }
}