<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ConnectionChecker_HostConnection implements Model_ConnectionChecker_CheckerInterface
{
    const HOST = "host";
    const PORT = "port";

    /** @var string */
    private $_host;
    /** @var int */
    private $_port;
    /** @var Model_ConnectionChecker_CurlConnectorInterface */
    private  $_curlConnector;
    /** @var int */
    private $_errorCode = 0;
    /** @var string */
    private $_errorMessage = '';

    /**
     * @param string $host
     * @param int $port
     * @param Model_ConnectionChecker_CurlConnectorInterface $curlConnector
     */
    public function __construct($host, $port, Model_ConnectionChecker_CurlConnectorInterface $curlConnector = NULL)
    {
        $this->_host = $host;
        $this->_port = $port;
        $this->_curlConnector = is_null($curlConnector) ? new Model_ConnectionChecker_CurlConnector() : $curlConnector;
    }

    /**
     * @return bool
     */
    public function check()
    {
        $this->_errorCode = 0;
        $this->_errorMessage = '';

        try
        {
            $this->_curlConnector->tryConnectToHost($this->_host, $this->_port);
            return true;
        }
        catch (Exception $e)
        {
            $this->_errorCode = $e->getCode();
            $this->_errorMessage = $e->getMessage();
            return false;
        }
    }

    /**
     * @return array
     */
    public function getCheckDescription()
    {
        return (object) array(
            self::MESSAGE => _('Verifying connection to host __host__, port __port__'),
            self::KEYS    => (object) array(
                self::HOST => $this->_host,
                self::PORT => $this->_port
            )
        );
    }

    /**
     * @return array
     */
    public function getCheckError()
    {
        return (object) array(
            self::CODE    => $this->_errorCode,
            self::MESSAGE => $this->_errorMessage
        );
    }
}