<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_OSA_AccountAttributesManager implements Model_OSA_AccountAttributesManagerInterface
{
    /** @var Service_PbaApi_ApiInterface */
    private $_pbaApi;

    /**
     * @param Service_PbaApi_ApiInterface $pbaApi
     */
    public function __construct(Service_PbaApi_ApiInterface $pbaApi)
    {
        $this->_pbaApi = $pbaApi;
    }

    /**
     * @param int $accountNumber
     * @return int[]
     */
    public function getAccountsWithAttributeAAN($accountNumber)
    {
        return $this->_getAccountsWithAttribute(self::ATTRIBUTE_ACRONIS_ACCOUNT_NUMBER, $accountNumber);
    }

    /**
     * @param $accountId
     * @return null|string
     */
    public function getAttributeAAN($accountId)
    {
        return $this->_fetchAccountAttributeValue($accountId, self::ATTRIBUTE_ACRONIS_ACCOUNT_NUMBER);
    }

    /**
     * @param int $accountId
     * @return string
     * @throws Exception
     */
    public function getAttributeMinimalCommitmentTier($accountId)
    {
        return $this->_fetchAccountAttributeValue($accountId, self::ATTRIBUTE_MINIMAL_COMMITMENT_TIER);
    }

    /**
     * @param int $accountId
     * @param int $value
     * @return Service_PbaApi_Api_Response_UpdateObjAttrList
     */
    public function setAttributeMinimalCommitmentTier($accountId, $value)
    {
        return $this->_updateAccountAttributeValue($accountId, self::ATTRIBUTE_MINIMAL_COMMITMENT_TIER, $value);
    }

    /**
     * @param int $accountId
     * @return string
     * @throws Exception
     */
    public function getAttributeMinimalCommitmentValue($accountId)
    {
        return $this->_fetchAccountAttributeValue($accountId, self::ATTRIBUTE_MINIMAL_COMMITMENT_VALUE);
    }

    /**
     * @param int $accountId
     * @param int $value
     * @return Service_PbaApi_Api_Response_UpdateObjAttrList
     */
    public function setAttributeMinimalCommitmentValue($accountId, $value)
    {
        return $this->_updateAccountAttributeValue($accountId, self::ATTRIBUTE_MINIMAL_COMMITMENT_VALUE, $value);
    }

    /**
     * @param int $accountId
     * @return string
     * @throws Exception
     */
    public function getAttributeMinimalCommitmentMonth($accountId)
    {
        return $this->_fetchAccountAttributeValue($accountId, self::ATTRIBUTE_MINIMAL_COMMITMENT_MONTH);
    }

    /**
     * @param int $accountId
     * @param int $value
     * @return Service_PbaApi_Api_Response_UpdateObjAttrList
     */
    public function setAttributeMinimalCommitmentMonth($accountId, $value)
    {
        return $this->_updateAccountAttributeValue($accountId, self::ATTRIBUTE_MINIMAL_COMMITMENT_MONTH, $value);
    }

    /**
     * @param int $accountId
     * @return string
     */
    public function getAttributeEnableAutoSwitch($accountId)
    {
        return $this->_fetchAccountAttributeValue($accountId, self::ATTRIBUTE_ENABLE_AUTO_SWITCH);
    }

    /**
     * @param int $accountId
     * @param int $value
     * @return Service_PbaApi_Api_Response_UpdateObjAttrList
     */
    public function setAttributeEnableAutoSwitch($accountId, $value)
    {
        return $this->_updateAccountAttributeValue($accountId, self::ATTRIBUTE_ENABLE_AUTO_SWITCH, $value);
    }
    /**
     * @param int $accountId
     * @param string $attributeId
     * @return Service_PbaApi_Api_Response_Struct_ObjectAttributesListItemInterface | null
     */
    private function _fetchAccountAttributeValue($accountId, $attributeId)
    {
        $attributes = $this->_pbaApi->getObjectAttributesList(
            Service_PbaApi_Api_Request_GetObjectAttributesList::OBJECT_TYPE_ACCOUNTS,
            $accountId
        );
        foreach ($attributes as $attribute) {
            if ($attribute->getAttributeId() == $attributeId) {
                return $attribute->getValue();
            }
        }
        return null;
    }

    /**
     * @param int $accountId
     * @param string $attributeId
     * @param mixed $value
     * @return Service_PbaApi_Api_Response_UpdateObjAttrList
     */
    private function _updateAccountAttributeValue($accountId, $attributeId, $value)
    {
        $attribute = new Service_PbaApi_Api_Request_Struct_UpdateObjAttrItem();
        $attribute->setId($attributeId);
        $attribute->setValue($value);

        return $this->_pbaApi->updateObjAttrList(
            Service_PbaApi_Api_Request_GetObjectAttributesList::OBJECT_TYPE_ACCOUNTS,
            $accountId,
            array($attribute)
        );
    }

    /**
     * @param string $attributeId
     * @param mixed $attributeValue
     * @return array
     */
    private function _getAccountsWithAttribute($attributeId, $attributeValue)
    {
        $accountList = $this->_pbaApi->getAccountListByCustomAttribute($attributeId, $attributeValue);
        $accountIds = array();
        foreach ($accountList as $account) {
            $accountIds[] = $account->getAccountId();
        }
        return $accountIds;
    }
}
