<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

/**
 * Class Model_Resource_Tenant_AcronisPartner
 */
class Model_Resource_Tenant_AcronisExternal extends Model_Resource_Tenant_AcronisPartner
{
    public function provision()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        $this->checkMinimalCommitmentTiers();
        $this->linkMinimalCommitmentTier(false);

        $this->setAccountAttributeMinimalCommitmentTier();

        $this->checkProfile();
        $this->linkProfile(false);

        $this->_resolveDatacenterAndStorageForPartner();
        $this->linkDatacenter(false);

        $this->updateCounterLimits();

        $tenant = $this->tenant;
        $subscriptionParametersManager = $this->getSubscriptionParametersManager();
        $tenant->wsDiscount = $subscriptionParametersManager->getParameterWSDiscount($tenant->subscriptionId);
        $groupId = $subscriptionParametersManager->getParameterGroupId($tenant->subscriptionId);
        if (empty($groupId)) {
            throw new Exception('Please specify the parameter "GroupId" to bind it with the subscription.');
        }
        $this->checkGroupCanBeAssignedOrEx($groupId, $tenant->datacenterId);
        $tenant->mspGroupId = $groupId;
        $this->_retrieveUsageForPartner();
    }

    public function provisionAsync()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        $this->updateStartProductionTime();

        $this->_subscribeToOnSubscriptionChangeEvent();
    }

    public function retrieve()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        $tenant = $this->tenant;
        $realLink = !$tenant->subscription->disabled;
        $this->linkMinimalCommitmentTier($realLink);
        $this->linkProfile($realLink);
        $this->linkDatacenter($realLink);

        if (is_null($tenant->mspGroupId)) {
            return;
        }
        $this->_retrieveUsageForPartner();
    }

    public function configure($new)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        // do nothing
    }

    public function unprovision()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        // do nothing
    }

    public function enable()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        // do nothing
    }

    public function disable()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        // do nothing
    }

    public function upgrade()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        $tenant = $this->tenant;
        if (is_null($tenant->wsDiscount)) {
            $tenant->wsDiscount = false;
        }

        $this->updateStartProductionTime();
        $this->updateCounterLimits();
        $this->update();

        $this->_subscribeToOnSubscriptionChangeEvent();

        $realLink = !$tenant->subscription->disabled;
        if ($realLink) {
            $this->checkProfile();
        }

        $this->linkMinimalCommitmentTier($realLink);
        $this->linkProfile($realLink);
        $this->linkDatacenter($realLink);
    }

    public function onTenantAvailable($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        $this->linkMinimalCommitmentTier();
        $this->linkProfile();
        $this->linkDatacenter();
    }

    public function onSubscriptionChange($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        $tenant = $this->tenant;

        $oldMinimalCommitmentTierId = null;
        if (!$tenant->minimalCommitmentTier->isEmpty()) {
             $oldMinimalCommitmentTierId = $tenant->minimalCommitmentTier->aps->id;
        }

        $realLink = !$tenant->subscription->disabled;
        $this->linkMinimalCommitmentTier($realLink);

        $newMinimalCommitmentTierId = null;
        if (!$tenant->minimalCommitmentTier->isEmpty()) {
            $newMinimalCommitmentTierId = $tenant->minimalCommitmentTier->aps->id;
        }

        if ($newMinimalCommitmentTierId && $oldMinimalCommitmentTierId != $newMinimalCommitmentTierId) {
            $this->setAccountAttributeMinimalCommitmentTier();
        }
    }

    public function onUserChanged($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        // do nothing
    }

    public function onAccountChange($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        // do nothing
    }

    public function getAdministrators()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function updateAdministrator($adminId, $parameters, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function resendAdministratorLink($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function enableAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function deleteAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function disableAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getStorages()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getDatacenter()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function loginExists($login, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getAgents($language)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function updateUsageAndLimits()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function createAdministrator($parameters, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getAdministratorConsole($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function adminUrlRedirect()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getOptionalCounters()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getDatacenters()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getServicePlans()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getServicePlan($planId)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function setStorageForPlan($planId, $parameters)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function unsetStorageForPlan($planId)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getQuotasStatistics()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }

    public function getUserGuideUrl(){
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));
        $this->throwNotApplicableMethodException();
    }
}