<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ScreenNotification_Builder
{
    /**
     * @var string
     */
    private $_id;

    /**
     * @var string
     */
    private $_type;

    /**
     * @var string
     */
    private $_status;

    /**
     * @var Model_ScreenNotification_Struct_LocalizedMessage
     */
    private $_message;

    /**
     * @var Model_ScreenNotification_Struct_LocalizedMessage
     */
    private $_details;

    /**
     * @var Model_ScreenNotification_Struct_LocalizedMessage
     */
    private $_fullDetails;

    /**
     * @var string
     */
    private $_link;

    /**
     * @var string
     */
    private $_linkMore;

    /**
     * @var Model_ScreenNotification_Struct_LocalizedMessage
     */
    private $_linkMoreText;

    /**
     * @var string
     */
    private $_packageId;

    /**
     * @var string
     */
    private $_accountId;

    /**
     * @var string
     */
    private $_userId;

    /**
     * @var string
     */
    private $_initiatorId;

    /**
     * @var string
     */
    private $_service;

    /**
     * @var string
     */
    private $_parentId;


    /**
     * @return Model_ScreenNotification_Struct_Message
     */
    public function getNotification()
    {
        $notification = new Model_ScreenNotification_Struct_Message();

        $notification->setId($this->_id);

        $notification->setPackageId($this->_packageId);
        $notification->setAccountId($this->_accountId);
        $notification->setUserId($this->_userId);
        $notification->setInitiatorId($this->_initiatorId);
        $notification->setService($this->_service);
        $notification->setParentId($this->_parentId);

        $notification->setType($this->_type);
        $notification->setStatus($this->_status);
        $notification->setMessage($this->_message);
        $notification->setDetails($this->_details);
        $notification->setFullDetails($this->_fullDetails);
        $notification->setLink($this->_link);
        $notification->setLinkMore($this->_linkMore);
        $notification->setLinkMoreText($this->_linkMoreText);

        return $notification;
    }

    /**
     * @param string $id
     * @return $this
     */
    public function setId($id)
    {
        $this->_id = $id;
        return $this;
    }

    /**
     * @param string $packageId
     * @return $this
     */
    public function setPackageId($packageId)
    {
        $this->_packageId = $packageId;
        return $this;
    }

    /**
     * @param string $accountId
     * @return $this
     */
    public function setAccountId($accountId)
    {
        $this->_accountId = $accountId;
        return $this;
    }

    /**
     * @param string $userId
     * @return $this
     */
    public function setUserId($userId)
    {
        $this->_userId = $userId;
        return $this;
    }

    /**
     * @param string $initiatorId
     * @return $this
     */
    public function setInitiatorId($initiatorId)
    {
        $this->_initiatorId = $initiatorId;
        return $this;
    }

    /**
     * @param string $service
     * @return $this
     */
    public function setService($service)
    {
        $this->_service = $service;
        return $this;
    }

    /**
     * @param string $parentId
     * @return $this
     */
    public function setParentId($parentId)
    {
        $this->_parentId = $parentId;
        return $this;
    }

    /**
     * @param string $type
     * @return $this
     */
    public function setType($type)
    {
        $this->_type = $type;
        return $this;
    }

    /**
     * @param string $status
     * @return $this
     */
    public function setStatus($status)
    {
        $this->_status = $status;
        return $this;
    }

    /**
     * @param string $message
     * @param array|null $keys
     * @return $this
     */
    public function setMessage($message, array $keys = null)
    {
        $localizedMessage = new Model_ScreenNotification_Struct_LocalizedMessage();
        $localizedMessage->setMessage($message);
        $localizedMessage->setKeys($keys);

        $this->_message = $localizedMessage;
        return $this;
    }

    /**
     * @param string $message
     * @param array|null $keys
     * @return $this
     */
    public function setDetails($message, array $keys = null)
    {
        $localizedMessage = new Model_ScreenNotification_Struct_LocalizedMessage();
        $localizedMessage->setMessage($message);
        $localizedMessage->setKeys($keys);

        $this->_details = $localizedMessage;
        return $this;
    }

    /**
     * @param string $message
     * @param array|null $keys
     * @return $this
     */
    public function setFullDetails($message, array $keys = null)
    {
        $localizedMessage = new Model_ScreenNotification_Struct_LocalizedMessage();
        $localizedMessage->setMessage($message);
        $localizedMessage->setKeys($keys);

        $this->_fullDetails = $localizedMessage;
        return $this;
    }

    /**
     * @param string $link
     * @return $this
     */
    public function setLink($link)
    {
        $this->_link = $link;
        return $this;
    }

    /**
     * @param string $link
     * @param string|null $message
     * @param array|null $keys
     * @return $this
     */
    public function setLinkMore($link, $message = null, array $keys = null)
    {
        $this->_linkMore = $link;
        if (!is_null($message))
        {
            $this->setLinkMoreText($message, $keys);
        }
        return $this;
    }

    /**
     * @param string|null $message
     * @param array|null $keys
     * @return $this
     */
    public function setLinkMoreText($message, array $keys = null)
    {
        $localizedMessage = new Model_ScreenNotification_Struct_LocalizedMessage();
        $localizedMessage->setMessage($message);
        $localizedMessage->setKeys($keys);
        $this->_linkMoreText = $localizedMessage;
        return $this;
    }
}