<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

interface Service_APSApi_ApiInterface
{
    /**
     * @param array $subscriptionResources
     * @throws Exception
     * @return \APS\ResourceBase
     */
    public function getProfile(array $subscriptionResources);

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $tenant
     * @throws Exception
     * @return \APS\ResourceBase|NULL
     */
    public function getAdmin($tenant);

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy $base
     * @param string $staffMemberId
     * @return \APS\ResourceBase|NULL
     */
    public function getStaffMember($base, $staffMemberId);

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $base
     * @param \APS\ResourceBase|\APS\ResourceProxy $account
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getStaffMembers($base, $account);

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy $tenant
     * @param string $staffMemberId
     * @return \APS\ResourceBase|NULL
     */
    public function getBackupAdministrator($tenant, $staffMemberId);

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $base
     * @param string | array $status
     * @param string | array | null $excludeId
     * @return \APS\ResourceBase[]
     */
    public function getPackageResourcesByStatusNot($base, $status, $excludeId = null);

    /**
     * @return \APS\ResourceBase[]
     * @throws Exception|\Rest\RestException
     */
    public function getServiceTemplatesWithLimits();

    /**
     * @param int $accountId
     * @param string|null $excludeTenant
     * @return \APS\ResourceBase[]
     */
    public function getTenantsByAccountId($accountId, $excludeTenant = null);

    /**
     * Receive tenants linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getTenantsForGlobal($global);

    /**
     * Receive profiles linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getProfilesForGlobal($global);

    /**
     * Receive profiles linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getMinimalCommitmentTiersForGlobal($global);

    /**
     * Receive datacenters linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getDatacentersForGlobal($global);

    /**
     * Receive users linked with tenant
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $tenant
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getUsersForTenant($tenant);

    /**
     * Receive admins linked with tenant
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $tenant
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getAdminsForTenant($tenant);

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $globals
     */
    public function makeDefaultProfile($globals);

    /**
     * @param int $groupId
     * @param int $datacenterId
     * @throws \Rest\RestException
     */
    public function checkGroupCanBeAssignedOrEx($groupId, $datacenterId = null);

    /**
     * @param string $rql
     * @param $base
     * @return mixed
     */
    public function countResources($rql, $base = null);

    /**
     * @param $accountId
     * @return \APS\ResourceBase[]
     */
    public function getResellersByAccountId($accountId);

    /**
     * @param $groupId
     * @param null $datacenterId
     * @return array|null
     */
    public function getTenantsByGroupIdAndDatacenterId($groupId, $datacenterId = null);

    /**
     * @param \APS\ResourceBase $account
     * @return \APS\ResourceBase[]
     */
    public function getChildAccounts($account);

    /**
     * @param \APS\ResourceBase $account
     * @return \APS\ResourceBase[]
     */
    public function getAccountSubscriptions($account);
}
