<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

class Service_AmsApiClient_Api extends Service_BaseClient_ApiAbstract implements Service_AmsApiClient_ApiInterface
{
    /**
     * @return array
     */
    public function getBackupItems()
    {
        $backupPlans = $this->_call(
            new Service_AmsApiClient_Request_BackupPlans()
        );

        return $backupPlans ? $backupPlans : [];
    }

    /**
     * @param string $accessToken
     * @return array
     */
    public function createRemoteConnection($accessToken)
    {
        $remoteConnection = new Service_AmsApiClient_Request_Struct_RemoteConnection();
        $remoteConnection->setAccessToken($accessToken);

        return $this->_call(
            new Service_AmsApiClient_Request_RemoteConnection($remoteConnection)
        );
    }

    /**
     * @param string $language
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    public function getAgents($language)
    {
        $links = $this->getLinks();
        $agents = $links->getAgents();

        return $agents;
    }

    /**
     * @return Service_AmsApiClient_Response_Struct_LinksList
     */
    public function getLinks()
    {
        return $this->_call(new Service_AmsApiClient_Request_GetLinksList());
    }

    /**
     * @param string $token
     * @return Service_AmsApiClient_Response_Struct_PersonifiedLinks
     * @throws Exception
     */
    public function getPersonifiedLinks($token)
    {
        $apiBaseUrl = $this->getApiBaseUrl();
        $transport = $this->createTransport();

        // Logging into MC
        $loginRequest = new Service_AmsApiClient_Request_Login($token);
        $loginResponseBody = $this->executeTransportRequest($transport, $apiBaseUrl . $loginRequest->getResource());
        $loginResponse = new Service_AmsApiClient_Response_Login($loginResponseBody);
        /** @var Service_AmsApiClient_Response_Struct_Login $loginResponseData */
        $loginResponseData = $loginResponse->getData();
        $bcUrl = $loginResponseData->getBackupUrl();

        // Logging into BC
        $this->executeTransportRequest($transport, $bcUrl, Service_BaseClient_Request_Base::METHOD_GET, null);

        // Getting links list
        $personifiedLinksRequest = new Service_AmsApiClient_Request_PersonifiedLinks();
        $linksResponseBody = $this->executeTransportRequest(
            $transport,
            $this->getRestClient()->getApiUri() . $personifiedLinksRequest->getResource()
        );

        return (new Service_AmsApiClient_Response_PersonifiedLinks($linksResponseBody))->getData();
    }

    /**
     * @return Service_BaseClient_RestTransport
     */
    private function createTransport()
    {
        $transport = new Service_BaseClient_RestTransport();
        $sendingStrategy = new Service_BaseClient_TransportStrategy_Json();
        $transport->setSendingStrategy($sendingStrategy);
        $transport->setCookieFile(true);

        return $transport;
    }

    /**
     * @param Service_BaseClient_RestTransport $transport
     * @param string $url
     * @param string $method
     * @param mixed $requestBody
     * @return string
     * @throws Exception
     */
    private function executeTransportRequest(
        Service_BaseClient_RestTransport $transport,
        $url,
        $method = Service_BaseClient_Request_Base::METHOD_GET,
        $requestBody = null
    ) {
        list($statusCode, $responseBody) = $transport->execute($url, $method, $requestBody);

        if ($statusCode < Service_BaseClient_RestTransport::HTTP_CODE_SUCCESS ||
            $statusCode >= Service_BaseClient_RestTransport::HTTP_CODE_MULTIPLE_CHOICE) {
            throw new Service_BaseClient_HTTPException(Str::format(
                'Failed request to "%s", response body: "%s".',
                $url, $responseBody
            ), $statusCode);
        }

        return $responseBody;
    }

    private function getApiBaseUrl()
    {
        $urlParts = parse_url($this->getRestClient()->getApiUri());

        return Str::format(
            '%s://%s%s',
            $urlParts['scheme'], $urlParts['host'], isset($urlParts['port']) ? ':' . $urlParts['port'] : ''
        );
    }
}
