<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_AmsApiClient_Api extends Service_BaseClient_ApiAbstract implements Service_AmsApiClient_ApiInterface
{
    const LANGUAGE_DEFAULT = 'en';
    const LANGUAGE_MULTI = 'multi';
    const ARCHITECTURE_64 = '64';
    const SYSTEM_WINDOWS = 'windows';
    const SYSTEM_LINUX = 'linux';
    const SYSTEM_APPLE = 'apple';
    const SYSTEM_ANDROID = 'android';
    const SYSTEM_IOS = 'ios';
    const SYSTEM_HYPERV = 'hyperv';
    const SYSTEM_VMWARE = 'vmware';
    const SYSTEM_VIRTUOZZO = 'pcs';
    const SYSTEM_MICROSOFT_SQL_SERVER = 'sql';
    const SYSTEM_MICROSOFT_EXCHANGE_SERVER = 'arx';
    const SYSTEM_MICROSOFT_ACTIVE_DIRECTORY = 'ad';
    const SYSTEM_MICROSOFT_OFFICE_365 = 'office';
    const TYPE_CONSUMER = 'consumer';
    const TYPE_ENTERPRISE = 'enterprise';
    const WEB_TRUE = 'true';

    /**
     * @return array
     */
    public function getBackupItems()
    {
        $backupPlans = $this->_call(
            new Service_AmsApiClient_Request_BackupPlans()
        );
        return $backupPlans ? $backupPlans : array();
    }

    /**
     * @param string $accessToken
     * @return array
     */
    public function createRemoteConnection($accessToken)
    {
        $remoteConnection = new Service_AmsApiClient_Request_Struct_RemoteConnection();
        $remoteConnection->setAccessToken($accessToken);
        return $this->_call(
            new Service_AmsApiClient_Request_RemoteConnection($remoteConnection)
        );
    }

    /**
     * @param string $language
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    public function getAgents($language)
    {
        $links = $this->getLinks();
        $agents = $links->getAgents();
        
        $clients =  array();

        $agent = $this->_getAgent($agents, self::SYSTEM_WINDOWS, $language);
        $clients[self::SYSTEM_WINDOWS] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_LINUX, $language);
        $clients[self::SYSTEM_LINUX] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_APPLE, $language);
        $clients[self::SYSTEM_APPLE] = $agent;
        
        $agent = $this->_getAgent($agents, self::SYSTEM_ANDROID, $language);
        $clients[self::SYSTEM_ANDROID] = $agent;
        
        $agent = $this->_getAgent($agents, self::SYSTEM_IOS, $language);
        $clients[self::SYSTEM_IOS] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_VMWARE, $language);
        $clients[self::SYSTEM_VMWARE] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_HYPERV, $language);
        $clients[self::SYSTEM_HYPERV] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_VIRTUOZZO, $language);
        $clients[self::SYSTEM_VIRTUOZZO] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_MICROSOFT_SQL_SERVER, $language);
        $clients[self::SYSTEM_MICROSOFT_SQL_SERVER] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_MICROSOFT_EXCHANGE_SERVER, $language);
        $clients[self::SYSTEM_MICROSOFT_EXCHANGE_SERVER] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_MICROSOFT_ACTIVE_DIRECTORY, $language);
        $clients[self::SYSTEM_MICROSOFT_ACTIVE_DIRECTORY] = $agent;

        $agent = $this->_getAgent($agents, self::SYSTEM_MICROSOFT_OFFICE_365, $language);
        $clients[self::SYSTEM_MICROSOFT_OFFICE_365] = $agent;
        
        return $clients;
    }

    /**
     * @return Service_AmsApiClient_Response_Struct_LinksList
     */
    public function getLinks()
    {
        return $this->_call(new Service_AmsApiClient_Request_GetLinksList());
    }

    /**
     * @param Service_AmsApiClient_Response_Struct_LinksListAgentsItem[] $agents
     * @param string $platform
     * @param string $language
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem|null
     */
    private function _getAgent(array $agents, $platform, $language = self::LANGUAGE_DEFAULT)
    {
        $agentsFilteredBySystem = $this->_filterAgentsBySystem($agents, $platform);
        if (empty($agentsFilteredBySystem))
        {
            return NULL;
        }

        if ($platform === self::SYSTEM_IOS || $platform === self::SYSTEM_ANDROID)
        {
            $agentsFilteredBySystem = $this->_filterAgentsByType($agentsFilteredBySystem, self::TYPE_CONSUMER);
        }
        else
        {
            $agentsFilteredBySystem = $this->_filterAgentsByType($agentsFilteredBySystem, self::TYPE_ENTERPRISE);
        }
        if (empty($agentsFilteredBySystem))
        {
            return NULL;
        }

        if ($platform === self::SYSTEM_WINDOWS)
        {
            $agentsFilteredBySystem = $this->_filterAgentsByWeb($agentsFilteredBySystem, self::WEB_TRUE);
        }
        elseif ($platform === self::SYSTEM_LINUX || $platform === self::SYSTEM_APPLE)
        {
            $agentsFilteredBySystem = $this->_filterAgentsByArchitecture($agentsFilteredBySystem, self::ARCHITECTURE_64);
        }

        $agentsFilteredByLanguage = $this->_filterAgentsByLanguage($agentsFilteredBySystem, self::LANGUAGE_MULTI);
        if (empty($agentsFilteredByLanguage))
        {
            $agentsFilteredByLanguage = $this->_filterAgentsByLanguage($agentsFilteredBySystem, $language);
            if (empty($agentsFilteredByLanguage))
            {
                $agentsFilteredByLanguage = $this->_filterAgentsByLanguage($agentsFilteredBySystem, self::LANGUAGE_DEFAULT);
                if (empty($agentsFilteredByLanguage))
                {
                    return NULL;
                }
            }
        }

        return reset($agentsFilteredByLanguage);
    }

    /**
     * @param Service_AmsApiClient_Response_Struct_LinksListAgentsItem[] $agents
     * @param string $value
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    private function _filterAgentsBySystem(array $agents, $value)
    {
        $result = array();
        foreach($agents as $agent)
        {
            if ($agent->getSystem() === $value)
            {
                $result[] = $agent;
            }
        }
        return $result;
    }

    /**
     * @param Service_AmsApiClient_Response_Struct_LinksListAgentsItem[] $agents
     * @param string $value
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    private function _filterAgentsByType(array $agents, $value)
    {
        $result = array();
        foreach($agents as $agent)
        {
            if ($agent->getType() === $value)
            {
                $result[] = $agent;
            }
        }
        return $result;
    }

    /**
     * @param Service_AmsApiClient_Response_Struct_LinksListAgentsItem[] $agents
     * @param string $value
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    private function _filterAgentsByWeb(array $agents, $value)
    {
        $result = array();
        foreach($agents as $agent)
        {
            if ($agent->getWeb() === $value)
            {
                $result[] = $agent;
            }
        }
        return $result;
    }

    /**
     * @param Service_AmsApiClient_Response_Struct_LinksListAgentsItem[] $agents
     * @param string $value
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    private function _filterAgentsByArchitecture(array $agents, $value)
    {
        $result = array();
        foreach($agents as $agent)
        {
            if ($agent->getArchitecture() === $value)
            {
                $result[] = $agent;
            }
        }
        return $result;
    }

    /**
     * @param Service_AmsApiClient_Response_Struct_LinksListAgentsItem[] $agents
     * @param string $value
     * @return Service_AmsApiClient_Response_Struct_LinksListAgentsItem[]
     */
    private function _filterAgentsByLanguage(array $agents, $value)
    {
        $result = array();
        foreach($agents as $agent)
        {
            if ($agent->getLanguage() === $value)
            {
                $result[] = $agent;
            }

        }
        return $result;
    }
}
