<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_BaseClient_HTTPException extends Exception
{
    const HTTP_UNAUTHORIZED_CODE = 401;
    const HTTP_NOT_FOUND_CODE = 404;
    const HTTP_CODE_VERSION_EXPIRED = 426;
    const HTTP_CODE_CONFLICT = 409;

    /**
     * @var array|null
     */
    private $_headers;

    /**
     * Service_BaseClient_HTTPException constructor.
     * @param string $message
     * @param int $code
     * @param Exception $previous
     * @param array $headers
     */
    public function __construct($message = "", $code = 0, Exception $previous = null, $headers = null)
    {
        $this->_headers = $headers;
        parent::__construct($message, $code, $previous);
    }

    public function getHeaders()
    {
        return $this->_headers;
    }

    /**
     * @param string $key
     * @return null|string|array
     */
    public function getHeader($key)
    {
        if (!is_array($this->_headers))
        {
            return NULL;
        }

        if (!isset($this->_headers[$key]))
        {
            return NULL;
        }

        return $this->_headers[$key];
    }
}