<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_ConfigEditor
{
    const PROTOCOL_PATTERN = "/('OPEN_API_PROTOCOL', ')(http|https)('\);)/";
    const PROTOCOL_REPLACEMENT_PATTERN = '${1}%s${3}';
    const CONFIG_NAME = 'config.php';
    const CONFIG_PATH_TEMPLATE = '{application_dir}{config_name}';

    /** @var  Service_FileInterface */
    private $_configFile;

    /**
     * @param Service_FileInterface $configFile
     */
    public function __construct(Service_FileInterface $configFile = NULL)
    {
        $configPath = strtr(
            self::CONFIG_PATH_TEMPLATE,
            array('{application_dir}' => DIR_APP, '{config_name}' => self::CONFIG_NAME)
        );
        $this->_configFile = is_null($configFile) ? new Service_File_Local($configPath) : $configFile;
    }

    /**
     * @param string $protocol
     */
    public function updateOpenApiProtocol($protocol)
    {
        $configContent = $this->_configFile->getContents();
        $newConfigContent = preg_replace(
            self::PROTOCOL_PATTERN,
            sprintf(self::PROTOCOL_REPLACEMENT_PATTERN, $protocol),
            $configContent
        );
        $this->_configFile->putContents($newConfigContent);
    }

    /**
     * @return Service_FileInterface
     */
    public function getConfigFile()
    {
        return $this->_configFile;
    }
} 