<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
class Service_MspApiClient_Request_Struct_GroupPrivileges
    extends Service_BaseClient_Request_Struct_Updatable
{
    const FIELD_WEBSITE_COUNT = 'website_count';
    const FIELD_MOBILE_COUNT = "mobile_count";
    const FIELD_MAILBOX_COUNT = "mailbox_count";
    const FIELD_WORKSTATION_COUNT = "workstation_count";
    const FIELD_SERVER_COUNT = "server_count";
    const FIELD_VM_COUNT = "vm_count";
    const FIELD_STORAGE_SIZE = "storage_size";

    const FIELD_LOCAL_BACKUP = "local_backup";
    const FIELD_CLOUD_TO_CLOUD_BACKUP = "cloud_to_cloud_backup";

    const FIELD_QUOTA_MOBILES_HARD = "quota_mobiles_hard";
    const FIELD_QUOTA_MAILBOX_HARD = "quota_mailboxes_hard";
    const FIELD_QUOTA_WEBSITES_HARD = "quota_websites_hard";
    const FIELD_QUOTA_WORKSTATIONS_HARD = "quota_workstations_hard";
    const FIELD_QUOTA_SERVERS_HARD = "quota_servers_hard";
    const FIELD_QUOTA_VMS_HARD = "quota_vms_hard";
    const FIELD_QUOTA_STORAGE_HARD = "quota_storage_hard";
    const FIELD_QUOTA_MAILBOX_INSTANCE_STORAGE = "quota_mailbox_instance_storage";

    const MSP_UNLIMITED = -1;

    /** @var  int */
    private $_websiteCount = self::MSP_UNLIMITED;
    /** @var  int */
    private $_mobileCount = self::MSP_UNLIMITED;
    /** @var  int */
    private $_mailboxCount = self::MSP_UNLIMITED;
    /** @var  int */
    private $_workstationCount = self::MSP_UNLIMITED;
    /** @var  int */
    private $_serverCount = self::MSP_UNLIMITED;
    /** @var  int */
    private $_vmCount = self::MSP_UNLIMITED;
    /** @var  int */
    private $_storageSize = self::MSP_UNLIMITED;

    /** @var  int */
    private $_mailboxInstanceStorageSize;

    /** @var Service_MspApiClient_Request_Struct_LocalBackup */
    private $_localBackup;

    /** @var  int */
    private $_cloudToCloudBackup;

    public function __construct(array $updateMap = array())
    {
        parent::__construct($updateMap);
        $this->setLocalBackup(new Service_MspApiClient_Request_Struct_LocalBackup());
    }

    /**
     * @return array
     */
    public function toArray()
    {
        //TODO - separate hard/soft quotas
        $result = array();

        $count = $this->getStorageSize();
        if (!is_null($count)) {
            $result[self::FIELD_STORAGE_SIZE] = $count;
            $result[self::FIELD_QUOTA_STORAGE_HARD] = $count;
        }

        $count = $this->getServerCount();
        if (!is_null($count)) {
            $result[self::FIELD_SERVER_COUNT] = $count;
            $result[self::FIELD_QUOTA_SERVERS_HARD] = $count;
        }

        $count = $this->getVmCount();
        if (!is_null($count)) {
            $result[self::FIELD_VM_COUNT] = $count;
            $result[self::FIELD_QUOTA_VMS_HARD] = $count;
        }

        $count = $this->getWorkstationCount();
        if (!is_null($count)) {
            $result[self::FIELD_WORKSTATION_COUNT] = $count;
            $result[self::FIELD_QUOTA_WORKSTATIONS_HARD] = $count;
        }

        $count = $this->getMobileCount();
        if (!is_null($count)) {
            $result[self::FIELD_MOBILE_COUNT] = $count;
            $result[self::FIELD_QUOTA_MOBILES_HARD] = $count;
        }

        $count = $this->getMailboxCount();
        if (!is_null($count)) {
            $result[self::FIELD_MAILBOX_COUNT] = $count;
            $result[self::FIELD_QUOTA_MAILBOX_HARD] = $count;
        }

        $count = $this->getWebsiteCount();
        if (!is_null($count)) {
            $result[self::FIELD_WEBSITE_COUNT] = $count;
            $result[self::FIELD_QUOTA_WEBSITES_HARD] = $count;
        }

        $count = $this->getMailboxInstanceStorageSize();
        if (!is_null($count)) {
            $result[self::FIELD_QUOTA_MAILBOX_INSTANCE_STORAGE] = $count;
        }

        $result[self::FIELD_LOCAL_BACKUP] = $this->getLocalBackup()->toArray();

        $count = $this->getCloudToCloudBackup();
        if (!is_null($count)) {
            $result[self::FIELD_CLOUD_TO_CLOUD_BACKUP] = $count;
        }

        return $result;
    }

    /**
     * @param int $serverCount
     */
    public function setServerCount($serverCount)
    {
        $this->_markFieldAsUpdated(self::FIELD_SERVER_COUNT);
        $this->_serverCount = $serverCount;
    }

    /**
     * @return int
     */
    public function getServerCount()
    {
        return $this->_serverCount;
    }

    /**
     * @param int $storageSize
     */
    public function setStorageSize($storageSize)
    {
        $this->_markFieldAsUpdated(self::FIELD_STORAGE_SIZE);
        $this->_storageSize = $storageSize;
    }

    /**
     * @return int
     */
    public function getStorageSize()
    {
        return $this->_storageSize;
    }

    /**
     * @param int $vmCount
     */
    public function setVmCount($vmCount)
    {
        $this->_markFieldAsUpdated(self::FIELD_VM_COUNT);
        $this->_vmCount = $vmCount;
    }

    /**
     * @return int
     */
    public function getVmCount()
    {
        return $this->_vmCount;
    }

    /**
     * @param int $workstationCount
     */
    public function setWorkstationCount($workstationCount)
    {
        $this->_markFieldAsUpdated(self::FIELD_WORKSTATION_COUNT);
        $this->_workstationCount = $workstationCount;
    }

    /**
     * @return int
     */
    public function getWorkstationCount()
    {
        return $this->_workstationCount;
    }

    /**
     * @return int
     */
    public function getMobileCount()
    {
        return $this->_mobileCount;
    }

    /**
     * @param int $mobileCount
     */
    public function setMobileCount($mobileCount)
    {
        $this->_markFieldAsUpdated(self::FIELD_MOBILE_COUNT);
        $this->_mobileCount = $mobileCount;
    }

    /**
     * @return int
     */
    public function getMailboxCount()
    {
        return $this->_mailboxCount;
    }

    /**
     * @param int $mailboxCount
     */
    public function setMailboxCount($mailboxCount)
    {
        $this->_markFieldAsUpdated(self::FIELD_MAILBOX_COUNT);
        $this->_mailboxCount = $mailboxCount;
    }

    /**
     * @return int
     */
    public function getWebsiteCount()
    {
        return $this->_websiteCount;
    }

    /**
     * @param int $websiteCount
     */
    public function setWebsiteCount($websiteCount)
    {
        $this->_markFieldAsUpdated(self::FIELD_WEBSITE_COUNT);
        $this->_websiteCount = $websiteCount;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_LocalBackup
     */
    public function getLocalBackup()
    {
        return $this->_localBackup;
    }

    /**
     * @param  $localBackup
     */
    public function setLocalBackup($localBackup)
    {
        $this->_markFieldAsUpdated(self::FIELD_LOCAL_BACKUP);
        $this->_localBackup = $localBackup;
    }

    /**
     * @return int
     */
    public function getMailboxInstanceStorageSize()
    {
        return $this->_mailboxInstanceStorageSize;
    }

    /**
     * @param int $mailboxStorageSize
     */
    public function setMailboxInstanceStorageSize($mailboxStorageSize)
    {
        $this->_markFieldAsUpdated(self::FIELD_QUOTA_MAILBOX_INSTANCE_STORAGE);
        $this->_mailboxInstanceStorageSize = $mailboxStorageSize;
    }

    /**
     * @return int
     */
    public function getCloudToCloudBackup()
    {
        return $this->_cloudToCloudBackup;
    }

    /**
     * @param int $cloudToCloudBackup
     */
    public function setCloudToCloudBackup($cloudToCloudBackup)
    {
        $this->_markFieldAsUpdated(self::FIELD_CLOUD_TO_CLOUD_BACKUP);
        $this->_cloudToCloudBackup = $cloudToCloudBackup;
    }
}
