<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_MspApiClient_Response_Storages extends Service_BaseClient_Response_Base
{
    const ID = 'id';
    const VERSION = 'version';
    const NAME = 'name';
    const UID = 'uid';
    const KIND = 'kind';
    const OWNER = 'owner';
    const SPACE_USAGE = 'space_usage';
    const INHERITED = 'inherited';
    const ACTIVE = 'active';
    const CONTENT_URL = 'content_url';
    const ITEMS = 'items';

    /** @var Service_MspApiClient_Response_Struct_FactoryInterface */
    private $_factory;

    /**
     * @param string|null $responseBody
     * @param Service_MspApiClient_Response_Struct_FactoryInterface $factory
     */
    public function __construct($responseBody = NULL, Service_MspApiClient_Response_Struct_FactoryInterface $factory = NULL)
    {
        $this->_factory = is_null($factory) ? new Service_MspApiClient_Response_Struct_Factory() : $factory;
        parent::__construct($responseBody);
    }

    /**
     * @param array $responseData
     * @return array of Service_MspApiClient_Response_Struct_Storage
     */
    protected function _init(array $responseData)
    {
        $storageList = array();
        try
        {
            $storageItems = $this->_getProperty(self::ITEMS, $responseData);
            foreach ($storageItems as $storage)
            {
                $storageList[] = $this->_factory->getStorage(
                    $this->_getProperty(self::ID, $storage),
                    $this->_getProperty(self::VERSION, $storage),
                    $this->_getProperty(self::NAME, $storage),
                    $this->_getProperty(self::UID, $storage),
                    $this->_getProperty(self::KIND, $storage),
                    $this->_getProperty(self::OWNER, $storage),
                    $this->_getProperty(self::SPACE_USAGE, $storage),
                    $this->_getProperty(self::INHERITED, $storage),
                    $this->_getProperty(self::ACTIVE, $storage),
                    $this->_getProperty(self::CONTENT_URL, $storage)
                );
            }
        }
        catch (Exception $e)
        {
            Log::getInstance()->error($e->getMessage());
            return array();
        }
        return $storageList;
    }
}
