<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
class Service_PbaApi_Api_Request_Struct_OrderAttributes
{
    /**
     * existing customer account ID to place the order for;
     * @var int
     */
    private $accountId;

    /**
     * the order type ID. The list of order types available can be viewed under the
     * System > Settings > Order Processing > Order Types submenu of the Navigation tree.
     * Each order type has its own order flow. It is recommended to place billing orders (BO)
     * if there are no special requirements to order flow;
     * @var string
     */
    private $orderTypeId;

    /**
     * status the order will be created in. The order processing will start from this status as well.
     * The list of statuses available depends on order type.
     * It can be viewed under the Statuses tab of the order type selected;
     * @var string
     */
    private $orderStatusId;

    /**
     * a promotion code. The parameter is optional and can be skipped or passed empty;
     * @var string
     */
    private $promoCode;

    /**
     * ID of a sales branch to be assigned to the order.
     * The parameter is optional and can be skipped or passed empty;
     * @var int
     */
    private $branchId;

    /**
     * ID of a sales person to be assigned to the order.
     * The parameter is optional and can be skipped or passed empty;
     * @var int
     */
    private $salesId;

    /**
     * signifies how the invoice is to be created for the order.
     * Available options are:
     * "0" - the invoice is generated according to the Billing Type parameter value set in the account customer class,
     * "10" - the Billing Type parameter value set in the account customer class is ignored and invoice is generated as
     * if the Billing Type parameter is set to the Per Subscription value,
     * "20" - forced consolidation of the first order for Billing Type set to Per Account.
     * The parameter is optional and can be skipped or passed empty;
     * @var int
     */
    private $consolidationType;

    /**
     * ID of the customer payment method (credit card or bank account) to be used to pay the order.
     * The parameter is optional and can be skipped or passed empty.
     * The customer default payment method will be used in this case;
     * @var int
     */
    private $soPayToolId;

    /**
     * customer IP address the order is placed from.
     * The parameter is optional and can be skipped or passed empty;
     * @var string
     */
    private $fromIp;

    /**
     * order creation date in Unix time format. The parameter is optional.
     * If it is not specified, system current time is assigned;
     * @var int
     */
    private $orderDate;

    /**
     * order general description, the order Comments field.
     * @var string
     */
    private $description;

    /**
     * @return int
     */
    public function getAccountId()
    {
        return $this->accountId;
    }

    /**
     * @param int $accountId
     */
    public function setAccountId($accountId)
    {
        $this->accountId = $accountId;
    }

    /**
     * @return string
     */
    public function getOrderTypeId()
    {
        return $this->orderTypeId;
    }

    /**
     * @param string $orderTypeId
     */
    public function setOrderTypeId($orderTypeId)
    {
        $this->orderTypeId = $orderTypeId;
    }

    /**
     * @return string
     */
    public function getOrderStatusId()
    {
        return $this->orderStatusId;
    }

    /**
     * @param string $orderStatusId
     */
    public function setOrderStatusId($orderStatusId)
    {
        $this->orderStatusId = $orderStatusId;
    }

    /**
     * @return string
     */
    public function getPromoCode()
    {
        return $this->promoCode;
    }

    /**
     * @param string $promoCode
     */
    public function setPromoCode($promoCode)
    {
        $this->promoCode = $promoCode;
    }

    /**
     * @return int
     */
    public function getBranchId()
    {
        return $this->branchId;
    }

    /**
     * @param int $branchId
     */
    public function setBranchId($branchId)
    {
        $this->branchId = $branchId;
    }

    /**
     * @return int
     */
    public function getSalesId()
    {
        return $this->salesId;
    }

    /**
     * @param int $salesId
     */
    public function setSalesId($salesId)
    {
        $this->salesId = $salesId;
    }

    /**
     * @return int
     */
    public function getConsolidationType()
    {
        return $this->consolidationType;
    }

    /**
     * @param int $consolidationType
     */
    public function setConsolidationType($consolidationType)
    {
        $this->consolidationType = $consolidationType;
    }

    /**
     * @return int
     */
    public function getSoPayToolId()
    {
        return $this->soPayToolId;
    }

    /**
     * @param int $soPayToolId
     */
    public function setSoPayToolId($soPayToolId)
    {
        $this->soPayToolId = $soPayToolId;
    }

    /**
     * @return string
     */
    public function getFromIp()
    {
        return $this->fromIp;
    }

    /**
     * @param string $fromIp
     */
    public function setFromIp($fromIp)
    {
        $this->fromIp = $fromIp;
    }

    /**
     * @return int
     */
    public function getOrderDate()
    {
        return $this->orderDate;
    }

    /**
     * @param int $orderDate
     */
    public function setOrderDate($orderDate)
    {
        $this->orderDate = $orderDate;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function __toString()
    {
        $attributes = array(
            'AccountID' => $this->accountId,
            'OrderTypeID' => $this->orderTypeId,
            'OrderStatusID' => $this->orderStatusId,
            'PromoCode' => $this->promoCode,
            'BranchID' => $this->branchId,
            'SalesID' => $this->salesId,
            'ConsolidationType' => $this->consolidationType,
            'SOPayToolID' => $this->soPayToolId,
            'FromIP' => $this->fromIp,
            'OrderDate' => $this->orderDate,
            'Descr' => $this->description,
        );

        $result = 'SalesOrder:';
        foreach ($attributes as $k => $v) {
            if (is_null($v)) {
                continue;
            }
            $result .= $k . '=' . $v . ';';
        }
        return $result;
    }
}
