<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_PbaApi_Api_Response_Struct_DeliverLine
{
    /**
     * Unique Identifier of a deliver line.
     * @var integer
     */
    private $deliverLineId;

    /**
     * Identifier of an order detail, from which the deliver line occurred in the system.
     * @var integer
     */
    private $detailId;

    /**
     * Identifier of a reseller transaction. This field is empty,
     * in case the deliver line is not related to any reseller activity.
     * @var integer
     */
    private $resellerTransactionId;

    /**
     * Customer account Identification.
     * @var integer
     */
    private $customerId;

    /**
     * Identifier of a subscription in OA Billing
     * @var integer
     */
    private $subscriptionId;

    /**
     * Identifier of a detail type (plan setup, plan recurring, resource setup, etc.)
     * @var integer
     */
    private $detailTypeId;

    /**
     * Identifier of the related service plan in OA Billing
     * @var integer
     */
    private $planId;

    /**
     * Resource Identifier in OA Billing. The field is not empty only for additional resource charges.
     * @var integer
     */
    private $resourceId;

    /**
     * Identifier of the parent deliver line. This field is not empty for both deliver line parts,
     * which appear from an original deliver line after splitting.
     * @var integer
     */
    private $parentDeliverLineId;

    /**
     * Status of the deliver line. This parameter can have one of the following values: 1 – Active, 999 – Deleted.
     * @var integer
     */
    private $deliverLineStatus;

    /**
     * Total tax amount applied to the deliver line.
     * @var float
     */
    private $taxAmount;

    /**
     * Included tax amount applied to the deliver line.
     * @var float
     */
    private $taxIncludedAmount;

    /**
     * Total price for the deliver line.
     * @var float
     */
    private $extendedPrice;

    /**
     * Total tax amount per month applied to the deliver line.
     * @var float
     */
    private $taxAmountPerMonth;

    /**
     * Included tax amount per month applied to the deliver line.
     * @var float
     */
    private $taxIncludedAmountPerMonth;

    /**
     * Total price per month applied to the deliver line.
     * @var float
     */
    private $extendedPricePerMonth;

    /**
     * Tax percent, applied to the deliver line.
     * @var float
     */
    private $taxPercent;

    /**
     * Start date of the deliver line.
     * @var integer
     */
    private $deliverLineStartDate;

    /**
     * End date of the deliver line.
     * @var integer
     */
    private $deliverLineEndDate;

    /**
     * Base date of the deliver line (usually taken from the start date of a subscription).
     * @var integer
     */
    private $deliverLineBaseDate;

    /**
     * Period between the start and end dates of the delivery line, in months.
     * @var float
     */
    private $periodInMonths;

    /**
     * Unit of measure applicable to the order items.
     * For example, “month(s)” is a unit of mesure for a subscription,
     * and “GB” is a unit of mesure for additional disk space.
     * @var string
     */
    private $unitMeasure;

    /**
     * Resource amount for resource deliver lines. This field is empty for the subscription-related lines.
     * @var float
     */
    private $resourceAmount;

    /**
     * Description of the deliver line.
     * @var string
     */
    private $description;

    /**
     * SKU of the deliver line.
     * @var string
     */
    private $SKU;

    /**
     * Amount of the discount applied to the deliver line.
     * @var float
     */
    private $discountPercent;

    /**
     * Price of the purchased services specified per unit.
     * @var float
     */
    private $unitPrice;

    /**
     * Identifier of the sales branch to which the purchase belongs.
     * @var string
     */
    private $salesId;

    /**
     * Identifier of the sales person with which the purchase is associated.
     * @var string
     */
    private $branchId;

    /**
     * Type of the applicable billing period.
     * The value of this parameter can be as follows:
     * 2 – Fixed Number of Months
     * 3 – Fixed Number of Years
     * 4 – Monthly on Statement Cycle Date
     * @var integer
     */
    private $billingPeriodType;

    /**
     * Value of the Billing Period Type parameter.
     * @var integer
     */
    private $billingPeriod;

    /**
     * For the current deliver line, Id of the second deliver line merged into the current deliver line. Can be null.
     * @var integer
     */
    private $parentDeliverLineId2;

    /**
     * For the current resource deliver line, the lower limit of the tier for which this line was generated.
     * Determined only for resource deliver lines. Non-zero only for the resource deliver lines for which
     * Tiered Pricing is configured (either for setup fees, or recurring fees, or overuse fees).
     * Note: Tiered Pricing is configured separately for setup fees, recurring fees, and overuse fees.
     * @var float
     */
    private $lowerBound;

    /**
     * For the current resource deliver line, Id of the resource which is switched/upgraded
     * to the resource represented by the current deliver line. Can be null. Determined only
     * for resource deliver lines. Non-zero only for the resource deliver lines
     * for which Tiered Pricing is configured for the recurring fees.
     * @var integer
     */
    private $fromResourceId;

    /**
     * @return int
     */
    public function getDeliverLineId()
    {
        return $this->deliverLineId;
    }

    /**
     * @param int $deliverLineId
     */
    public function setDeliverLineId($deliverLineId)
    {
        $this->deliverLineId = $deliverLineId;
    }

    /**
     * @return int
     */
    public function getDetailId()
    {
        return $this->detailId;
    }

    /**
     * @param int $detailId
     */
    public function setDetailId($detailId)
    {
        $this->detailId = $detailId;
    }

    /**
     * @return int
     */
    public function getResellerTransactionId()
    {
        return $this->resellerTransactionId;
    }

    /**
     * @param int $resellerTransactionId
     */
    public function setResellerTransactionId($resellerTransactionId)
    {
        $this->resellerTransactionId = $resellerTransactionId;
    }

    /**
     * @return int
     */
    public function getCustomerId()
    {
        return $this->customerId;
    }

    /**
     * @param int $customerId
     */
    public function setCustomerId($customerId)
    {
        $this->customerId = $customerId;
    }

    /**
     * @return int
     */
    public function getSubscriptionId()
    {
        return $this->subscriptionId;
    }

    /**
     * @param int $subscriptionId
     */
    public function setSubscriptionId($subscriptionId)
    {
        $this->subscriptionId = $subscriptionId;
    }

    /**
     * @return int
     */
    public function getDetailTypeId()
    {
        return $this->detailTypeId;
    }

    /**
     * @param int $detailTypeId
     */
    public function setDetailTypeId($detailTypeId)
    {
        $this->detailTypeId = $detailTypeId;
    }

    /**
     * @return int
     */
    public function getPlanId()
    {
        return $this->planId;
    }

    /**
     * @param int $planId
     */
    public function setPlanId($planId)
    {
        $this->planId = $planId;
    }

    /**
     * @return int
     */
    public function getResourceId()
    {
        return $this->resourceId;
    }

    /**
     * @param int $resourceId
     */
    public function setResourceId($resourceId)
    {
        $this->resourceId = $resourceId;
    }

    /**
     * @return int
     */
    public function getParentDeliverLineId()
    {
        return $this->parentDeliverLineId;
    }

    /**
     * @param int $parentDeliverLineId
     */
    public function setParentDeliverLineId($parentDeliverLineId)
    {
        $this->parentDeliverLineId = $parentDeliverLineId;
    }

    /**
     * @return int
     */
    public function getDeliverLineStatus()
    {
        return $this->deliverLineStatus;
    }

    /**
     * @param int $deliverLineStatus
     */
    public function setDeliverLineStatus($deliverLineStatus)
    {
        $this->deliverLineStatus = $deliverLineStatus;
    }

    /**
     * @return float
     */
    public function getTaxAmount()
    {
        return $this->taxAmount;
    }

    /**
     * @param float $taxAmount
     */
    public function setTaxAmount($taxAmount)
    {
        $this->taxAmount = $taxAmount;
    }

    /**
     * @return float
     */
    public function getTaxIncludedAmount()
    {
        return $this->taxIncludedAmount;
    }

    /**
     * @param float $taxIncludedAmount
     */
    public function setTaxIncludedAmount($taxIncludedAmount)
    {
        $this->taxIncludedAmount = $taxIncludedAmount;
    }

    /**
     * @return float
     */
    public function getExtendedPrice()
    {
        return $this->extendedPrice;
    }

    /**
     * @param float $extendedPrice
     */
    public function setExtendedPrice($extendedPrice)
    {
        $this->extendedPrice = $extendedPrice;
    }

    /**
     * @return float
     */
    public function getTaxAmountPerMonth()
    {
        return $this->taxAmountPerMonth;
    }

    /**
     * @param float $taxAmountPerMonth
     */
    public function setTaxAmountPerMonth($taxAmountPerMonth)
    {
        $this->taxAmountPerMonth = $taxAmountPerMonth;
    }

    /**
     * @return float
     */
    public function getTaxIncludedAmountPerMonth()
    {
        return $this->taxIncludedAmountPerMonth;
    }

    /**
     * @param float $taxIncludedAmountPerMonth
     */
    public function setTaxIncludedAmountPerMonth($taxIncludedAmountPerMonth)
    {
        $this->taxIncludedAmountPerMonth = $taxIncludedAmountPerMonth;
    }

    /**
     * @return float
     */
    public function getExtendedPricePerMonth()
    {
        return $this->extendedPricePerMonth;
    }

    /**
     * @param float $extendedPricePerMonth
     */
    public function setExtendedPricePerMonth($extendedPricePerMonth)
    {
        $this->extendedPricePerMonth = $extendedPricePerMonth;
    }

    /**
     * @return float
     */
    public function getTaxPercent()
    {
        return $this->taxPercent;
    }

    /**
     * @param float $taxPercent
     */
    public function setTaxPercent($taxPercent)
    {
        $this->taxPercent = $taxPercent;
    }

    /**
     * @return int
     */
    public function getDeliverLineStartDate()
    {
        return $this->deliverLineStartDate;
    }

    /**
     * @param int $deliverLineStartDate
     */
    public function setDeliverLineStartDate($deliverLineStartDate)
    {
        $this->deliverLineStartDate = $deliverLineStartDate;
    }

    /**
     * @return int
     */
    public function getDeliverLineEndDate()
    {
        return $this->deliverLineEndDate;
    }

    /**
     * @param int $deliverLineEndDate
     */
    public function setDeliverLineEndDate($deliverLineEndDate)
    {
        $this->deliverLineEndDate = $deliverLineEndDate;
    }

    /**
     * @return int
     */
    public function getDeliverLineBaseDate()
    {
        return $this->deliverLineBaseDate;
    }

    /**
     * @param int $deliverLineBaseDate
     */
    public function setDeliverLineBaseDate($deliverLineBaseDate)
    {
        $this->deliverLineBaseDate = $deliverLineBaseDate;
    }

    /**
     * @return float
     */
    public function getPeriodInMonths()
    {
        return $this->periodInMonths;
    }

    /**
     * @param float $periodInMonths
     */
    public function setPeriodInMonths($periodInMonths)
    {
        $this->periodInMonths = $periodInMonths;
    }

    /**
     * @return string
     */
    public function getUnitMeasure()
    {
        return $this->unitMeasure;
    }

    /**
     * @param string $unitMeasure
     */
    public function setUnitMeasure($unitMeasure)
    {
        $this->unitMeasure = $unitMeasure;
    }

    /**
     * @return float
     */
    public function getResourceAmount()
    {
        return $this->resourceAmount;
    }

    /**
     * @param float $resourceAmount
     */
    public function setResourceAmount($resourceAmount)
    {
        $this->resourceAmount = $resourceAmount;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return string
     */
    public function getSKU()
    {
        return $this->SKU;
    }

    /**
     * @param string $SKU
     */
    public function setSKU($SKU)
    {
        $this->SKU = $SKU;
    }

    /**
     * @return float
     */
    public function getDiscountPercent()
    {
        return $this->discountPercent;
    }

    /**
     * @param float $discountPercent
     */
    public function setDiscountPercent($discountPercent)
    {
        $this->discountPercent = $discountPercent;
    }

    /**
     * @return float
     */
    public function getUnitPrice()
    {
        return $this->unitPrice;
    }

    /**
     * @param float $unitPrice
     */
    public function setUnitPrice($unitPrice)
    {
        $this->unitPrice = $unitPrice;
    }

    /**
     * @return string
     */
    public function getSalesId()
    {
        return $this->salesId;
    }

    /**
     * @param string $salesId
     */
    public function setSalesId($salesId)
    {
        $this->salesId = $salesId;
    }

    /**
     * @return string
     */
    public function getBranchId()
    {
        return $this->branchId;
    }

    /**
     * @param string $branchId
     */
    public function setBranchId($branchId)
    {
        $this->branchId = $branchId;
    }

    /**
     * @return int
     */
    public function getBillingPeriodType()
    {
        return $this->billingPeriodType;
    }

    /**
     * @param int $billingPeriodType
     */
    public function setBillingPeriodType($billingPeriodType)
    {
        $this->billingPeriodType = $billingPeriodType;
    }

    /**
     * @return int
     */
    public function getBillingPeriod()
    {
        return $this->billingPeriod;
    }

    /**
     * @param int $billingPeriod
     */
    public function setBillingPeriod($billingPeriod)
    {
        $this->billingPeriod = $billingPeriod;
    }

    /**
     * @return int
     */
    public function getParentDeliverLineId2()
    {
        return $this->parentDeliverLineId2;
    }

    /**
     * @param int $parentDeliverLineId2
     */
    public function setParentDeliverLineId2($parentDeliverLineId2)
    {
        $this->parentDeliverLineId2 = $parentDeliverLineId2;
    }

    /**
     * @return float
     */
    public function getLowerBound()
    {
        return $this->lowerBound;
    }

    /**
     * @param float $lowerBound
     */
    public function setLowerBound($lowerBound)
    {
        $this->lowerBound = $lowerBound;
    }

    /**
     * @return int
     */
    public function getFromResourceId()
    {
        return $this->fromResourceId;
    }

    /**
     * @param int $fromResourceId
     */
    public function setFromResourceId($fromResourceId)
    {
        $this->fromResourceId = $fromResourceId;
    }
}
