<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

abstract class Service_PbaApi_Client_Response implements Service_PbaApi_Client_ResponseInterface
{
    const PROPERTY_NOT_FOUND = 'Property with name %s not found';

    /** @var mixed */
    private $_data = NULL;

    /**
     * @param array|NULL $responseArray
     */
    public function __construct(array $responseArray = NULL)
    {
        $this->_data = $this->_init($responseArray);
    }

    /**
     * @param array $responseArray
     * @return mixed
     */
    abstract protected function _init(array $responseArray = NULL);

    /**
     * @return mixed
     */
    public function getData()
    {
        return $this->_data;
    }

    /**
     * @param string $property
     * @param array $data
     * @param bool $throwException
     * @return mixed
     * @throws Service_PbaApi_Client_InvalidPropertyException
     */
    protected function _getProperty($property, array $data, $throwException = true)
    {
        if (array_key_exists($property, $data))
        {
            return $data[$property];
        }
        $errorMessage = sprintf(self::PROPERTY_NOT_FOUND, $property);
        if ($throwException)
        {
            throw new Service_PbaApi_Client_InvalidPropertyException($errorMessage);
        }
        return NULL;
    }

    /**
     * @param array $nestedProperties
     * @param array $data
     * @param bool $throwException
     * @return mixed
     */
    protected function _getNestedProperty(array $nestedProperties, array $data, $throwException = true)
    {
        foreach($nestedProperties as $property)
        {
            $data = $this->_getProperty($property, $data, $throwException);
        }
        return $data;
    }
}
