<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
abstract class AbstractSettings
{
    const MODULE_IS_NOT_CONFIGURED_MESSAGE = '%s module is not configured';
    const CONFIG_COULD_NOT_BE_WRITTEN= 'The config %s could not be written';

    /**
     * Read ini configuration file
     * @param string $fileName
     * @param string $configName
     * @return array
     * @throws Exception
     */
    protected function readConfig($fileName, $configName)
    {
        $this->checkFileAvailable($fileName, $configName);
        return parse_ini_file($fileName);
    }

    /**
     * @param $fileName
     * @param $configName
     * @throws Exception
     */
    protected function checkFileAvailable($fileName, $configName)
    {
        if (!is_file($fileName) || !is_readable($fileName)) {
            Log::getInstance()->error(sprintf(self::MODULE_IS_NOT_CONFIGURED_MESSAGE, $configName));
            throw new Exception(sprintf(self::MODULE_IS_NOT_CONFIGURED_MESSAGE, $configName));
        }
    }

    /**
     * @param $fileName
     * @param $configName
     * @throws Exception
     */
    protected function checkWritable($fileName, $configName)
    {
        if (!is_writable($fileName)) {
            Log::getInstance()->error(sprintf(self::CONFIG_COULD_NOT_BE_WRITTEN, $configName));
            throw new Exception(sprintf(self::CONFIG_COULD_NOT_BE_WRITTEN, $configName));
        }
    }

    /**
     * @param $fileName
     * @param $configName
     * @param $content
     * @throws Exception
     */
    protected function writeConfig($fileName, $configName, $content)
    {
        $this->checkWritable($fileName, $configName);
        file_put_contents($fileName, $content);
    }
}
