<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

class ApiFactory
{
    /**
     * Gets Open API instance to work with Odin API
     * @param string $openApiHost
     * @return Service_OpenApi_Api
     */
    public function getOpenApi($openApiHost)
    {
        $client = new Service_OpenApi_Client($openApiHost);
        return new Service_OpenApi_Api($client);
    }

    /**
     * @param string $controllerUri
     * @param string $apsToken
     * @param string $apsInstanceId
     * @return \APS\ControllerProxy
     */
    public function getApsController($controllerUri, $apsToken, $apsInstanceId = null)
    {
        $_SERVER['HTTP_APS_INSTANCE_ID'] = $apsInstanceId ?: self::findPathIdInConfigFolder();
        $apsc = new \APS\ControllerProxy($controllerUri, null, $apsToken);
        APS\Request::setController($apsc);
        return $apsc;
    }

    /**
     * @param $datacenter
     * @param $instanceId
     * @return Service_MspApiClient_Api
     */
    public function getMspApi($datacenter, $instanceId)
    {
        $instanceId = sprintf(
            APSResource::UNDERSCORE_DELIMITER,
            $instanceId,
            md5($datacenter->login . $datacenter->url)
        );

        $restClient = new Service_BaseClient_RestClient();
        $restClient->setApiUri($datacenter->url . MSP_API_URI);
        $cookieFile = DIR_COOKIES . $instanceId;
        $restClient->setCookieFile($cookieFile);
        $store = new Session_StoreStub();
        $mspApi = new Service_MspApiClient_Api($restClient, $store);

        $mspApi->setCredentials(
            $datacenter->login,
            $datacenter->password,
            $datacenter->url,
            false
        );

        $mspApi->login();//todo:check work without login
        return $mspApi;
    }

    /**
     * @param object $config
     * @return Service_PbaApi_Api
     */
    public function getPbaApi($config)
    {
        $xmlRpcClient = new Service_PbaApi_Client($config->host, $config->port, $config->path, $config->secure);
        if ($config->auth) {
            $xmlRpcClient->setCredentials($config->username, $config->password);
        }
        return new Service_PbaApi_Api($xmlRpcClient);
    }

    /**
     * Finds id in APS config folder
     * @return string
     * @throws Exception
     */
    private static function findPathIdInConfigFolder()
    {
        $instances = \APS\ControllerProxy::listInstances();
        if (empty($instances)) {
            throw new Exception("There is no application instance in the configuration folder.");
        }
        return reset($instances);
    }
}
