<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Cache_Store implements Cache_StoreInterface
{
    const TTL = 90;
    const CACHE_UNIQUE_KEY_MASK = '%s_%s';

    /**
     * @var
     */
    private static $_instance;

    /**
     * @var string
     */
    private $_cachePrefix;

    private function __construct()
    {
    }

    /**
     * Singleton implementation
     *
     * @return Cache_Store object
     */
    public static function getInstance()
    {
        if(is_null(self::$_instance))
        {
            self::$_instance = new Cache_Store();
        }
        return self::$_instance;
    }

    /**
     * @param string $cachePrefix
     */
    public function setCachePrefix($cachePrefix)
    {
        $this->_cachePrefix = $cachePrefix;
    }

    /**
     * @return string
     */
    public function getCachePrefix()
    {
        return $this->_cachePrefix;
    }

    /**
     * @param string $key
     * @return string
     */
    public function getCacheUniqueKey($key)
    {
        if (isset($this->_cachePrefix))
        {
            return sprintf(self::CACHE_UNIQUE_KEY_MASK, $this->_cachePrefix, $key);
        }
        return $key;
    }

    /**
     * Sets value to cache
     *
     * @param string $key
     * @param mixed $value
     */
    public function setValue($key, $value)
    {
        $uniqueKey = $this->getCacheUniqueKey($key);
        apc_store($uniqueKey, $value, self::TTL);
    }

    /**
     * Gets value from cache
     *
     * @param string $key
     * @return mixed|NULL
     */
    public function getValue($key)
    {
        $uniqueKey = $this->getCacheUniqueKey($key);
        $data = apc_fetch($uniqueKey, $success);
        return $success ? $data : NULL;
    }

    /**
     * Delete value from cache
     *
     * @param string $key
     * @return mixed|NULL
     */
    public function unsetValue($key)
    {
        $uniqueKey = $this->getCacheUniqueKey($key);
        $data = apc_fetch($uniqueKey, $success);
        apc_delete($key);
        return $success ? $data : NULL;
    }
}
