<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

interface Session_StoreInterface
{
    /**
     * Singleton implementation
     *
     * @return Session_Store object
     */
    public static function getInstance();

    /**
     * Set session id
     *
     * @param $id
     * @return string
     */
    public function setSessionId($id);

    /**
     * start session if it's not started
     *
     * @return true
     */
    public function startSession();

    /**
     * stop session if it's started
     * @return boolean
     */
    public function stopSession();

    /**
     * increse for session_lifetime current expire value for key
     * @param string $key
     */
    public function increaseExpireTime($key);

    /**
     * Sets value to session by key and create expire time for this value
     *
     * @param string $key
     * @param mixed $value
     * @param int|null $ttl
     */
    public function setValue($key,$value,$ttl=NULL);


    /**
     * Return value from session by key and update expire time
     *
     * @param string $key
     * @return mixed
     */
    public function getValue($key);

    /**
     * Check that session value by key is not expired
     *
     * @param string $key
     * @return bool
     */
    public function isExpired($key);

    /**
     * Delete value from session by key
     *
     * @param string $key
     */
    public function unsetValue($key);

}
