<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
class OpenApiFacade
{

    const FIELD_NAME = 'name';
    const FIELD_SUBSCRIPTION_ID = 'subscription_id';
    const FIELD_VALUE = 'value';
    const SUBSCRIPTION_ID_NOT_FOUND_TEMPLATE = "'Not found subscription in application instance with id %s'";
    const FIELD_GROUP_LOGIN = 'group_login';
    const INSTANCE_NOT_FOUND_TEMPLATE = "Not found application instance with id %s";
    const INSTANCE_NOT_FOUND_BY_APP_TEMPLATE = "Not found instance for group - %s in application with id - %s";
    const FIELD_APPLICATION_INSTANCE_ID = 'application_instance_id';

    /**
     * @var Service_OpenApi_Api
     */
    protected $api;

    public function __construct($api)
    {
        $this->api = $api;
    }

    /**
     * Return subscriptionId
     * @param $instanceId
     * @return int
     * @throws Exception
     */
    public function findSubscriptionId($instanceId)
    {
        $subscriptionId = null;

        $instanceSettings = $this->getInstanceSettings($instanceId);

        foreach ($instanceSettings as $setting) {
            if ($setting[self::FIELD_NAME] == self::FIELD_SUBSCRIPTION_ID) {
                $subscriptionId = $setting[self::FIELD_VALUE];
                break;
            }
        }

        if (empty($subscriptionId)) {
            throw new Exception(sprintf(self::SUBSCRIPTION_ID_NOT_FOUND_TEMPLATE, $instanceId));
        }
        return intval($subscriptionId);
    }

    /**
     * @param $groupLogin
     * @param $applicationId
     * @return int|null|string
     * @throws Exception
     */
    public function getApplicationInstanceId($groupLogin, $applicationId)
    {
        $instancesInfo = $this->getApplicationInstancesSettings($applicationId);

        $applicationInstanceId = null;
        foreach ($instancesInfo as $instanceId => $instanceInfo) {
            foreach ($instanceInfo as $setting) {
                if ($setting[self::FIELD_NAME] == self::FIELD_GROUP_LOGIN && $setting[self::FIELD_VALUE] == $groupLogin) {
                    $applicationInstanceId = $instanceId;
                    break 2;
                }
            }
        }

        if (is_null($applicationInstanceId)) {
            throw new Exception(sprintf(self::INSTANCE_NOT_FOUND_BY_APP_TEMPLATE, $groupLogin, $applicationId));
        }
        return $applicationInstanceId;
    }

    /**
     * @param $instanceId
     * @return array
     * @throws Exception
     */
    private function getInstanceSettings($instanceId)
    {
        $instanceSettings = $this->api->getApplicationInstanceSettings($instanceId);

        if (is_null($instanceSettings)) {
            throw new Exception(sprintf(self::INSTANCE_NOT_FOUND_TEMPLATE, $instanceId));
        }
        return $instanceSettings;
    }


    /**
     * @param $applicationId
     * @return array
     */
    private function getApplicationInstancesSettings($applicationId)
    {
        $applicationInstances = $this->api->getApplicationInstances($applicationId);
        $instancesSettings = array();
        foreach ($applicationInstances as $instance) {
            $instancesSettings[$instance[self::FIELD_APPLICATION_INSTANCE_ID]] = $this->api->getApplicationInstanceSettings($instance[self::FIELD_APPLICATION_INSTANCE_ID]);
        }
        return $instancesSettings;
    }

    /**
     * @param $subscriptionId
     * @return Service_OpenApi_Response_SubscriptionInfo
     */
    public function getSubscriptionById($subscriptionId)
    {
        return $this->api->getSubscription($subscriptionId);
    }

    public function getServiceInstances($instanceId, $serviceName)
    {
        return $this->api->getServiceInstances($instanceId, $serviceName);
    }
}
