<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once __DIR__ . "/../app/bootstrap.php";

error_reporting(E_ALL);
ini_set('display_errors', true);
ini_set('display_startup_errors', true);
ini_set('memory_limit', '2048M');

define('REPORT_DIR_ROOT', __DIR__ . DS . 'build_billing_report' . DS);
define('REPORT_DIR_SRC', REPORT_DIR_ROOT . 'src' . DS);
define('REPORT_DIR_SQL', REPORT_DIR_ROOT . 'sql' . DS);
define('REPORT_DIR_CONFIG', REPORT_DIR_ROOT . 'config' . DS);

require_once REPORT_DIR_SRC . 'BillingReportHelper.php';

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;

const OUTPUT_OPTION_UNDEFINED = "Option '%s' undefined \n";

const OPTION_PARSE_CHAR = ':';
const OPTION_VERBOSE = 'v';
const OPTION_HELP = 'h';

/**
 *  Prints usage to the screen
 */
function printScriptUsage()
{
    $fileName = basename($_SERVER['PHP_SELF']);
    echo "Usage: php {$fileName} [OPTIONS]
Example: php {$fileName} -vv
Options:
  -v               Verbose
";
}

function validateCommandLineParameters($opt)
{
    if (isset($opt[OPTION_HELP])) {
        return true;
    }
    return true;
}

/**
 * @param string $optionName
 * @param array $options
 * @return mixed | null
 */
function getOptions($optionName, $options)
{
    $option = null;
    switch ($optionName) {
        case OPTION_HELP:
            $option = isset($options[OPTION_HELP]);
            break;
        case OPTION_VERBOSE:
            if (isset($options[OPTION_VERBOSE])) {
                $option = is_array($options[OPTION_VERBOSE]) ? count($options[OPTION_VERBOSE]) : 1;
            } else {
                $option = 0;
            }
            break;
        default:
            echo sprintf(OUTPUT_OPTION_UNDEFINED, $optionName);
            printScriptUsage();
            exit;
    }
    return $option;
}

$opt = getopt(
    OPTION_VERBOSE .
    OPTION_HELP
);

if (!validateCommandLineParameters($opt)) {
    exit(1);
};

if (getOptions(OPTION_HELP, $opt)) {
    printScriptUsage();
    exit;
}

$verbose = getOptions(OPTION_VERBOSE, $opt);

$output = Log::FIELD_TIME . ' ' . Log::FIELD_REQUEST . ' ' . Log::FIELD_TYPE . ' ' . Log::FIELD_MESSAGE . PHP_EOL;
$formatter = new LineFormatter($output);

$stream = new StreamHandler('php://stdout', Logger::NOTICE + $verbose * 50);
$stream->setFormatter($formatter);
Log::getInstance()->getLogger()->pushHandler($stream);

Log::getInstance()->notice('Start.');
try {
    $excelHelper = new BillingReportHelper();
    $excelHelper->generate();
} catch (Exception $e) {
    Log::getInstance()->error($e->getMessage());
    exit(1);
}
Log::getInstance()->notice('Finish.');
