SELECT
  CASE WHEN gr."Value" IS NULL
    THEN cr."CompanyName"
  ELSE gr."Value" END                                              AS "ReportGroup",
  br."AccountID"                                                   AS "SPID",
  br."CompanyName"                                                 AS "SPName",
  cr."CompanyName"                                                 AS "CRName",
  br."AAN"                                                         AS "SPAAN",
  craan."Value"                                                    AS "CRAAN",
  br."ContactEmail"                                                AS "SPEmail",
  br."LType"                                                       AS "LType",
  trim(LEADING 'en ' FROM split_part(c."Name", chr(9), 1))           AS "SPCountry",
  ''                                                               AS "Region",
  br."DatacenterID"                                                AS "SPDatacenter",
  br."Product"                                                     AS "Product",
  br."SKU"                                                         AS "SKU",
  br."CurrencyID"                                                  AS "SPCurrency",
  br."StatusName"                                                  AS "StatusName",
  mc."Value"                                                       AS "MinCom",
  mc_tier."Value"                                                  AS "MinComTier",
  mc_month."Value"                                                 AS "MinComMonth",
  to_char(to_timestamp(min(br."StartDate")), 'MM/DD/YYYY')         AS "Start",
  to_char(to_timestamp(max(br."EndDate")), 'MM/DD/YYYY')           AS "End",
  array_to_string((array_agg(DISTINCT br."subscriptionID")), ', ') AS "subscriptions",
  br."Discount"                                                    AS "Discount",
  sum(br."ExtendedPrice")                                          AS "Overall"
FROM
  "AcronisServiceProviderBreakdown" AS br
  JOIN "Account" AS cr ON cr."AccountID" = br."VendorAccountID"
  JOIN "AccAttr" AS craan ON craan."ObjID" = br."VendorAccountID" AND craan."ATID" = 'AAN'
  JOIN "Country" AS c ON c."CountryID" = br."CountryID"
  LEFT JOIN "AccAttr" AS mc ON mc."ObjID" = br."AccountID" AND mc."ATID" = 'MinCom'
  LEFT JOIN "AccAttr" AS mc_tier ON mc_tier."ObjID" = br."AccountID" AND mc_tier."ATID" = 'MinComTier'
  LEFT JOIN "AccAttr" AS mc_month ON mc_month."ObjID" = br."AccountID" AND mc_month."ATID" = 'MinComMonth'
  LEFT JOIN "AccAttr" AS gr ON gr."ObjID" = br."VendorAccountID" AND gr."ATID" = 'ReportGroup'
WHERE
  br."ResourceSKU" IS NOT NULL
GROUP BY
  "ReportGroup",
  "SPID",
  "SPName",
  "CRName",
  "SPAAN",
  "CRAAN",
  "SPEmail",
  "LType",
  "SPCountry",
  "SPDatacenter",
  "Product",
  "SKU",
  "SPCurrency",
  "StatusName",
  "MinCom",
  "MinComTier",
  "MinComMonth",
  "Discount"
ORDER BY
  "SPName",
  "LType",
  "SPDatacenter",
  "StatusName";
