<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

ini_set('memory_limit', -1);

if (!ini_get('date.timezone')) {
    $timezone = exec("date +%Z");
    date_default_timezone_set(timezone_name_from_abbr($timezone));
}

class UomConvertHelper
{
    const MEASURABLE_MULTIPLIER = 1024;
    const MEASURABLE_B = 'b';
    const MEASURABLE_KB = 'kb';
    const MEASURABLE_MB = 'mb';
    const MEASURABLE_GB = 'gb';
    const MEASURABLE_TB = 'tb';

    /**
     * Measurement units converter (b, kb, mg, gb, tb)
     *
     * @param int|float $value
     * @param string $from
     * @param string $to
     * @return int|float|NULL
     */
    public static function convert($value, $from, $to)
    {
        $uomFromName = strtolower($from);
        $uomToName = strtolower($to);

        $init = 1;
        $names = array(
            self::MEASURABLE_B => $init,
            self::MEASURABLE_KB => ($init *= self::MEASURABLE_MULTIPLIER),
            self::MEASURABLE_MB => ($init *= self::MEASURABLE_MULTIPLIER),
            self::MEASURABLE_GB => ($init *= self::MEASURABLE_MULTIPLIER),
            self::MEASURABLE_TB => ($init * self::MEASURABLE_MULTIPLIER),
        );

        if (!isset($names[$uomFromName])) {
            return null;
        }

        if (!isset($names[$uomToName])) {
            return null;
        }

        $uomFromValue = $names[$uomFromName];
        $uomToValue = $names[$uomToName];

        return $value * $uomFromValue / $uomToValue;
    }
}

function convertStorageUsageToMb($value)
{
    return ceil(UomConvertHelper::convert($value, 'b', 'mb'));
}

$dt = new DateTime('last day of previous month');
$dt->setTime(0, 0, 0);
$date = $dt->format('Ymd');

$path = '/var/www/html/backupservice/tmp/';

$datacenters = json_decode(file_get_contents('tmp/datacenters.json'));

$files = array();
foreach($datacenters as $datacenter) {
   $files[$datacenter->id] = $path . $datacenter->id . "_$date-$date.dat"; 
}
$files = array_filter($files, function($file) {
    return file_exists($file);
});
$data = array_map(function($file) {
    return  unserialize(file_get_contents($file));
}, $files);
$tenants = json_decode(file_get_contents('tmp/tenants.json'));

$stat = array();
foreach ($tenants as $tenant) {
    if (empty($tenant->datacenterId) || empty($tenant->mspGroupId)) {
        echo '-- empty ' . $tenant->aps->id . "\n";
        continue;
    }
    if(!isset($data[$tenant->datacenterId][$tenant->mspGroupId])) {
        continue;
    }
    $report = $data[$tenant->datacenterId][$tenant->mspGroupId];
    $usage = new stdClass();
    $stat[$tenant->subscriptionId] = $usage;

    $usage->StorageUsage = $report->totalStorage;
    $usage->ServerCount = $report->servers;
    $usage->VmCount = $report->vms;
    $usage->WorkstationCount = $report->workstations;
    $usage->MobileDeviceCount = $report->mobiles;
    $usage->MailboxCount = $report->mailboxes;

    $usage->StorageUsageMbH = convertStorageUsageToMb($usage->StorageUsage);
    $usage->ServerCountItemH = $usage->ServerCount;
    $usage->VmCountItemH = $usage->VmCount;
    $usage->WorkstationCountItemH = $usage->WorkstationCount;
    $usage->MobileDeviceCountItemH = $usage->MobileDeviceCount;
    $usage->MailboxCountItemH = $usage->MailboxCount;

    $usage->AcronisHostedStorage = $report->acronisStorage;
    $usage->ServiceProviderHostedStorage = $report->partnerStorage;

    $usage->AcronisHostedStorageMbH = convertStorageUsageToMb($usage->AcronisHostedStorage);
    $usage->ServiceProviderHostedStorageMbH = convertStorageUsageToMb($usage->ServiceProviderHostedStorage);

    if (!$tenant->wsDiscount || $usage->StorageUsage == 0) {
        continue;
    }
    $discountInBytes = floor(0.4 * ($report->acronisWorkstationStorage + $report->partnerWorkstationStorage));
    $usage->StorageUsage -= $discountInBytes;
    $usage->StorageUsageMbH = convertStorageUsageToMb($usage->StorageUsage);

    if ($usage->AcronisHostedStorage > 0) {
        $discountInBytes = floor(0.4 * $report->acronisWorkstationStorage);
        $usage->AcronisHostedStorage -= $discountInBytes;
        $usage->AcronisHostedStorageMbH = convertStorageUsageToMb($usage->AcronisHostedStorage);
    }
    if ($usage->ServiceProviderHostedStorage > 0) {
        $discountInBytes = floor(0.4 * $report->partnerWorkstationStorage);
        $usage->ServiceProviderHostedStorage -= $discountInBytes;
        $usage->ServiceProviderHostedStorageMbH = convertStorageUsageToMb($usage->ServiceProviderHostedStorage);
    }
}

/**
 * select "OrderID", "subscriptionID" from "OrdDet" where "OrderID" in (select "OrderID" from "SalesOrder" where "OrderTypeID" = 'BO' and "OrderDate" = '1483221600' and "Customer_AccountID" in (select "AccountID" from "Subscription" where "Status" = 30 and "serviceTemplateID" in (11, 20, 14, 27))) and "resourceID" = '1000186';
 * select "OrderID", "subscriptionID" from "OrdDet" where "OrderID" in (select "OrderID" from "SalesOrder" where "OrderTypeID" = 'BO' and "OrderDate" = extract(epoch from date_trunc('month', now())) and "Customer_AccountID" in (select "AccountID" from "Subscription" where "Status" = 30 and "serviceTemplateID" in (11, 20, 14, 27))) and "resourceID" = '1000186';
 */
if (($handle = fopen("tmp/orders.csv", "r")) !== FALSE) {
    while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
        $orderId = $data[0];
        $subscriptionId = $data[1];

        if (!isset($stat[$subscriptionId])) {
            echo sprintf('-- Subscription: %s Order: %s
-- NO USAGE             
',
                $subscriptionId, $orderId
            );
            continue;
        }

        echo sprintf('-- Subscription: %s Order: %s
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
update "OrdDet" set "ServQty" = \'%s\' where "resourceID" = \'%s\' and "OrderID" = \'%s\' and "subscriptionID" = \'%s\';
',
            $subscriptionId, $orderId,
            round(UomConvertHelper::convert($stat[$subscriptionId]->AcronisHostedStorageMbH, 'mb', 'gb'),2),         '1000232', $orderId, $subscriptionId, // 'Acronis Hosted Storage'
            round(UomConvertHelper::convert($stat[$subscriptionId]->ServiceProviderHostedStorageMbH, 'mb', 'gb'),2), '1000235', $orderId, $subscriptionId, // 'Service Provider Hosted Storage'
            $stat[$subscriptionId]->WorkstationCountItemH,                                                           '1000244', $orderId, $subscriptionId, // 'Physical Workstation'
            $stat[$subscriptionId]->ServerCountItemH,                                                                '1000238', $orderId, $subscriptionId, // 'Physical Server'
            $stat[$subscriptionId]->VmCountItemH,                                                                    '1000241', $orderId, $subscriptionId, // 'Virtual Machine'
            $stat[$subscriptionId]->MobileDeviceCountItemH,                                                          '1000247', $orderId, $subscriptionId, // 'Mobile Device'
            $stat[$subscriptionId]->MailboxCountItemH,                                                               '1000250', $orderId, $subscriptionId  // 'Office 365 mailbox'
        );
    }
    fclose($handle);
}
