<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

/**
 * It's a tool to test generation of billing orders/
 * The script generates fake Billing Order
 * Place the script into /var/www/html/backupservice
 *
 * Usage: php createBillingOrder.php [ACCOUNT ID] [SUBSCRIPTION ID] [AMOUNT] [DATE]
 *
 * Example: php createBillingOrder.php 1000001 1006819 123.45 13.10.2016
 */

require_once(__DIR__ . "/../app/bootstrap.php");

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

/**
 * @param Service_PbaApi_Api $pbaApi
 * @param int $accountId
 * @param int $subscriptionId
 * @param int $amount
 * @return mixed
 */
function placeBillingOrder($pbaApi, $accountId, $subscriptionId, $amount, $date)
{
    $orderAttributes = new Service_PbaApi_Api_Request_Struct_OrderAttributes();
    $orderAttributes->setAccountId($accountId);
    $orderAttributes->setOrderTypeId('BO');
    $orderAttributes->setOrderStatusId('NW');
    $orderAttributes->setConsolidationType(10);
    $orderAttributes->setFromIp('10.250.55.20');
    $orderAttributes->setDescription('[auto generated] Billing Order');
    $orderAttributes->setOrderDate($date);

    $orderDetail = new Service_PbaApi_Api_Request_Struct_OrderDetailAttributes();
    $orderDetail->setExtendedPrice($amount);
    $orderDetail->setTaxAmount(0);
    $orderDetail->setTaxIncludedAmount(0);
    $orderDetail->setTaxAlgorithm(0);
    $orderDetail->setUnitPrice($amount);
    $orderDetail->setServiceQuantity(1);
    $orderDetail->setServiceUnitMeasure('-');
    $orderDetail->setDuration(1);
    $orderDetail->setDurationBillPeriod(1);
    $orderDetail->setDurationBillPeriodType(4);
    $orderDetail->setSubscriptionId($subscriptionId);
    $orderDetail->setDescription('[auto generated] Billing Order');

    $orderId = $pbaApi->createManualOrder($orderAttributes, array($orderDetail));
    return $pbaApi->getOrder($orderId);
}

$config = Settings::getInstance()->getBillingConfig();
$xmlRpcClient = new Service_PbaApi_Client($config->host, $config->port, $config->path, $config->secure);
if ($config->auth) {
    $xmlRpcClient->setCredentials($config->username, $config->password);
}
$pbaApi = new Service_PbaApi_Api($xmlRpcClient);

print_r(placeBillingOrder($pbaApi, $argv[1], $argv[2], $argv[3], isset($argv[4]) ? strtotime($argv[4]) : mktime(0, 0, 0, date("n"), 1, date("Y"))));