<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

/**
 * It's a tool to test generation of billing orders/
 * The script generates fake Billing Order
 * Place the script into /var/www/html/backupservice
 * Change constants before using
 *
 * Usage: php placeAndAuthorize.php [COUNT] [DATACENTER ID] [PLAN ID] [PERIOD ID] [AAN] [CUSTOMER EMAIL HOST]
 *
 * Example: php placeAndAuthorize.php 5 1 2 3 00000000 fake.acronis.com
 */

require_once(__DIR__ . "/../app/bootstrap.php");

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

class PlaceAndAuthorizeClass
{
    /**
     * @var Service_PbaApi_Api
     */
    private $pbaApi;

    /**
     * @param $count
     * @param $datacenterId
     * @param $planId
     * @param $planPeriodId
     * @param $aan
     * @param $mailHost
     * @throws Exception
     */
    public function run($count, $datacenterId, $planId, $planPeriodId, $aan, $mailHost)
    {
        $factory = new ApiFactory();
        $this->pbaApi = $factory->getPbaApi(Settings::getInstance()->getBillingConfig());

        echo 'Start creating...' . PHP_EOL;
        if (!isset($planId)) {
            throw new Exception('Not specified Plan ID.');
        }
        if (!isset($mailHost)) {
            $mailHost = 'fake.acronis.com';
        }

        while ($count > 0) {
            sleep(2);
            echo 'Left ' . $count . PHP_EOL;
            $this->placeAndAuthorize($datacenterId, $planId, $planPeriodId, $mailHost, $aan);
            $count--;
        }
        echo "All done!\r\n";
    }

    /**
     * @param int $datacenterId
     * @param int $planId
     * @param int $planPeriod
     * @param string $emailHost
     * @param int $aan
     * @throws \Service_PbaApi_Api_Exception
     */
    public function placeAndAuthorize($datacenterId, $planId, $planPeriod, $emailHost, $aan)
    {
        $ipAddress = '127.0.0.1';
        $parentAccountId = 1;

        $login = 'user_' . \APS\generatePassword(6, '');
        $password = \APS\generatePassword(10);
        $email = $login . '@' . $emailHost;

        $companyName = 'Ultra Hard Party';
        $firstName = 'FirstName';
        $middleName = '';
        $lastName = 'LastName';
        $country = 'US';
        $state = '';
        $city = 'City';
        $zip = '11111';
        $address = 'Address';
        $phone = '+11111111111';

        $payToolTypeId = 3; // Check/Cash
        $provisioningItemsDefaultValue = 1;
        $provisioningDataSlotCounter = 1;
        $provisioningItemIdDefaultValue = 0;
        $provisioningParametersCount = 1;
        $contactDataParametersCount = 22;
        $payToolCount = 3;

        $parameters = array(
            $provisioningItemsDefaultValue,
            "{$planId}={$planPeriod}=0=-1",
            $provisioningDataSlotCounter,
            $provisioningItemIdDefaultValue
        );

        array_push($parameters,
            $provisioningParametersCount,
            "Datacenter={$datacenterId}"
        );

        if (!empty($aan)) {
            $contactDataParametersCount++;
        }

        array_push($parameters,
            $contactDataParametersCount,
            "LoginID={$login}",
            "XXXPasswordID={$password}",
            "CompanyNameID={$companyName}",
            "FirstNameID={$firstName}",
            "MiddleNameID={$middleName}",
            "LastNameID={$lastName}",
            "AddressID={$address}",
            'Address2ID=',
            "CityID={$city}",
            "StateID={$state}",
            "ZipID={$zip}",
            "CountryID={$country}",
            "EmailID={$email}",
            'PhoneCountryID=1',
            'PhoneAreaID=0',
            "PhoneNumberID={$phone}",
            'PhoneExtensionID=0',
            'FaxCountryID=',
            'FaxAreaID=',
            'FaxNumberID=',
            'FaxExtensionID=',
            'PromoCodeID='
        );

        if (!empty($aan)) {
            $parameters[] = "AAN={$aan}";
        }

        array_push($parameters,
            $payToolCount,
            "PayToolTypeID={$payToolTypeId}",
            'PluginID=0',
            "IPAddressID={$ipAddress}"
        );

        $this->pbaApi->placeOrderAndAuthorize(
            $parentAccountId,
            $parameters
        );
    }
}

$object = new PlaceAndAuthorizeClass();
$object->run(
    isset($argv[1]) ? $argv[1] : null,
    isset($argv[2]) ? $argv[2] : null,
    isset($argv[3]) ? $argv[3] : null,
    isset($argv[4]) ? $argv[4] : null,
    isset($argv[5]) ? $argv[5] : null,
    isset($argv[6]) ? $argv[6] : null
);