<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . "/../app/bootstrap.php");
require_once(__DIR__ . "/../app/classes/ApiFactory.php");
require_once(__DIR__ . "/../migration/OpenApiFacade.php");

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

if (empty($argv[1])) {
    echo 'File path is empty.';
    exit(1);
}
if (!is_file($argv[1]) || !is_readable($argv[1])) {
    echo 'File does not exist or is not readable.';
    exit(1);
}

if (empty($argv[2])) {
    echo 'Application ID is empty';
}

if (!is_numeric($argv[2])) {
    echo 'Application ID is not number';
}

$filePath = $argv[1];
$applicationId = (int)$argv[2];
$groupNames = getGroupNamesFromFile($filePath);
$apiFactory = new ApiFactory();
$openApi = $apiFactory->getOpenApi(defined('OPEN_API_URI') ? OPEN_API_URI : POA_API_URI);
$apiHelper = new OpenApiFacade($openApi);

$reportData = array();
foreach ($groupNames as $groupName) {
    $instanceId = $apiHelper->getApplicationInstanceId($groupName, $applicationId);
    $subscriptionId = $apiHelper->findSubscriptionId($instanceId);

    $reportData[] = array($groupName, $subscriptionId);
}
generateCsvReport('/tmp/subscriptionIdReport.scv', $reportData);

function getGroupNamesFromFile($filePath)
{
    $groupNames = array();
    $f = fopen($filePath, 'r');
    while (($line = fgetcsv($f)) !== false) {
        $groupNames[] = trim($line[0], '\r\n');
    }
    fclose($f);
    return $groupNames;
}

function generateCsvReport($filePath, $data)
{
    $f = fopen($filePath, 'w');
    foreach ($data as $fields) {
        fputcsv($f, $fields, '|');
    }
    fclose($f);
}