<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . '/app/bootstrap.php');

/**
 * Class users
 * @type("http://acronis.com/aps/cloud-backup-v1/users/7.4")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 * @implements("http://aps-standard.org/types/core/user/service/1.0")
 */
class users extends APSResource
{
    const LOG_FORMAT_PROVISION_START = 'Provision service from customer subscription (%s) for user (%s).';
    const LOG_MIGRATION_WAS_STARTED = 'User migration was started.';
    const LOG_FORMAT_ATTEMPT_TO_CREATE_EXISTING_MSP_USER = 'Attempt to create user (%s) for subscription (%s), which exists in msp';
    const LOG_FORMAT_CREATING_USER_TASK = 'Creating user (%s) for customer subscribtion (%s)';
    const LOG_FORMAT_SEND_NOTIFICATION = 'Could not send notification "%s". Error: %s';
    const LOG_FORMAT_DELETE_USER_FAILED = 'Delete user (%) failed with message: %s';
    const LOG_FORMAT_UPDATE_NOTIFICATION_STATUS = 'Can\'t update notification status "%s". error: %s';
    const LOG_FORMAT_DELETE_NOTIFICATION = 'Can\'t delete notification "%s". error: %s';

    const EXCEPTION_USER_ALREADY_EXISTS = 'The account with email address already exists in the backup service. Delete the old account or specify a different service user.';
    const EXCEPTION_USER_ALREADY_ACTIVATED = 'User already activated';
    const EXCEPTION_PASSWORD_MUST_BE_PRESENT = 'Password must be present';
    const EXCEPTION_ERROR_WHILE_USER_DELETING = 'Error while deleting user (%s). Please try again.';
    const EXCEPTION_CHANGE_STATUS_NOT_ACTIVATED = 'Can\'t change account status on not activated user.';
    const EXCEPTION_TENANT_HAS_NOT_LINKED_WITH_DATACENTER_MESSAGE = 'Tenant has not linked with a datacenter.';

    const ACCOUNT_ENABLED = '1';
    const ACCOUNT_DISABLED = '0';

    const WIN_URL = 'winurl';
    const LINUX_URL = 'linuxurl';
    const MAC_URL = 'macurl';
    const ANDROID_URL = 'androidurl';
    const IOS_URL = 'iosurl';
    const VMWARE_URL = 'vmwareurl';
    const HYPERV_URL = 'hypervurl';
    const VIRTUOZZO_URL = 'virtuozzourl';
    const MICROSOFT_SQL_SERVER_URL = 'microsoftSqlServerUrl';
    const MICROSOFT_EXCHANGE_SERVER_URL = 'microsoftExchangeServerUrl';
    const MICROSOFT_ACTIVE_DIRECTORY_URL = 'microsoftActiveDirectoryUrl';
    const MICROSOFT_O365_URL = 'microsoftO365Url';

    const BACKUP_LIST_NAME = 'name';
    const BACKUP_LIST_STATUS = 'status';
    const BACKUP_LIST_LAST_BACKUP = 'last_backup';
    const BACKUP_LIST_NEXT_BACKUP = 'next_backup';
    const BACKUP_LIST_TYPE = 'type';
    const STATISTICS_USER_STORAGE_USAGE = 'userStorageUsage';
    const STATISTICS_USER_LOCAL_STORAGE_USAGE = 'userLocalStorageUsage';
    const STATISTICS_USER_SERVER_COUNT = 'userServerCount';
    const STATISTICS_USER_VM_COUNT = 'UserVmCount';
    const STATISTICS_USER_WORKSTATION_COUNT = 'UserWorkstationCount';
    const STATISTICS_USER_MOBILE_DEVICE_COUNT = 'UserMobileDeviceCount';
    const STATISTICS_USER_MAILBOX_COUNT = 'UserMailboxCount';
    const STATISTICS_USER_WEBSITE_COUNT = 'UserWebsiteCount';
    const STATISTICS_USAGE = 'usage';
    const STATISTICS_LIMIT = 'limit';
    const ACTION_ON_USER_CHANGE = 'onUserChange';
    const USER_STATUS_NOT_ACTIVATED = 3;
    const USER_STATUS_IN_PROGRESS = 2;
    const USER_STATUS_ACTIVE = 1;
    const USER_STATUS_INACTIVE = 0;
    const USER_STATUS_ERROR = -1;
    const CLIENTS = 'clients';
    const STATISTICS = 'statistics';

    const LOG_CALLED_CONFIGURE = 'Configure user (%s) in subscription (%s) was called.';
    const LOG_CALLED_PROVISION = 'User provision was called.';
    const LOG_CALLED_PROVISION_ASYNC = 'Asynchronous provision for user (%s) from subscription (%s) was called.';
    const LOG_CALLED_UNPROVISION = 'Unprovision service from customer subscription (%s) for user (%s) was called.';
    const LOG_CALLED_RETRIEVE = 'User retrieve was called.';
    const LOG_CALLED_DISABLE_USER = 'Disable user (%s) on subscription (%s) suspend was called.';
    const LOG_CALLED_ENABLE_USER = 'Enable user (%s) on subscription (%s) restore was called.';
    const LOG_CALLED_GET_CLIENT = 'Get client was called.';
    const LOG_CALLED_GET_MACHINE_USAGE = 'Get machine usage was called.';
    const LOG_CALLED_GET_WEB_RESTORE_DIGEST = 'Get Web restore digest was called.';
    const LOG_CALLED_GET_TENANT = 'Get tenant was called.';
    const LOG_CALLED_UPDATE_STATISTICS = 'Update statistics was called.';
    const LOG_CALLED_ON_USER_CHANGE = 'Update user (%s) on HS user change in subscription (%s) was called.';
    const LOG_CALLED_SUBSCRIBE_TO_ON_USER_CHANGE = 'Subscribe to onUserChange event was called.';
    const LOG_CALLED_ACTIVATE_ACCOUNT = 'Activate account was called.';
    const LOG_CALLED_CHANGE_PASSWORD = 'Change password was called.';
    const LOG_CALLED_GET_PARTNER_CREDENTIALS = 'Get partner credentials was called.';
    const LOG_CALLED_SHOW_PROFILE = 'Show profile was called.';
    const LOG_CALLED_UPGRADE = 'User (%s) upgrade was called';
    const LOG_CALLED_GET_AGENTS = 'Get agents was called.';
    const LOG_METHOD_CALL = 'Resource: %s Account: %s Subscription: %s Call: %s';

    const FIELD_PROFILE_NAME = 'name';
    const FIELD_PROFILE_BACKUP_NOTIFICATION = 'backupNotification';
    const FIELD_PROFILE_MANAGEMENT_NOTIFICATION = 'managementNotification';
    const FIELD_PROFILE_STORAGE_ID = 'storageId';
    const FIELD_PROFILE_USER_QUOTA = 'userQuota';
    const ADMINISTRATOR_PRIVILEGES_NO = 0;
    const ADMINISTRATOR_PRIVILEGES_YES = 1;

    const URL_REDIRECT_MASK = '%s?jwt=%s&to=backup';

    const ROLES_BUYER = 1;

    const STAFF_MEMBER_AVAILABLE = 'Staff member available %s';

    /* ======= RESOURCE PROPERTIES ======= */

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/tenant/9.0")
     * @required
     */
    public $tenant;

    /**
     * @link("http://aps-standard.org/types/core/service-user/1.0")
     * @required
     */
    public $user;

    /**
     * @link("http://aps-standard.org/types/core/admin-user/1.0[]");
     * @deprecated
     */
    public $adminusers;

    /**
     * true - sync password from service user account
     * false - not synchronize
     * @type(boolean)
     */
    public $activateDuringCreation = true;

    /**
     * @type(string)
     * @title("Login")
     * @Description("End user login")
     */
    public $endUserLogin;

    /**
     * @type(string)
     * @encrypted
     */
    public $endUserPassword;

    /**
     * @type(string)
     * @readonly
     */
    public $endUserEmail;

    /**
     * @type(string)
     * @readonly
     */
    public $displayName;

    /**
     * @type(string)
     * @readonly
     * @title("User Status")
     * @Description("User Status")
     */
    public $userstatus;

    /**
     * @type(string)
     * @title("Service Status")
     * @Description("Service Status")
     */
    public $servicestatus;

    /**
     * @type("Counter")
     */
    public $userStorageUsage;

    /**
     * @type("Counter")
     */
    public $userLocalStorageUsage;

    /**
     * @type("Counter")
     */
    public $userServerCount;

    /**
     * @type("Counter")
     */
    public $UserVmCount;

    /**
     * @type("Counter")
     */
    public $UserWorkstationCount;

    /**
     * @type("Counter")
     */
    public $UserMobileDeviceCount;

    /**
     * @type("Counter")
     */
    public $UserMailboxCount;

    /**
     * @type("Counter")
     */
    public $UserWebsiteCount;

    /**
     * @type(string)
     * @title("Service User ID")
     **/
    public $serviceUserId;

    /**
     * @type(string)
     * @title("Service subscription ID")
     **/
    public $subscriptionId;

    /**
     * @type(integer)
     */
    public $mspUserId;

    /**
     * @type(integer)
     */
    public $notifyBackup;

    /**
     * @type(boolean)
     */
    public $notificationsManagement;

    /**
     * @type(integer)
     */
    public $privileged;

    /**
     * @type(integer)
     */
    public $roles = null;

    /* ======= OTHER PROPERTIES ======= */

    /**
     * @var array
     */
    protected static $counterMap = array(
        self::STATISTICS_USER_STORAGE_USAGE,
        self::STATISTICS_USER_LOCAL_STORAGE_USAGE,
        self::STATISTICS_USER_WORKSTATION_COUNT,
        self::STATISTICS_USER_MOBILE_DEVICE_COUNT,
        self::STATISTICS_USER_SERVER_COUNT,
        self::STATISTICS_USER_VM_COUNT,
        self::STATISTICS_USER_MAILBOX_COUNT,
        self::STATISTICS_USER_WEBSITE_COUNT,
    );

    /** @var Model_UserManagerInterface */
    private $_userManager;

    /* ======= RESOURCE METHODS ======= */

    public function provision()
    {
        Log::getInstance()->event(self::LOG_CALLED_PROVISION);

        if (is_null($this->endUserLogin)) {
            $this->endUserLogin = $this->user->login;
        }
        if ($this->activateDuringCreation && is_null($this->endUserPassword)) {
            $this->endUserPassword = $this->user->password;
        }
        $this->endUserEmail = $this->getServiceUserEmail();
        $this->subscriptionId = $this->tenant->subscriptionId;
        $this->serviceUserId = $this->user->aps->id;
        $this->displayName = $this->user->displayName;
        $this->userstatus = self::USER_STATUS_IN_PROGRESS;
        $this->servicestatus = self::ACCOUNT_ENABLED;
        $this->initializeCounters();

        Log::getInstance()->event(sprintf(
            self::LOG_FORMAT_PROVISION_START,
            $this->subscriptionId,
            $this->endUserLogin
        ));

        $this->applyProfileUserQuotaMode();

        $this->throwRestAccepted();
    }

    public function provisionAsync()
    {
        Log::getInstance()->event(sprintf(
            self::LOG_CALLED_PROVISION_ASYNC,
            $this->endUserLogin,
            $this->subscriptionId
        ));

        if (!$this->tenant->ismigration) {
            $this->createMspUser();
        } else {
            $this->updateMspUser();
        }

        $this->_activateDuringCreation();
    }


    /**
     * @param users $new
     * @throws Exception
     */
    public function configure($new = NULL)
    {
        Log::getInstance()->event(sprintf(
            self::LOG_CALLED_CONFIGURE,
            $this->subscriptionId,
            $this->endUserLogin
        ));

        if (is_null($new)) {
            return;
        }
        if ($this->servicestatus != $new->servicestatus && $this->userstatus == self::USER_STATUS_NOT_ACTIVATED) {
            throw new \Rest\RestException(409, self::EXCEPTION_CHANGE_STATUS_NOT_ACTIVATED);
        }

        $userManager = $this->_getUserManager();
        $mspApi = $this->getMspApi();
        $mspUserInfo = $mspApi->getUserById($this->mspUserId);
        $version = $mspUserInfo->getVersion();
        $status = $mspUserInfo->getStatus();
        if ($status == Service_MspApiClient_Request_Struct_UpdateUserInfo::STATUS_DISABLE) {
            $version = $userManager->enableUser($this->mspUserId, $version);
        }

        if (!$this->_getProfileUserQuota()) {
            $new->userStorageUsage->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $new->userLocalStorageUsage->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $new->userServerCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $new->UserVmCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $new->UserWorkstationCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $new->UserMobileDeviceCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $new->UserMailboxCount->limit = $new->privileged == self::ADMINISTRATOR_PRIVILEGES_YES ?
                Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE : 0;
            $new->UserWebsiteCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
        }
        $version = $mspApi->updateUser(
            $this->mspUserId,
            $this->_getUpdateUserInfo($new),
            $version
        );

        if ($this->userstatus != self::USER_STATUS_NOT_ACTIVATED) {
            if ($this->user->disabled || $new->servicestatus == self::ACCOUNT_DISABLED) {
                $userManager->disableUser($this->mspUserId, $version);
                $this->userstatus = self::ACCOUNT_DISABLED;
            } else {
                $this->userstatus = self::ACCOUNT_ENABLED;
            }
        }
        $this->privileged = $new->privileged;
        $this->servicestatus = $new->servicestatus;
        $this->notifyBackup = $new->notifyBackup;
        $this->notificationsManagement = $new->notificationsManagement;

        $this->initializeCounters();

        foreach (self::$counterMap as $counter) {
            if (!isset($new->{$counter}, $new->{$counter}->limit)) {
                continue;
            }
            $this->{$counter}->limit = $new->{$counter}->limit;
        }
    }

    public function unprovision()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_UNPROVISION, $this->subscriptionId, $this->endUserLogin));
        if (is_null($this->mspUserId)) {
            return;
        }

        if ($this->tenant->ismigration) {
            return;
        }

        $userInfo = $this->getMspApi()->getUserById($this->mspUserId);
        if (is_null($userInfo)) {
            return;
        }
        $version = $userInfo->getVersion();

        if ($userInfo->getStatus() == 1) {
            $userManager = $this->_getUserManager();
            $version = $userManager->disableUser($this->mspUserId, $version);
        }
        try {
            $this->getMspApi()->deleteUser($this->mspUserId, $version);
        } catch (Exception $exception) {
            Log::getInstance()->error(sprintf(self::LOG_FORMAT_DELETE_USER_FAILED, $exception->getMessage(),
                $this->endUserLogin));
            throw new Exception(sprintf(self::EXCEPTION_ERROR_WHILE_USER_DELETING, $this->endUserLogin));
        }
    }

    public function retrieve()
    {
        Log::getInstance()->event(self::LOG_CALLED_RETRIEVE);
        $userInfo = $this->getMspApi()->getUserById($this->mspUserId);
        $this->initializeCounters();

        $this->userStorageUsage->usage = $userInfo->getUsage()->getStorageSize();
        $this->userStorageUsage->limit = $userInfo->getPrivileges()->getStorageSize();
        $this->userLocalStorageUsage->usage = $userInfo->getUsage()->getLocalStorageSize();
        $this->userLocalStorageUsage->limit = $userInfo->getPrivileges()->getLocalStorageSize();
        $this->userServerCount->usage = $userInfo->getUsage()->getServerCount();
        $this->userServerCount->limit = $userInfo->getPrivileges()->getServerCount();
        $this->UserVmCount->usage = $userInfo->getUsage()->getVmCount();
        $this->UserVmCount->limit = $userInfo->getPrivileges()->getVmCount();
        $this->UserWorkstationCount->usage = $userInfo->getUsage()->getWorkstationCount();
        $this->UserWorkstationCount->limit = $userInfo->getPrivileges()->getWorkstationCount();
        $this->UserMobileDeviceCount->usage = $userInfo->getUsage()->getMobileCount();
        $this->UserMobileDeviceCount->limit = $userInfo->getPrivileges()->getMobileCount();
        $this->UserMailboxCount->usage = $userInfo->getUsage()->getMailboxCount();
        $this->UserMailboxCount->limit = $userInfo->getPrivileges()->getMailboxCount();
        $this->UserWebsiteCount->usage = $userInfo->getUsage()->getWebsiteCount();
        $this->UserWebsiteCount->limit = $userInfo->getPrivileges()->getWebsiteCount();
    }

    /* ======= RESOURCE EVENTS ======= */

    /**
     * We define the operation on post event
     * @verb(POST)
     * @path("/onUsrChange")
     *
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onUserChange($event)
    {
        Log::getInstance()->event(sprintf(
            self::LOG_CALLED_ON_USER_CHANGE,
            $this->endUserLogin,
            $this->subscriptionId
        ));

        if ($this->userstatus != self::USER_STATUS_ACTIVE && $this->userstatus != self::USER_STATUS_INACTIVE) {
            return;
        }

        $userManager = $this->_getUserManager();
        $mspApi = $this->getMspApi();
        $mspUserInfo = $mspApi->getUserById($this->mspUserId);
        $version = $mspUserInfo->getVersion();
        $status = $mspUserInfo->getStatus();
        if ($status == Service_MspApiClient_Request_Struct_UpdateUserInfo::STATUS_DISABLE) {
            $version = $userManager->enableUser($this->mspUserId, $version);
        }

        if ($this->activateDuringCreation && !$this->isServiceUserAdministrator()) {
            $endUserPassword = $this->user->password;
            if (isset($endUserPassword) && ($this->endUserPassword !== $endUserPassword)) {
                $this->endUserPassword = $endUserPassword;
                $userManager->resetUserPassword($this->endUserLogin, $this->endUserPassword);
            }
        }

        $serviceUserEmail = $this->getServiceUserEmail();
        if ($this->endUserEmail != $serviceUserEmail) {
            $this->endUserEmail = $serviceUserEmail;
            $mspApi->mailUpdate($this->endUserLogin, $this->endUserEmail);
        }

        $updateUserInfo = new Service_MspApiClient_Request_Struct_UpdateUserInfo();
        $status = !$this->user->disabled && $this->servicestatus != self::ACCOUNT_DISABLED ?
            $updateUserInfo::STATUS_ENABLE :
            $updateUserInfo::STATUS_DISABLE;
        $updateUserInfo->setStatus($status);
        $updateUserInfo->setFirstName($this->user->givenName ? $this->user->givenName : " ");
        $updateUserInfo->setLastName($this->user->familyName ? $this->user->familyName : " ");
        $mspApi->updateUser($this->mspUserId, $updateUserInfo, $version);

        $this->userstatus = $status;
        $this->displayName = $this->user->displayName;
        $this->update();
    }


    /* ======= RESOURCE OPERATIONS ======= */

    /**
     * @verb(GET)
     * @path("/getTenant")
     */
    public function getTenant()
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_TENANT);
        return $this->tenant->toArray();
    }

    /**
     * @verb(GET)
     * @path("/getclient")
     * @param(object)
     */
    public function getclient($language)
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_CLIENT);

        $backupConsole = $this->getMspApi()->getBackupConsole($this->mspUserId, $language);
        $amsApi = $this->_facade->getAmsApi($backupConsole->getHost());
        $agents = $amsApi->getAgents($language);

        return array(
            // load agent links directly from BaaS backup console
            self::CLIENTS => array(
                self::WIN_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_WINDOWS]) ?
                    $agents[Service_AmsApiClient_Api::SYSTEM_WINDOWS]->getUrl() : NULL,

                self::LINUX_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_LINUX]) ?
                    $agents[Service_AmsApiClient_Api::SYSTEM_LINUX]->getUrl() : NULL,

                self::MAC_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_APPLE]) ?
                    $agents[Service_AmsApiClient_Api::SYSTEM_APPLE]->getUrl() : NULL,

                self::ANDROID_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_ANDROID]) ?
                    $agents[Service_AmsApiClient_Api::SYSTEM_ANDROID]->getUrl() : NULL,

                self::IOS_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_IOS]) ?
                    $agents[Service_AmsApiClient_Api::SYSTEM_IOS]->getUrl() : NULL,

                self::MICROSOFT_O365_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_OFFICE_365]) ?
                    $agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_OFFICE_365]->getUrl() : NULL,
            ),
            self::STATISTICS => $this->updateStatistics()
        );
    }

    /**
     * @verb(GET)
     * @path("/agents")
     * @param(string)
     * @return(object)
     */
    public function getAgents($language)
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_AGENTS);
        $mspApi = $this->getMspApi();
        $backupConsole = $mspApi->getBackupConsole($this->mspUserId, $language);
        $amsApi = $this->_facade->getAmsApi($backupConsole->getHost());
        $agents = $amsApi->getAgents($language);

        return array(
            self::WIN_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_WINDOWS]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_WINDOWS]->getUrl() : null,

            self::LINUX_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_LINUX]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_LINUX]->getUrl() : null,

            self::MAC_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_APPLE]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_APPLE]->getUrl() : null,

            self::ANDROID_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_ANDROID]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_ANDROID]->getUrl() : null,

            self::IOS_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_IOS]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_IOS]->getUrl() : null,

            self::VMWARE_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_VMWARE]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_VMWARE]->getUrl() : null,

            self::HYPERV_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_HYPERV]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_HYPERV]->getUrl() : null,

            self::VIRTUOZZO_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_VIRTUOZZO]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_VIRTUOZZO]->getUrl() : null,

            self::MICROSOFT_SQL_SERVER_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_SQL_SERVER]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_SQL_SERVER]->getUrl() : null,

            self::MICROSOFT_EXCHANGE_SERVER_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_EXCHANGE_SERVER]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_EXCHANGE_SERVER]->getUrl() : null,

            self::MICROSOFT_ACTIVE_DIRECTORY_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_ACTIVE_DIRECTORY]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_ACTIVE_DIRECTORY]->getUrl() : null,

            self::MICROSOFT_O365_URL => isset($agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_OFFICE_365]) ?
                $agents[Service_AmsApiClient_Api::SYSTEM_MICROSOFT_OFFICE_365]->getUrl() : null,
        );
    }

    /**
     * @verb(GET)
     * @path("/loginByToken")
     */
    public function loginByToken()
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_WEB_RESTORE_DIGEST);
        $mspApi = $this->getMspApi();
        $brand = $mspApi->getBrand($this->tenant->mspGroupId);
        $url = $brand->getAccountServerUrl();
        $token = $mspApi->getImpersonationUserToken($this->mspUserId);
        return sprintf(self::URL_REDIRECT_MASK, $url, $token);
    }

    /**
     * @verb(GET)
     * @path("/machineUsage")
     * @param()
     */
    public function getMachineUsage()
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_MACHINE_USAGE);
        $backupsItems = $this->_getUserManager()->getBackItems($this->mspUserId);

        $backupList = array();
        if (!is_array($backupsItems)) {
            return $backupList;
        }
        foreach ($backupsItems as $item) {
            /** @var Service_AmsApiClient_Response_Struct_BackupRecord $item */
            $backupList[] = array(
                self::BACKUP_LIST_NAME => $item->getTitle(),
                self::BACKUP_LIST_STATUS => $item->getStatus(),
                self::BACKUP_LIST_LAST_BACKUP => $this->formatBackupTime($item->getLastBackup()),
                self::BACKUP_LIST_NEXT_BACKUP => $this->formatBackupTime($item->getNextBackup()),
                self::BACKUP_LIST_TYPE => $item->getType()
            );
        }
        return $backupList;
    }

    /**
     * @verb(GET)
     * @path("/updateStatistics")
     * @param()
     */
    public function updateStatistics()
    {
        Log::getInstance()->event(self::LOG_CALLED_UPDATE_STATISTICS);
        $this->retrieve();
        $this->update();

        $userQuota = $this->_getProfileUserQuota();
        return array(
            self::STATISTICS_USER_STORAGE_USAGE => array(
                self::STATISTICS_USAGE => $this->userStorageUsage->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->StorageUsage->limit,
                    $this->userStorageUsage->limit
                )
            ),
            self::STATISTICS_USER_LOCAL_STORAGE_USAGE => array(
                self::STATISTICS_USAGE => $this->userLocalStorageUsage->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->LocalStorageUsage->limit,
                    $this->userLocalStorageUsage->limit
                )
            ),
            self::STATISTICS_USER_SERVER_COUNT => array(
                self::STATISTICS_USAGE => $this->userServerCount->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->ServerCount->limit,
                    $this->userServerCount->limit
                )
            ),
            self::STATISTICS_USER_VM_COUNT => array(
                self::STATISTICS_USAGE => $this->UserVmCount->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->VmCount->limit,
                    $this->UserVmCount->limit
                )
            ),
            self::STATISTICS_USER_WORKSTATION_COUNT => array(
                self::STATISTICS_USAGE => $this->UserWorkstationCount->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->WorkstationCount->limit,
                    $this->UserWorkstationCount->limit
                )
            ),
            self::STATISTICS_USER_MOBILE_DEVICE_COUNT => array(
                self::STATISTICS_USAGE => $this->UserMobileDeviceCount->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->MobileDeviceCount->limit,
                    $this->UserMobileDeviceCount->limit
                )
            ),
            self::STATISTICS_USER_MAILBOX_COUNT => array(
                self::STATISTICS_USAGE => $this->UserMailboxCount->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->MailboxCount->limit,
                    $this->UserMailboxCount->limit
                )
            ),
            self::STATISTICS_USER_WEBSITE_COUNT => array(
                self::STATISTICS_USAGE => $this->UserWebsiteCount->usage,
                self::STATISTICS_LIMIT => $this->_getCounterLimit(
                    $userQuota,
                    $this->tenant->WebsiteCount->limit,
                    $this->UserWebsiteCount->limit
                )
            ),
        );
    }

    /**
     * @verb(POST)
     * @path("/activateAccount")
     * @param(object,body)
     * @access(admin,true)
     * @access(owner,true)
     * @access(referrer,true)
     */
    public function activateAccount($configuration)
    {
        Log::getInstance()->event(self::LOG_CALLED_ACTIVATE_ACCOUNT);
        if ($this->userstatus != self::USER_STATUS_NOT_ACTIVATED) {
            throw new Exception(self::EXCEPTION_USER_ALREADY_ACTIVATED);
        }

        if (!isset($configuration->password)) {
            throw new Exception(self::EXCEPTION_PASSWORD_MUST_BE_PRESENT);
        }

        $this->endUserPassword = $configuration->password;

        $this->_activate();

        if (!$this->_getProfileUserQuota()) {
            $this->userStorageUsage->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->userLocalStorageUsage->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->userServerCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserVmCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserWorkstationCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserMobileDeviceCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserMailboxCount->limit = $this->privileged == self::ADMINISTRATOR_PRIVILEGES_YES ?
                Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE : 0;
            $this->UserWebsiteCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
        }
        $mspApi = $this->getMspApi();
        $version = $mspApi->getUserById($this->mspUserId)->getVersion();
        $mspApi->updateUser(
            $this->mspUserId,
            $this->_getUpdateUserInfo($this),
            $version
        );

        $this->update();
    }

    /**
     * @verb(POST)
     * @path("/changePassword")
     * @param(object,body)
     * @access(admin,true)
     * @access(owner,true)
     * @access(referrer,true)
     */
    public function changePassword($configuration)
    {
        Log::getInstance()->event(self::LOG_CALLED_CHANGE_PASSWORD);
        if (!isset($configuration->password)) {
            throw new Exception(self::EXCEPTION_PASSWORD_MUST_BE_PRESENT);
        }

        $this->endUserPassword = $configuration->password;
        $this->activateDuringCreation = false; // disable password synchronization

        $userManager = $this->_getUserManager();

        $userManager->resetUserPassword(
            $this->endUserLogin,
            $this->endUserPassword
        );

        $this->update();
    }

    /* ======= OTHER METHODS ======= */

    public function upgrade()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_UPGRADE, $this->aps->id));

        $this->aps->type = $this->getSelfApsType();
        $this->subscriptionId = $this->tenant->subscriptionId;
        $this->serviceUserId = $this->user->aps->id;
        $this->displayName = $this->user->displayName;
        $this->endUserEmail = $this->getServiceUserEmail();
        if (is_null($this->servicestatus)) {
            $this->servicestatus = $this->userstatus == self::USER_STATUS_INACTIVE ?
                self::ACCOUNT_DISABLED :
                self::ACCOUNT_ENABLED;
        }
        if ($this->userstatus == self::USER_STATUS_ACTIVE || $this->userstatus == self::USER_STATUS_INACTIVE) {
            $this->userstatus = $this->user->disabled || $this->servicestatus == self::ACCOUNT_DISABLED ?
                self::USER_STATUS_INACTIVE :
                self::USER_STATUS_ACTIVE;
        }
        if (is_null($this->activateDuringCreation)) {
            $this->activateDuringCreation = true;
        }

        $this->initializeCounters();

        // remove deprecated properties
        if (isset($this->previousStatus)) {
            unset($this->previousStatus);
        }
        if (isset($this->notificationsBackup)) {
            unset($this->notificationsBackup);
        }
        if (isset($this->UserWindowsServerEssentialsCount)) {
            unset($this->UserWindowsServerEssentialsCount); // todo: unset in abc
        }
        if (isset($this->UserVirtualHostsCount)) {
            unset($this->UserVirtualHostsCount);  // todo: unset in abc
        }

        // workaround for empty link to the datacenter
        if ($this->tenant->datacenter->isEmpty()) {
            $datacenterId = $this->tenant->datacenterId;
            if (empty($datacenterId)) {
                throw new Exception(self::EXCEPTION_TENANT_HAS_NOT_LINKED_WITH_DATACENTER_MESSAGE);
            }
            $datacenter = $this->_getDatacenterById($this->tenant->global, $datacenterId);
            $this->tenant->datacenter->set($datacenter);
        }

        $mspApi = $this->getMspApi();
        $userManager = $this->_getUserManager();

        $mspUserInfo = $mspApi->getUserById($this->mspUserId);
        $version = $mspUserInfo->getVersion();
        $status = $mspUserInfo->getStatus();

        // restore properties from ABC
        $this->endUserLogin = $mspUserInfo->getLogin();
        if (is_null($this->endUserPassword) && $status != Service_MspApiClient_Request_Struct_UpdateUserInfo::STATUS_INACTIVE) {
            $this->endUserPassword = $this->user->password;
        }
        if (is_null($this->privileged)) {
            $this->privileged = $mspUserInfo->getPrivileged();
        }
        if (is_null($this->notifyBackup)) {
            $this->notifyBackup = $mspUserInfo->getNotificationsBackup();
        }
        if (is_null($this->notificationsManagement)) {
            $this->notificationsManagement = $mspUserInfo->getNotificationsManagement();
        }

        // update properties in ABC
        if ($status == Service_MspApiClient_Request_Struct_UpdateUserInfo::STATUS_DISABLE) {
            $version = $userManager->enableUser($this->mspUserId, $version);
        }
        if ($this->endUserEmail != $mspUserInfo->getEmail()) {
            $mspApi->mailUpdate($this->endUserLogin, $this->endUserEmail);
        }
        $updateUserInfo = $this->_getUpdateUserInfo($this);

        $exception = null;
        try {
            $version = $mspApi->updateUser($this->mspUserId, $updateUserInfo, $version);
        } catch (Exception $e) {
            Log::getInstance()->error($e->getMessage());
            $exception = $e;
        }

        if ($this->userstatus == Service_MspApiClient_Request_Struct_UpdateUserInfo::STATUS_DISABLE) {
            $userManager->disableUser($this->mspUserId, $version);
        }

        if (!is_null($exception)) {
            throw $exception;
        }

        // update properties in APS
        $this->update();
    }

    /**
     * @return string
     */
    protected function getSelfApsType()
    {
        return self::CLOUD_BACKUP_USERS_TYPE;
    }

    private function _getCounterLimit($userQuota, $tenantCounterLimit, $userCounterLimit)
    {
        if ($userQuota || $tenantCounterLimit) {
            return $userCounterLimit;
        }
        return $tenantCounterLimit;
    }

    protected function subscribeToOnUserChangeEvent()
    {
        Log::getInstance()->event(self::LOG_CALLED_SUBSCRIBE_TO_ON_USER_CHANGE);
        $eventSubscription = new \APS\EventSubscription(\APS\EventSubscription::Changed, self::ACTION_ON_USER_CHANGE);
        $this->subscribeOnEventById($eventSubscription, $this->user->aps->id);
    }

    /**
     * @return PartnerCredentials
     */
    public function getPartnerCredentials()
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_PARTNER_CREDENTIALS);
        $datacenter = $this->tenant->datacenter;
        $credentials = new PartnerCredentials();
        $credentials->baasUrl = $datacenter->url;
        $credentials->partnerLogin = $datacenter->login;
        $credentials->partnerPassword = $datacenter->password;
        return $credentials;
    }

    /**
     * @return bool
     */
    private function _getProfileUserQuota()
    {
        $profile = $this->tenant->profile;
        if ($profile->isEmpty()) {
            $userQuota = true;
        } else {
            $userQuota = $profile->userQuota !== Service_APSApi_Api::PROFILE_USER_QUOTA_OFF;
        }
        return $userQuota;
    }

    private function _activate()
    {
        $userManager = $this->_getUserManager();
        $userInfo = $this->getMspApi()->getUserById($this->mspUserId);
        if ($userInfo->getStatus() == Service_MspApiClient_Response_Struct_UserInfo::STATUS_INACTIVE) {
            $userManager->activateUser(
                $this->endUserLogin,
                $this->endUserPassword
            );
        }

        $this->subscribeToOnUserChangeEvent();
        $this->userstatus = self::USER_STATUS_ACTIVE;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_UserInfo
     */
    private function _createUserInfo()
    {
        $userInfo = new Service_MspApiClient_Request_Struct_UserInfo();
        $userInfo->setFirstName($this->user->givenName ? $this->user->givenName : " ");
        $userInfo->setLastName($this->user->familyName ? $this->user->familyName : " ");
        $userInfo->setLogin($this->endUserLogin);
        $userInfo->setEmail($this->endUserEmail);
        $userInfo->setPrivileged($this->privileged);
        if (!is_null($this->roles)) {
            $userInfo->setRoles($this->roles);
        }
        return $this->_initUserInfoPrivileges($userInfo, $this);
    }

    protected function throwRestAccepted()
    {
        throw new \Rest\Accepted(
            $this,
            sprintf(
                self::LOG_FORMAT_CREATING_USER_TASK,
                $this->endUserLogin,
                $this->subscriptionId
            ),
            self::ASYNC_OPERATION_TIMEOUT
        );
    }

    /**
     * @return Model_UserManager
     */
    private function _getUserManager()
    {
        if (!isset($this->_userManager)) {
            $mspApi = $this->getMspApi();
            $this->_userManager = $this->_managerFactory->getUserManager($mspApi);
        }
        return $this->_userManager;
    }

    /**
     * @param string $time
     * @return int
     */
    private function formatBackupTime($time)
    {
        return $time ? strtotime($time) : NULL;
    }

    /**
     * @param $new
     * @return Service_MspApiClient_Request_Struct_UpdateUserInfo
     */
    private function _getUpdateUserInfo($new)
    {
        $userInfo = new Service_MspApiClient_Request_Struct_UpdateUserInfo();
        $userInfo->setPrivileged($new->privileged);
        $userInfo->setFirstName($new->user->givenName ? $new->user->givenName : " ");
        $userInfo->setLastName($new->user->familyName ? $new->user->familyName : " ");
        return $this->_initUserInfoPrivileges($userInfo, $new);
    }

    /**
     * @param Service_MspApiClient_Request_Struct_UserInfo $userInfo
     * @param users $usersObject
     * @return Service_MspApiClient_Request_Struct_UserInfo
     */
    private function _initUserInfoPrivileges(Service_MspApiClient_Request_Struct_UserInfoInterface $userInfo, $usersObject)
    {
        $userInfo->setNotificationsBackup(new Enum_NotificationsBackup((int)$usersObject->notifyBackup));
        $userInfo->setNotificationsManagement((int)$usersObject->notificationsManagement);
        if ($usersObject->UserWorkstationCount) {
            $userInfo->getPrivileges()->setWorkstationCount($usersObject->UserWorkstationCount->limit);
        }
        if ($usersObject->UserMobileDeviceCount) {
            $userInfo->getPrivileges()->setMobileCount($usersObject->UserMobileDeviceCount->limit);
        }
        if ($usersObject->UserMailboxCount) {
            $userInfo->getPrivileges()->setMailboxCount($usersObject->UserMailboxCount->limit);
        }
        if ($usersObject->userServerCount) {
            $userInfo->getPrivileges()->setServerCount($usersObject->userServerCount->limit);
        }
        if ($usersObject->UserVmCount) {
            $userInfo->getPrivileges()->setVmCount($usersObject->UserVmCount->limit);
        }
        if ($usersObject->userStorageUsage) {
            $userInfo->getPrivileges()->setStorageSize($usersObject->userStorageUsage->limit);
        }
        if ($usersObject->userLocalStorageUsage) {
            $userInfo->getPrivileges()->setLocalStorageSize($usersObject->userLocalStorageUsage->limit);
        }
        if ($usersObject->UserWebsiteCount) {
            $userInfo->getPrivileges()->setWebsiteCount($usersObject->UserWebsiteCount->limit);
        }
        return $userInfo;
    }

    private function isServiceUserAdministrator()
    {
        return !is_null($this->_getApsApi()->getStaffMember($this->tenant, $this->user->aps->id));
    }

    private function initializeCounters()
    {
        foreach (self::$counterMap as $counter) {
            if (is_null($this->{$counter})) {
                $this->{$counter} = new Counter();
            }
            if (is_null($this->{$counter}->limit)) {
                $this->{$counter}->limit = 0;
            }
            if (is_null($this->{$counter}->usage)) {
                $this->{$counter}->usage = 0;
            }
        }
    }

    private function createMspUser()
    {
        $userInfo = $this->_createUserInfo();
        $mspGroupId = $this->tenant->mspGroupId;
        try {
            $creationResult = $this->getMspApi()->createUser($userInfo, $mspGroupId);
        } catch (Service_BaseClient_HTTPException $e) {
            if ($e->getCode() != Service_BaseClient_RestTransport::HTTP_CODE_UNKNOWN_STATUS) {
                throw new Exception($e->getMessage());
            }

            $user = $this->findUserByLogin($mspGroupId, $userInfo->getLogin());
            if (empty($user)) {
                throw new Exception(self::EXCEPTION_USER_ALREADY_EXISTS);
            }

            $this->mspUserId = $user->getId();
            $this->updateMspUser();
            return;
        }

        if (!$creationResult) {
            $message = sprintf(
                self::LOG_FORMAT_ATTEMPT_TO_CREATE_EXISTING_MSP_USER,
                $this->endUserLogin,
                $this->subscriptionId
            );
            Log::getInstance()->event($message);
            throw new Exception($message);
        }

        $this->mspUserId = $creationResult->getId();
    }

    private function updateMspUser()
    {
        Log::getInstance()->event(self::LOG_MIGRATION_WAS_STARTED);

        $mspApi = $this->getMspApi();
        $userInfo = $mspApi->getUserById($this->mspUserId);
        $this->endUserLogin = $userInfo->getLogin();
        if ($this->endUserEmail != $userInfo->getEmail()) {
            $mspApi->mailUpdate($this->endUserLogin, $this->endUserEmail);
        }
        $updateUserInfo = $this->_getUpdateUserInfo($this);
        $this->getMspApi()->updateUser($this->mspUserId, $updateUserInfo, $userInfo->getVersion());
    }

    private function applyProfileUserQuotaMode()
    {
        if (!$this->_getProfileUserQuota()) {
            $this->userStorageUsage->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->userLocalStorageUsage->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->userServerCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserVmCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserWorkstationCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserMobileDeviceCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
            $this->UserMailboxCount->limit = $this->privileged == self::ADMINISTRATOR_PRIVILEGES_YES ?
                Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE : 0;
            $this->UserWebsiteCount->limit = Service_MspApiClient_Request_Struct_UserPrivileges::USER_UNLIMITED_VALUE;
        }
    }

    private function _activateDuringCreation()
    {
        if (!$this->activateDuringCreation) {
            $this->userstatus = self::USER_STATUS_NOT_ACTIVATED;
            return;
        }
        $this->_activate();
    }

    protected function getServiceUserEmail()
    {
        $user = $this->user;
        try {
            $email = $user->email;
            if (empty($email)) {
                $email = $user->login;
            }
            return $email;
        } catch (Exception $e) {
            // it is workaround for OSA 6.0
        }
        return $user->login;
    }

    /**
     * @param $mspGroupId
     * @param $endUserLogin
     * @return null|Service_MspApiClient_Response_Struct_UserItem
     */
    private function findUserByLogin($mspGroupId, $endUserLogin)
    {
        $users = $this->getMspApi()->getUsers($mspGroupId);
        foreach ($users as $user) {
            if ($user->getLogin() === $endUserLogin) {
                return $user;
            }
        }
        return null;
    }
}
